@@ -1,7 +1,8 @@
 (in-package :maxima)
 
 ;; wxMaxima xml format (based on David Drysdale MathML printing)
-;; Andrej Vodopivec, 2004-2014
+;; Andrej Vodopivec,  2004-2014
+;; Gunter Königsmann, 2014-2015
 
 ;; MathML-printing
 ;; Created by David Drysdale (DMD), December 2002/January 2003
@@ -15,7 +16,7 @@
 
 ;; Method:
 
-;; Producing wxxml from a maxima internal expression is done by
+;; Producing wxml from a Maxima internal expression is done by
 ;; a reversal of the parsing process.  Fundamentally, a
 ;; traversal of the expression tree is produced by the program,
 ;; with appropriate substitutions and recognition of the
@@ -29,20 +30,38 @@
 (defvar $wxfilename "")
 (defvar $wxdirname "")
 
+(defun $wxstatusbar (status)
+  (format t "<statusbar>~a</statusbar>~%" status))
+
+#+sbcl (setf SB-IMPL::*DEFAULT-EXTERNAL-FORMAT* :utf-8)
+
 (defun wx-cd (dir)
   (when $wxchangedir
     (let ((dir (cond ((pathnamep dir) dir)
                      ((stringp dir)
                       (make-pathname :directory (pathname-directory dir)
+				     :host (pathname-host dir)
                                      :device (pathname-device dir)))
                      (t (error "cd(dir): dir must be a string or pathname.")))))
-      (and (xchdir dir) (setf *default-pathname-defaults* dir) (namestring dir)))))
+       #+allegro (excl:chdir dir)
+       #+clisp (ext:cd dir)
+       #+cmu (setf (ext:default-directory) dir)
+       #+cormanlisp (ccl:set-current-directory dir)
+       #+gcl (si:chdir dir)
+       #+lispworks (hcl:change-directory dir)
+       #+lucid (lcl:working-directory dir)
+       #+sbcl (sb-posix:chdir dir)
+       #+sbcl (setf *default-pathname-defaults* (sb-ext:native-pathname (format nil "~A~A" (sb-posix:getcwd) "/")))
+       #+ccl (ccl:cwd dir)
+       #+ecl (si:chdir dir)
+
+       (namestring dir))))
 
 #+ccl (setf *print-circle* nil)
 
 
 ;;; Without this command encountering unicode characters might cause
-;;; maxima to stop responding on windows.
+;;; Maxima to stop responding on windows.
 #+(and clisp win32) (setf (stream-external-format *socket-connection*) charset:utf-8)
 #+(and clisp win32) (setf custom:*default-file-encoding* charset:utf-8)
 
@@ -56,7 +75,15 @@
   #+clisp `(let ((custom:*suppress-check-redefinition* t)) ,form)
   #-(or sbcl clisp) `(progn ,form))
 
-($put '$wxmaxima `((mlist simp) 16 4 2) '$version)
+(defun read-wxmaxima-version (v)
+  (let* ((d1 (position #\. v))
+         (year (subseq v 0 d1))
+         (d2 (position #\. v :start (1+ d1)))
+         (month (subseq v (1+ d1) d2))
+         (rest (subseq v (1+ d2))))
+    (list '(mlist simp) (parse-integer year) (parse-integer month) rest)))
+
+($put '$wxmaxima (read-wxmaxima-version "16.12.2") '$version)
 
 (defun $wxbuild_info ()
   (let ((wxmaxima-version (cdr ($get '$wxmaxima '$version)))
@@ -78,8 +105,9 @@
 
 (defmfun $wxbug_report ()
   (format t "~%The Maxima bug database is available at~%")
-  (format t "    http://sourceforge.net/tracker/?atid=104933&group_id=4933&func=browse~%")
-  (format t "Submit bug reports by following the 'Add new' link on that page.~%~%")
+  (format t "    http://sourceforge.net/p/maxima/bugs~%")
+  (format t "Submit bug reports by following the 'Create Ticket' link on that page.~%")
+  (format t "To report a Maxima bug, you must have a Sourceforge account.~%~%")
   (format t "The wxMaxima bug database is available at~%")
   (format t "    https://github.com/andrejv/wxmaxima/issues?direction=desc&sort=created&state=open~%")
   (format t "Submit bug reports by following the 'New issue' link on that page.~%~%")
@@ -265,7 +293,7 @@
   (concatenate 'string (car *var-tag*) pname (cadr *var-tag*)))
 
 (defun wxxml-paren (x l r)
-  (wxxml x (append l '("<p>")) (cons "</p>" r) 'mparen 'mparen))
+  (wxxml x (append l '("<r><p>")) (cons "</p></r>" r) 'mparen 'mparen))
 
 (defun wxxml-array (x l r &aux f)
   (if (eq 'mqapply (caar x))
@@ -394,22 +422,22 @@
 (defprop $false "<t>false</t>" wxxmlword)
 
 (defprop mprogn wxxml-matchfix wxxml)
-(defprop mprogn (("<p>") "</p>") wxxmlsym)
+(defprop mprogn (("<r><p>") "</p></r>") wxxmlsym)
 
 (defprop mlist wxxml-matchfix wxxml)
-(defprop mlist (("<t>[</t>")"<t>]</t>") wxxmlsym)
+(defprop mlist (("<r><t>[</t>")"<t>]</t></r>") wxxmlsym)
 
 (defprop $set wxxml-matchfix wxxml)
-(defprop $set (("<t>{</t>")"<t>}</t>") wxxmlsym)
+(defprop $set (("<r><t>{</t>")"<t>}</t></r>") wxxmlsym)
 
 (defprop mabs wxxml-matchfix wxxml)
-(defprop mabs (("<a>")"</a>") wxxmlsym)
+(defprop mabs (("<r><a>")"</a></r>") wxxmlsym)
 
 (defprop $conjugate wxxml-matchfix wxxml)
-(defprop $conjugate (("<cj>")"</cj>") wxxmlsym)
+(defprop $conjugate (("<r><cj>")"</cj></r>") wxxmlsym)
 
 (defprop %conjugate wxxml-matchfix wxxml)
-(defprop %conjugate (("<cj>")"</cj>") wxxmlsym)
+(defprop %conjugate (("<r><cj>")"</cj></r>") wxxmlsym)
 
 (defprop mbox wxxml-mbox wxxml)
 (defprop mlabox wxxml-mbox wxxml)
@@ -421,8 +449,8 @@
 (defprop mlabbox 10. wxxml-lbp)
 
 (defun wxxml-mbox (x l r)
-  (setq l (wxxml (cadr x) (append l '("<hl>")) nil 'mparen 'mparen)
-        r (append '("</hl>") r))
+  (setq l (wxxml (cadr x) (append l '("<r><hl>")) nil 'mparen 'mparen)
+        r (append '("</hl></r>") r))
   (append l r))
 
 (defprop mqapply wxxml-mqapply wxxml)
@@ -971,11 +999,11 @@
 (defun wxxml-d-abbrev-subscript (l_vars l_ords &aux var_xml)
   (let ((sub ()))
     (loop while l_vars do
-	  (setq var_xml (car (wxxml (car l_vars) nil nil 'mparen 'mparen)))
-	  (loop for i from 1 to (car l_ords) do
-		(setq sub (cons var_xml sub)))
-	  (setq l_vars (cdr l_vars)
-		l_ords (cdr l_ords)))
+         (setq var_xml (car (wxxml (car l_vars) nil nil 'mparen 'mparen)))
+         (loop for i from 1 to (car l_ords) do
+              (setq sub (cons var_xml sub)))
+         (setq l_vars (cdr l_vars)
+               l_ords (cdr l_ords)))
     (reverse sub)))
 
 (defun wxxml-d-abbrev (x)
@@ -1525,9 +1553,18 @@
 
 (defvar $display_graphics t)
 
+(defun option-sublist (lst)
+  (cons '(mlist simp)
+        (loop for l in lst
+           when (and (listp l) (= ($length l) 2))
+           collect l)))
+
 (defun $wxdraw (&rest args)
   (unless ($get '$draw '$version) ($load "draw"))
-  (let* ((filename (wxplot-filename nil))
+  (let* ((file_name_spec ($assoc '$file_name
+                                 (option-sublist (append (cdar args)
+                                                         (cdr args)))))
+         (filename (or file_name_spec (wxplot-filename nil)))
 	 (*windows-OS* t)
 	 res)
     (setq res ($apply '$draw
@@ -1546,7 +1583,9 @@
                           `(((mequal simp) $dimensions ,$wxplot_size)))))))
     (if $display_graphics
 	(progn
-          ($ldisp `((wxxmltag simp) ,(format nil "~a.png" filename) "img"))
+          ($ldisp `((wxxmltag simp) ,(format nil "~a.png" filename) "img"
+                    ,(when file_name_spec
+                           "del=\"no\"")))
           (setq res ""))
 	(setf res `((wxxmltag simp) ,(format nil "~a.png" filename) "img")))
     res))
@@ -1737,7 +1776,8 @@
 
 ;;;
 ;;; Rewrite of the function load (maxima/src/mload.lisp)
-;;; (displays functions after loading a maxima package
+;;; (displays functions and variable names after loading a Maxima package so the
+;;; autocomplete functionality knows which function and variable names exist
 ;;;
 (no-warning
  (defun $load (filename)
@@ -1774,7 +1814,9 @@
     ($error "table_form: the argument should not be an atom."))
   (setq mat ($args mat))
   (unless (every #'$listp (cdr mat))
-    ($error "table_form: data can not be displayed as a table."))
+    (setq mat (cons '(mlist simp)
+                    (mapcar (lambda (e) (list '(mlist simp) e))
+                            (cdr mat)))))
   (setq opts (cons '(mlist simp) opts))
   (let ((row-names ($assoc '$row_names opts))
 	(col-names ($assoc '$column_names opts))
@@ -1812,6 +1854,8 @@
     ($ldisp (cons (append '($matrix simp) mtrx) (cdr mat)))
     '$done))
 
+(putprop '$table_form t 'evfun)
+
 ;; Load the initial functions (from mac-init.mac)
 (let ((*print-circle* nil))
   (format t "<wxxml-symbols>~{~a~^$~}</wxxml-symbols>"
