@@ -1,7 +1,7 @@
 // -*- mode: c++; c-file-style: "linux"; c-basic-offset: 2; indent-tabs-mode: nil -*-
 //
 //  Copyright (C) 2004-2015 Andrej Vodopivec <andrej.vodopivec@gmail.com>
-//  Copyright (C) 2015      Gunter Königsmann <wxMaxima@physikbuch.de>
+//  Copyright (C) 2015-2016 Gunter Königsmann <wxMaxima@physikbuch.de>
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
@@ -19,20 +19,28 @@
 //  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 //
 
+/*! \file
+  This file defines the class BitMap that renders math as bitmap.
+ */
+
 #include "Bitmap.h"
-#include "CellParser.h"
+#include "Configuration.h"
 #include "GroupCell.h"
 
 #include <wx/config.h>
 #include <wx/clipbrd.h>
 
 #define BM_FULL_WIDTH 1000
-
 Bitmap::Bitmap(int scale)
 {
   m_tree = NULL;
   m_scale=scale;
-  m_bmp.Create(10,10);
+  // The depth 24 hinders wxWidgets from creating rgb0 bitmaps that some
+  // windows applications will interpret as rgba if they appear on
+  // the clipboards and therefore render them all-transparent.
+  m_bmp.CreateScaled(10,10,24,1.0/((double)scale));
+  m_width = 10;
+  m_height = 10;
 }
 
 Bitmap::~Bitmap()
@@ -41,47 +49,64 @@ Bitmap::~Bitmap()
     DestroyTree();
 }
 
-void Bitmap::SetData(MathCell* tree)
+bool Bitmap::SetData(MathCell* tree,long int maxSize)
 {
   if (m_tree != NULL)
     delete m_tree;
   m_tree = tree;
-  Layout();
+  return Layout(maxSize);
 }
 
-void Bitmap::Layout()
+bool Bitmap::Layout(long int maxSize)
 {
   if (m_tree->GetType() != MC_TYPE_GROUP)
   {
     RecalculateWidths();
     BreakUpCells();
     BreakLines();
-    RecalculateSize();
+    RecalculateHeight();
   }
   else {
-    int fontsize = 12;
-    wxConfig::Get()->Read(wxT("fontSize"), &fontsize);
-    int mfontsize = fontsize;
-    wxConfig::Get()->Read(wxT("mathfontsize"), &mfontsize);
     GroupCell* tmp = (GroupCell *)m_tree;
 
     wxMemoryDC dc;
     dc.SelectObject(m_bmp);
     dc.SetUserScale(m_scale,m_scale);
-    CellParser parser(dc);
-    parser.SetClientWidth(BM_FULL_WIDTH);
+    Configuration configuration(dc);
+    configuration.SetClientWidth(BM_FULL_WIDTH);
 
     while (tmp != NULL)
     {
-      tmp->Recalculate(parser, fontsize, mfontsize);
+      tmp->Recalculate();
       tmp = (GroupCell *)tmp->m_next;
     }
   }
 
   int width, height;
   GetMaxPoint(&width, &height);
-  m_bmp.Create(m_width=width * m_scale, m_height=height * m_scale);
-  Draw();
+
+  // Too big bitmaps or bitmaps that are too wide or high can crash windows
+  // or the X server.
+  if((maxSize < 0) ||
+     (
+       (width*height*m_scale*m_scale < maxSize) &&
+       (width*m_scale < 20000) &&
+       (height*m_scale < 20000)
+       )
+    )
+  {
+    // The depth 24 hinders wxWidgets from creating rgb0 bitmaps that some
+    // windows applications will interpret as rgba if they appear on
+    // the clipboards and therefore render them all-transparent.
+    m_bmp.Create(m_width=width * m_scale, m_height=height * m_scale,24);
+    Draw();
+    return true;
+  }
+  else
+  {
+    m_bmp = wxNullBitmap;
+    return false;
+  }
 }
 
 double Bitmap::GetRealWidth()
@@ -94,7 +119,7 @@ double Bitmap::GetRealHeight()
   return m_height/m_scale;
 }
 
-void Bitmap::RecalculateSize()
+void Bitmap::RecalculateHeight()
 {
   int fontsize = 12;
   wxConfig::Get()->Read(wxT("fontSize"), &fontsize);
@@ -105,11 +130,11 @@ void Bitmap::RecalculateSize()
   wxMemoryDC dc;
   dc.SelectObject(m_bmp);
   dc.SetUserScale(m_scale,m_scale);
-  CellParser parser(dc);
+  Configuration configuration(dc);
 
   while (tmp != NULL)
   {
-    tmp->RecalculateSize(parser, tmp->IsMath() ? mfontsize : fontsize);
+    tmp->RecalculateHeight(tmp->IsMath() ? mfontsize : fontsize);
     tmp = tmp->m_next;
   }
 }
@@ -126,12 +151,12 @@ void Bitmap::RecalculateWidths()
   wxMemoryDC dc;
   dc.SelectObject(m_bmp);
   dc.SetUserScale(m_scale,m_scale);
-  CellParser parser(dc);
-  parser.SetClientWidth(BM_FULL_WIDTH);
+  Configuration configuration(dc);
+  configuration.SetClientWidth(BM_FULL_WIDTH);
 
   while (tmp != NULL)
   {
-    tmp->RecalculateWidths(parser,  tmp->IsMath() ? mfontsize : fontsize);
+    tmp->RecalculateWidths(tmp->IsMath() ? mfontsize : fontsize);
     tmp = tmp->m_next;
   }
 }
@@ -198,6 +223,7 @@ void Bitmap::GetMaxPoint(int* width, int* height)
 
 void Bitmap::Draw()
 {
+  MathCell::ClipToDrawRegion(false);
   MathCell* tmp = m_tree;
   wxMemoryDC dc;
   dc.SelectObject(m_bmp);
@@ -220,13 +246,13 @@ void Bitmap::Draw()
     int mfontsize = fontsize;
     wxConfig::Get()->Read(wxT("mathfontsize"), &mfontsize);
 
-    CellParser parser(dc);
+    Configuration configuration(dc);
 
     while (tmp != NULL)
     {
       if (!tmp->m_isBroken)
       {
-        tmp->Draw(parser, point, tmp->IsMath() ? mfontsize : fontsize);
+        tmp->Draw(point, tmp->IsMath() ? mfontsize : fontsize);
         if ((tmp->m_next != NULL) && (tmp->m_next->BreakLineHere()))
         {
           point.x = 0;
@@ -257,22 +283,30 @@ void Bitmap::Draw()
   m_ppi = dc.GetPPI();
   m_ppi.x *= m_scale;
   m_ppi.y *= m_scale;
+  MathCell::ClipToDrawRegion(true);
 }
 
 wxSize Bitmap::ToFile(wxString file)
 {
+  // Assign an resolution to the bitmap.
+  wxImage img=m_bmp.ConvertToImage();
+  int resolution = img.GetOptionInt(wxIMAGE_OPTION_RESOLUTION);
+  if(resolution <= 0)
+    resolution = 75;
+  img.SetOption(wxIMAGE_OPTION_RESOLUTION,resolution * m_scale);
+
   bool success = false;
   if (file.Right(4) == wxT(".bmp"))
-    success = m_bmp.SaveFile(file, wxBITMAP_TYPE_BMP);
+    success = img.SaveFile(file, wxBITMAP_TYPE_BMP);
   else if (file.Right(4) == wxT(".xpm"))
-    success = m_bmp.SaveFile(file, wxBITMAP_TYPE_XPM);
+    success = img.SaveFile(file, wxBITMAP_TYPE_XPM);
   else if (file.Right(4) == wxT(".jpg"))
-    success = m_bmp.SaveFile(file, wxBITMAP_TYPE_JPEG);
+    success = img.SaveFile(file, wxBITMAP_TYPE_JPEG);
   else
   {
     if (file.Right(4) != wxT(".png"))
       file = file + wxT(".png");
-    success = m_bmp.SaveFile(file, wxBITMAP_TYPE_PNG);
+    success = img.SaveFile(file, wxBITMAP_TYPE_PNG);
   }
 
   wxSize retval;
@@ -327,7 +361,7 @@ void Bitmap::BreakUpCells()
   wxMemoryDC dc;
   dc.SelectObject(m_bmp);
   dc.SetUserScale(m_scale,m_scale);
-  CellParser parser(dc);
+  Configuration configuration(dc);
 
   while (tmp != NULL)
   {
@@ -335,8 +369,8 @@ void Bitmap::BreakUpCells()
     {
       if (tmp->BreakUp())
       {
-        tmp->RecalculateWidths(parser, tmp->IsMath() ? mfontsize : fontsize);
-        tmp->RecalculateSize(parser, tmp->IsMath() ? mfontsize : fontsize);
+        tmp->RecalculateWidths(tmp->IsMath() ? mfontsize : fontsize);
+        tmp->RecalculateHeight(tmp->IsMath() ? mfontsize : fontsize);
       }
     }
     tmp = tmp->m_nextToDraw;
