@@ -22,11 +22,12 @@
 
 /*\file
 
-  The C code for the preferences dialog.
+  The C code for ConfigDialogue, the preferences dialog.
 */
 
 #include "ConfigDialogue.h"
 #include "MathCell.h"
+#include "Configuration.h"
 
 #include <wx/config.h>
 #include <wx/fileconf.h>
@@ -63,6 +64,9 @@ const int langs[] =
   wxLANGUAGE_HUNGARIAN,
   wxLANGUAGE_ITALIAN,
   wxLANGUAGE_JAPANESE,
+#if wxCHECK_VERSION(3,0,1)
+  wxLANGUAGE_KABYLE,
+#endif
   wxLANGUAGE_NORWEGIAN_BOKMAL,
   wxLANGUAGE_POLISH,
   wxLANGUAGE_PORTUGUESE_BRAZILIAN,
@@ -72,7 +76,7 @@ const int langs[] =
   wxLANGUAGE_UKRAINIAN
 };
 
-#define LANGUAGE_NUMBER sizeof(langs)/(signed)sizeof(langs[1])
+#define LANGUAGE_NUMBER (long)(sizeof(langs)/(signed)sizeof(langs[1]))
 
 int ConfigDialogue::GetImageWidth()
 {
@@ -166,9 +170,10 @@ void ConfigDialogue::SetProperties()
 {
   SetTitle(_("wxMaxima configuration"));
 
-  m_showUserDefinedLabels->SetToolTip(_("If a command begins with a label followed by a : wxMaxima will show this label instead of the \%o style label maxima has automatically assigned to the same output cell."));
+  m_showUserDefinedLabels->SetToolTip(_("If a command begins with a label followed by a : wxMaxima will show this label instead of the %o style label maxima has automatically assigned to the same output cell."));
   m_abortOnError->SetToolTip(_("If multiple cells are evaluated in one go: Abort evaluation if wxMaxima detects that maxima has encountered any error."));
   m_pollStdOut->SetToolTip(_("Once the local network link between maxima and wxMaxima has been established maxima has no reason to send any messages using the system's stdout stream so all this stream transport should be a greeting message; The lisp running maxima will send eventual error messages using the system's stderr stream instead. If this box is checked we will nonetheless watch maxima's stdout stream for messages."));
+  m_restartOnReEvaluation->SetToolTip(_("Maxima provides no \"forget all\" command that flushes all settings a maxima session could make. wxMaxima therefore normally defaults to starting a fresh maxima process every time the worksheet is to be re-evaluated. As this needs a little bit of time this switch allows to disable this behavior."));
   m_maximaProgram->SetToolTip(_("Enter the path to the Maxima executable."));
   m_additionalParameters->SetToolTip(_("Additional parameters for Maxima"
                                        " (e.g. -l clisp)."));
@@ -184,14 +189,12 @@ void ConfigDialogue::SetProperties()
   m_TeXExponentsAfterSubscript->SetToolTip(_("In the LaTeX output: Put exponents after an eventual subscript instead of above it. Might increase readability for some fonts and short subscripts."));
   m_usePartialForDiff->SetToolTip(_("Use the \"partial derivative\" symbol to represent the fraction that represents a diff() when exporting LaTeX"));
   m_wrapLatexMath->SetToolTip(_("Wrap equations exported by the \"copy as LaTeX\" feature between \\[ and \\] as equation markers"));
-  m_flowedTextRequested->SetToolTip(_("While text cells in LaTeX are broken into lines by TeX the text displayed on the screen is broken into lines manually. This option, if set tells that lines in HTML output will be broken where they are broken in the worksheet. If this option isn't set manual linebreaks can still be introduced by introducing an empty line."));
   m_bitmapScale->SetToolTip(_("Normally html expects images to be rather low-res but space saving. These images tend to look rather blurry when viewed on modern screens. Therefore this setting was introduces that selects the factor by which the HTML export increases the resolution in respect to the default value."));
-  m_exportInput->SetToolTip(_("Normally we export the whole worksheet to TeX or HTML. But sometimes the maxima input does scare the user. This option turns off exporting of maxima's input."));
   m_exportContainsWXMX->SetToolTip(_("If this option is set the .wxmx source of the current file is copied to a place a link to is put into the result of an export."));
-  m_exportWithMathJAX->SetToolTip(_("Use MathJAX instead of images in HTML exports to display maxima output. The advantage of MathJAX is that it allows one to copy the displayed equations as if they were text, to choose if they should be copied as TeX or MathML instead and displays them in a scaleable format that is really nice to look at. The disadvantage of MathJAX is that it will need JavaScript and a little bit of time in order to typeset an equation."));
+  m_exportWithMathJAX->SetToolTip(_("MathJAX creates scalable High-Quality representations of 2D Maths that can be used for Drag-And-Drop and provides accessability options. The disadvantage of MathJAX is that it needs JavaScript and a little bit of time in order to typeset equations.\nMathML is much faster than MathJaX, if it is supported by the browser. But many MathML implementations tend to lack necessary features.\nBitmaps tend to need more band width than the other two options. They lack support for advanced features like drag-and-drop or accessibility. Also they have problems aligning and scaling with the rest of the text and might use fonts that don't match the rest of the document."));
   m_savePanes->SetToolTip(_("Save panes layout between sessions."));
   m_usepngCairo->SetToolTip(_("The pngCairo terminal offers much better graphics quality (antialiassing and additional line styles). But it will only produce plots if the gnuplot installed on the current system actually supports it."));
-  m_matchParens->SetToolTip(_("Write matching parenthesis in text controls."));
+  m_matchParens->SetToolTip(_("Automatically insert matching parenthesis in text controls. Automatic highlighting of matching parenthesis can be suppressed by setting the respective color to match the background of ordinary text."));
   m_showLength->SetToolTip(_("Show long expressions in wxMaxima document."));
   m_autosubscript->SetToolTip(_("false=Don't generate subscripts\ntrue=Automatically convert underscores to subscript markers if the would-be subscript is a number or a single letter\nall=_ marks subscripts."));
   m_language->SetToolTip(_("Language used for wxMaxima GUI."));
@@ -205,6 +208,8 @@ void ConfigDialogue::SetProperties()
   m_changeAsterisk->SetToolTip(_("Use centered dot character for multiplication"));
   m_defaultPort->SetToolTip(_("The default port used for communication between Maxima and wxMaxima."));
   m_undoLimit->SetToolTip(_("Save only this number of actions in the undo buffer. 0 means: save an infinite number of actions."));
+  m_recentItems->SetToolTip(_("The number of recently opened files that is to be remembered."));
+  m_incrementalSearch->SetToolTip(_("Start searching while the phrase to search for is still being typed."));
 
   #ifdef __WXMSW__
   m_wxcd->SetToolTip(_("Automatically change maxima's working directory to the one the current document is in: "
@@ -217,23 +222,27 @@ void ConfigDialogue::SetProperties()
 
   // The default values for all config items that will be used if there is no saved
   // configuration data for this item.
-  bool match = true, savePanes = true, UncompressedWXMX=true;
+  bool match, savePanes = true, UncompressedWXMX=true;
   bool fixedFontTC = true, changeAsterisk = false, usejsmath = true, keepPercent = true, abortOnError = true, pollStdOut = false;
   bool enterEvaluates = false, saveUntitled = true,
-    openHCaret = false, AnimateLaTeX = true, TeXExponentsAfterSubscript=false,
+    AnimateLaTeX = true, TeXExponentsAfterSubscript=false,
     usePartialForDiff = false,
     wrapLatexMath = true,
-    flowedTextRequested = true, exportInput = true, exportContainsWXMX = false,
-    exportWithMathJAX = true;
-  bool insertAns = true;
+    exportContainsWXMX = false;
+  int exportWithMathJAX = 0;
+  bool insertAns;
   bool autoIndent = true;
+  bool cursorJump = true;
+  bool autoWrap   = true;
 
   int labelWidth = 4;
-  int  undoLimit = 0;
+  int undoLimit  = 0;
+  int recentItems= 10;
   int showLength = 0;
   int autosubscript = 1;
   int  bitmapScale = 3;
   bool fixReorderedIndices = false;
+  bool incrementalSearch = true;
   bool showUserDefinedLabels = true;
   int defaultFramerate = 2;
   int displayedDigits = 100;
@@ -272,12 +281,10 @@ void ConfigDialogue::SetProperties()
   config->Read(wxT("TeXExponentsAfterSubscript"), &TeXExponentsAfterSubscript);
   config->Read(wxT("usePartialForDiff"), &usePartialForDiff);
   config->Read(wxT("wrapLatexMath"), &wrapLatexMath);
-  config->Read(wxT("flowedTextRequested"), &flowedTextRequested);
-  config->Read(wxT("exportInput"), &exportInput);
   config->Read(wxT("exportContainsWXMX"), &exportContainsWXMX);
-  config->Read(wxT("exportWithMathJAX"), &exportWithMathJAX);
+  config->Read(wxT("HTMLequationFormat"), &exportWithMathJAX);
   config->Read(wxT("pos-restore"), &rs);
-  config->Read(wxT("matchParens"), &match);
+  match = Configuration::Get()->GetMatchParens();
   config->Read(wxT("showLength"), &showLength);
   config->Read(wxT("autosubscript"), &autosubscript);
   config->Read(wxT("language"), &lang);
@@ -289,13 +296,16 @@ void ConfigDialogue::SetProperties()
   config->Read(wxT("panelSize"), &panelSize);
   config->Read(wxT("enterEvaluates"), &enterEvaluates);
   config->Read(wxT("saveUntitled"), &saveUntitled);
-  config->Read(wxT("openHCaret"), &openHCaret);
-  config->Read(wxT("insertAns"), &insertAns);
+  insertAns = Configuration::Get()->GetInsertAns();
   config->Read(wxT("autoIndent"), &autoIndent);
+  config->Read(wxT("cursorJump"), &cursorJump);
+  config->Read(wxT("autoWrap"), &autoWrap);
   config->Read(wxT("labelWidth"), &labelWidth);
   config->Read(wxT("undoLimit"), &undoLimit);
+  config->Read(wxT("recentItems"), &recentItems);
   config->Read(wxT("bitmapScale"), &bitmapScale);
   config->Read(wxT("fixReorderedIndices"), &fixReorderedIndices);
+  config->Read(wxT("incrementalSearch"), &incrementalSearch);
   config->Read(wxT("showUserDefinedLabels"), &showUserDefinedLabels);
   config->Read(wxT("usejsmath"), &usejsmath);
   config->Read(wxT("keepPercent"), &keepPercent);
@@ -347,29 +357,32 @@ void ConfigDialogue::SetProperties()
   m_TeXExponentsAfterSubscript->SetValue(TeXExponentsAfterSubscript);
   m_usePartialForDiff->SetValue(usePartialForDiff);
   m_wrapLatexMath->SetValue(wrapLatexMath);
-  m_flowedTextRequested->SetValue(flowedTextRequested);
-  m_exportInput->SetValue(exportInput);
   m_exportContainsWXMX->SetValue(exportContainsWXMX);
-  m_exportWithMathJAX->SetValue(exportWithMathJAX);
+  m_exportWithMathJAX->SetSelection(exportWithMathJAX);
   m_matchParens->SetValue(match);
   m_showLength->SetSelection(showLength);
   m_autosubscript->SetSelection(autosubscript);
   m_changeAsterisk->SetValue(changeAsterisk);
   m_enterEvaluates->SetValue(enterEvaluates);
   m_saveUntitled->SetValue(saveUntitled);
-  m_openHCaret->SetValue(openHCaret);
   m_insertAns->SetValue(insertAns);
   m_autoIndent->SetValue(autoIndent);
+  m_openHCaret->SetValue(Configuration::Get()->GetOpenHCaret());
+  m_cursorJump->SetValue(cursorJump);
+  m_autoWrap->SetValue(autoWrap);
   m_labelWidth->SetValue(labelWidth);
   m_undoLimit->SetValue(undoLimit);
+  m_recentItems->SetValue(recentItems);
   m_bitmapScale->SetValue(bitmapScale);
   m_fixReorderedIndices->SetValue(fixReorderedIndices);
+  m_incrementalSearch->SetValue(incrementalSearch);
   m_showUserDefinedLabels->SetValue(showUserDefinedLabels);
   m_fixedFontInTC->SetValue(fixedFontTC);
   m_useJSMath->SetValue(usejsmath);
   m_keepPercentWithSpecials->SetValue(keepPercent);
   m_abortOnError->SetValue(abortOnError);
   m_pollStdOut->SetValue(pollStdOut);
+  m_restartOnReEvaluation->SetValue(Configuration::Get()->RestartOnReEvaluation());
   m_defaultFramerate->SetValue(defaultFramerate);
   m_defaultPlotWidth->SetValue(defaultPlotWidth);
   m_defaultPlotHeight->SetValue(defaultPlotHeight);
@@ -396,7 +409,7 @@ wxPanel* ConfigDialogue::CreateWorksheetPanel()
   wxArrayString showLengths;
   wxArrayString autosubscripts;
   wxFlexGridSizer* grid_sizer = new wxFlexGridSizer(8, 2, 5, 5);
-  wxFlexGridSizer* vsizer = new wxFlexGridSizer(17,1,5,5);
+  wxFlexGridSizer* vsizer = new wxFlexGridSizer(18,1,5,5);
   
   wxStaticText* pw = new wxStaticText(panel, -1, _("Default plot size for new maxima sessions:"));
   wxBoxSizer *PlotWidthHbox=new wxBoxSizer(wxHORIZONTAL);
@@ -442,7 +455,7 @@ wxPanel* ConfigDialogue::CreateWorksheetPanel()
 
   vsizer->Add(grid_sizer, 1, wxEXPAND, 5);
 
-  m_showUserDefinedLabels = new wxCheckBox(panel, -1, _("Show user-defined labels instead of (\%oxx)"));
+  m_showUserDefinedLabels = new wxCheckBox(panel, -1, _("Show user-defined labels instead of (%oxx)"));
   vsizer->Add(m_showUserDefinedLabels, 0, wxALL, 5);
 
   m_matchParens = new wxCheckBox(panel, -1, _("Match parenthesis in text controls"));
@@ -454,7 +467,7 @@ wxPanel* ConfigDialogue::CreateWorksheetPanel()
   m_changeAsterisk = new wxCheckBox(panel, -1, _("Use centered dot character for multiplication"));
   vsizer->Add(m_changeAsterisk, 0, wxALL, 5);
 
-  m_keepPercentWithSpecials = new wxCheckBox(panel, -1, _("Keep percent sign with special symbols: \%e, \%i, etc."));
+  m_keepPercentWithSpecials = new wxCheckBox(panel, -1, _("Keep percent sign with special symbols: %e, %i, etc."));
   vsizer->Add(m_keepPercentWithSpecials, 0, wxALL, 5);
 
   m_enterEvaluates = new wxCheckBox(panel, -1, _("Enter evaluates cells"));
@@ -463,12 +476,18 @@ wxPanel* ConfigDialogue::CreateWorksheetPanel()
   m_openHCaret = new wxCheckBox(panel, -1, _("Open a cell when Maxima expects input"));
   vsizer->Add(m_openHCaret, 0, wxALL, 5);
 
-  m_insertAns = new wxCheckBox(panel, -1, _("Insert \% before an operator at the beginning of a cell"));
+  m_insertAns = new wxCheckBox(panel, -1, _("Insert % before an operator at the beginning of a cell"));
   vsizer->Add(m_insertAns, 0, wxALL, 5);
 
   m_autoIndent = new wxCheckBox(panel, -1, _("Auto-indent new lines"));
   vsizer->Add(m_autoIndent, 0, wxALL, 5);
 
+  m_cursorJump = new wxCheckBox(panel, -1, _("New lines: Jump to text"));
+  vsizer->Add(m_cursorJump, 0, wxALL, 5);
+
+  m_autoWrap = new wxCheckBox(panel, -1, _("Automatically wrap long text lines"));
+  vsizer->Add(m_autoWrap, 0, wxALL, 5);
+
   vsizer->AddGrowableRow(10);
   panel->SetSizer(vsizer);
   vsizer->Fit(panel);
@@ -480,8 +499,8 @@ wxPanel* ConfigDialogue::CreateExportPanel()
 {
   wxPanel *panel = new wxPanel(m_notebook, -1);
 
-  wxFlexGridSizer* grid_sizer = new wxFlexGridSizer(4, 2, 5, 5);
-  wxFlexGridSizer* vsizer = new wxFlexGridSizer(18,1,5,5);
+  wxFlexGridSizer* grid_sizer = new wxFlexGridSizer(5, 2, 5, 5);
+  wxFlexGridSizer* vsizer = new wxFlexGridSizer(17,1,5,5);
 
   wxStaticText *dc = new wxStaticText(panel, -1, _("Documentclass for TeX export:"));
   m_documentclass = new wxTextCtrl(panel, -1, wxEmptyString, wxDefaultPosition, wxSize(250, wxDefaultSize.GetY()));
@@ -499,6 +518,15 @@ wxPanel* ConfigDialogue::CreateExportPanel()
   grid_sizer->Add(bs, 0, wxALL | wxALIGN_CENTER_VERTICAL, 5);
   grid_sizer->Add(m_bitmapScale, 0, wxALL, 5);  
 
+  wxStaticText* mj = new wxStaticText(panel, -1, _("Export equations to HTML as:"));
+  wxArrayString mathJaxChoices;
+  mathJaxChoices.Add(_("TeX, interpreted by MathJaX"));
+  mathJaxChoices.Add(_("Bitmaps"));
+  mathJaxChoices.Add(_("MathML + MathJaX as Fill-In"));
+  m_exportWithMathJAX = new wxChoice(panel,-1,wxDefaultPosition,wxDefaultSize,mathJaxChoices);
+  grid_sizer->Add(mj, 0, wxALL | wxALIGN_CENTER_VERTICAL, 5);
+  grid_sizer->Add(m_exportWithMathJAX, 0, wxALL, 5);  
+  
   m_AnimateLaTeX = new wxCheckBox(panel, -1, _("Export animations to TeX (Images only move if the PDF viewer supports this)"));
   vsizer->Add(m_AnimateLaTeX, 0, wxALL, 5);
 
@@ -511,18 +539,9 @@ wxPanel* ConfigDialogue::CreateExportPanel()
   m_wrapLatexMath = new wxCheckBox(panel, -1, _("\"Copy LaTeX\" adds equation markers"));
   vsizer->Add(m_wrapLatexMath, 0, wxALL, 5);
 
-  m_flowedTextRequested = new wxCheckBox(panel, -1, _("HTML/Text Cells: Export all linebreaks"));
-  vsizer->Add(m_flowedTextRequested, 0, wxALL, 5);
-
-  m_exportInput = new wxCheckBox(panel, -1, _("Include input cells in the export of a worksheet"));
-  vsizer->Add(m_exportInput, 0, wxALL, 5);
-
   m_exportContainsWXMX = new wxCheckBox(panel, -1, _("Add the .wxmx file to the HTML export"));
   vsizer->Add(m_exportContainsWXMX, 0, wxALL, 5);
 
-  m_exportWithMathJAX = new wxCheckBox(panel, -1, _("Use MathJAX in HTML export"));
-  vsizer->Add(m_exportWithMathJAX, 0, wxALL, 5);
-  
   vsizer->AddGrowableRow(10);
   panel->SetSizer(vsizer);
   vsizer->Fit(panel);
@@ -534,7 +553,7 @@ wxPanel* ConfigDialogue::CreateOptionsPanel()
 {
   wxPanel *panel = new wxPanel(m_notebook, -1);
 
-  wxFlexGridSizer* grid_sizer = new wxFlexGridSizer(5, 2, 5, 5);
+  wxFlexGridSizer* grid_sizer = new wxFlexGridSizer(6, 2, 5, 5);
   wxFlexGridSizer* vsizer = new wxFlexGridSizer(17,1,5,5);
 
   wxStaticText *lang = new wxStaticText(panel, -1, _("Language:"));
@@ -555,6 +574,9 @@ wxPanel* ConfigDialogue::CreateOptionsPanel()
       _("Hungarian"),
       _("Italian"),
       _("Japanese"),
+#if wxCHECK_VERSION(3,0,1)
+      _("Kabyle"),
+#endif
       _("Norwegian"),
       _("Polish"),
       _("Portuguese (Brazilian)"),
@@ -568,7 +590,7 @@ wxPanel* ConfigDialogue::CreateOptionsPanel()
   grid_sizer->Add(m_language, 0, wxALL | wxALIGN_CENTER_VERTICAL, 5);
 
 #ifdef wxUSE_UNICODE
-  wxStaticText *additionalSymbols = new wxStaticText(panel, -1, _("Custom symbols sidebar glyphs:"));
+  wxStaticText *additionalSymbols = new wxStaticText(panel, -1, _("Additional symbols for the \"symbols\" sidebar:"));
   m_symbolPaneAdditionalChars = new wxTextCtrl(panel, -1);
   grid_sizer->Add(additionalSymbols, 0, wxALL | wxALIGN_CENTER_VERTICAL, 5);
   grid_sizer->Add(m_symbolPaneAdditionalChars, 0, wxALL | wxALIGN_CENTER_VERTICAL, 5);
@@ -584,6 +606,11 @@ wxPanel* ConfigDialogue::CreateOptionsPanel()
   grid_sizer->Add(ul, 0, wxALL | wxALIGN_CENTER_VERTICAL, 5);
   grid_sizer->Add(m_undoLimit, 0, wxALL, 5);
 
+  wxStaticText* rf = new wxStaticText(panel, -1, _("Recent files list length:"));
+  m_recentItems = new wxSpinCtrl(panel, -1, wxEmptyString, wxDefaultPosition, wxSize(100, -1), wxSP_ARROW_KEYS, 5, 30);
+  grid_sizer->Add(rf, 0, wxALL | wxALIGN_CENTER_VERTICAL, 5);
+  grid_sizer->Add(m_recentItems, 0, wxALL, 5);
+
   wxStaticText* df = new wxStaticText(panel, -1, _("Default animation framerate:"));
   m_defaultFramerate = new wxSpinCtrl(panel, -1, wxEmptyString, wxDefaultPosition, wxSize(100, -1), wxSP_ARROW_KEYS, 1, 200);
   grid_sizer->Add(df, 0, wxALL | wxALIGN_CENTER_VERTICAL, 5);
@@ -606,8 +633,10 @@ wxPanel* ConfigDialogue::CreateOptionsPanel()
   m_saveUntitled = new wxCheckBox(panel, -1, _("Ask to save untitled documents"));
   vsizer->Add(m_saveUntitled, 0, wxALL, 5);
 
-  m_fixReorderedIndices = new wxCheckBox(panel, -1, _("Fix reordered reference indices (of \%i, \%o) before saving"));
+  m_fixReorderedIndices = new wxCheckBox(panel, -1, _("Fix reordered reference indices (of %i, %o) before saving"));
   vsizer->Add(m_fixReorderedIndices, 0, wxALL, 5);
+  m_incrementalSearch = new wxCheckBox(panel, -1, _("Incremental Search"));
+  vsizer->Add(m_incrementalSearch, 0, wxALL, 5);
 
   vsizer->AddGrowableRow(10);
   panel->SetSizer(vsizer);
@@ -622,7 +651,7 @@ wxPanel* ConfigDialogue::CreateMaximaPanel()
 
   wxFlexGridSizer* sizer  = new wxFlexGridSizer(4, 2, 0, 0);  
   wxFlexGridSizer* sizer2 = new wxFlexGridSizer(6, 2, 0, 0);  
-  wxFlexGridSizer* vsizer = new wxFlexGridSizer(8,1,0,0);
+  wxFlexGridSizer* vsizer = new wxFlexGridSizer(9,1,0,0);
 
   wxStaticText *mp = new wxStaticText(panel, -1, _("Maxima program:"));
   m_maximaProgram = new wxTextCtrl(panel, -1, wxEmptyString, wxDefaultPosition, wxSize(250, -1), wxTE_RICH);
@@ -632,9 +661,9 @@ wxPanel* ConfigDialogue::CreateMaximaPanel()
   sizer->Add(m_maximaProgram, 0, wxALL | wxALIGN_CENTER_VERTICAL, 5);
   sizer->Add(m_mpBrowse, 0, wxALL | wxALIGN_CENTER_VERTICAL, 5);
 
-    int defaultPort = 4010;
+  int defaultPort = 4010;
   wxConfig::Get()->Read(wxT("defaultPort"), &defaultPort);
-  m_defaultPort = new wxSpinCtrl(panel, -1, wxEmptyString, wxDefaultPosition, wxSize(230, -1), wxSP_ARROW_KEYS, 50, 5000, defaultPort);
+  m_defaultPort = new wxSpinCtrl(panel, -1, wxEmptyString, wxDefaultPosition, wxSize(230, -1), wxSP_ARROW_KEYS, 50, 65534, defaultPort);
   m_defaultPort->SetValue(defaultPort);
   wxStaticText* dp = new wxStaticText(panel, -1, _("Default port for communication with wxMaxima:"));
   sizer->Add(dp, 0, wxALL | wxALIGN_CENTER_VERTICAL, 5);
@@ -678,6 +707,10 @@ wxPanel* ConfigDialogue::CreateMaximaPanel()
   vsizer->Add(m_pollStdOut,0,wxALL, 5);
   panel->SetSizerAndFit(vsizer);
 
+  m_restartOnReEvaluation = new wxCheckBox(panel, -1, _("Start a new maxima for each re-evaluation"));
+  vsizer->Add(m_restartOnReEvaluation,0,wxALL, 5);
+  panel->SetSizerAndFit(vsizer);
+
   return panel;
 }
 
@@ -801,13 +834,13 @@ void ConfigDialogue::OnClose(wxCloseEvent& event)
 
 void ConfigDialogue::WriteSettings()
 {
-  int i = 0;
   wxString search = wxT("maxima-htmldir");
   wxArrayString out;
   wxString maxima = m_maximaProgram->GetValue();
   wxConfig *config = (wxConfig *)wxConfig::Get();
   config->Write(wxT("abortOnError"), m_abortOnError->GetValue());
   config->Write(wxT("pollStdOut"), m_pollStdOut->GetValue());
+  Configuration::Get()->RestartOnReEvaluation(m_restartOnReEvaluation->GetValue());
   config->Write(wxT("maxima"), m_maximaProgram->GetValue());
   config->Write(wxT("parameters"), m_additionalParameters->GetValue());
   config->Write(wxT("fontSize"), m_fontSize);
@@ -819,13 +852,17 @@ void ConfigDialogue::WriteSettings()
   config->Write(wxT("changeAsterisk"), m_changeAsterisk->GetValue());
   config->Write(wxT("enterEvaluates"), m_enterEvaluates->GetValue());
   config->Write(wxT("saveUntitled"), m_saveUntitled->GetValue());
-  config->Write(wxT("openHCaret"), m_openHCaret->GetValue());
   config->Write(wxT("insertAns"), m_insertAns->GetValue());
   config->Write(wxT("autoIndent"), m_autoIndent->GetValue());
+  Configuration::Get()->SetOpenHCaret(m_openHCaret->GetValue());
+  config->Write(wxT("cursorJump"), m_cursorJump->GetValue());
+  config->Write(wxT("autoWrap"), m_autoWrap->GetValue());
   config->Write(wxT("labelWidth"), m_labelWidth->GetValue());
   config->Write(wxT("undoLimit"), m_undoLimit->GetValue());
+  config->Write(wxT("recentItems"), m_recentItems->GetValue());
   config->Write(wxT("bitmapScale"), m_bitmapScale->GetValue());
   config->Write(wxT("fixReorderedIndices"), m_fixReorderedIndices->GetValue());
+  config->Write(wxT("incrementalSearch"), m_incrementalSearch->GetValue());
   config->Write(wxT("showUserDefinedLabels"), m_showUserDefinedLabels->GetValue());
   config->Write(wxT("defaultPort"), m_defaultPort->GetValue());
   #ifdef __WXMSW__
@@ -842,10 +879,8 @@ void ConfigDialogue::WriteSettings()
   config->Write(wxT("TeXExponentsAfterSubscript"), m_TeXExponentsAfterSubscript->GetValue());
   config->Write(wxT("usePartialForDiff"), m_usePartialForDiff->GetValue());
   config->Write(wxT("wrapLatexMath"), m_wrapLatexMath->GetValue());
-  config->Write(wxT("flowedTextRequested"), m_flowedTextRequested->GetValue());
-  config->Write(wxT("exportInput"), m_exportInput->GetValue());
   config->Write(wxT("exportContainsWXMX"), m_exportContainsWXMX->GetValue());
-  config->Write(wxT("exportWithMathJAX"), m_exportWithMathJAX->GetValue());
+  config->Write(wxT("HTMLequationFormat"), m_exportWithMathJAX->GetSelection());
   config->Write(wxT("usejsmath"), m_useJSMath->GetValue());
   config->Write(wxT("keepPercent"), m_keepPercentWithSpecials->GetValue());
   config->Write(wxT("texPreamble"), m_texPreamble->GetValue());
@@ -855,6 +890,7 @@ void ConfigDialogue::WriteSettings()
     config->Write(wxT("pos-restore"), 1);
   else
     config->Write(wxT("pos-restore"), 0);
+  long i = 0;
   i = m_language->GetSelection();
   if (i > -1 && i < LANGUAGE_NUMBER)
     config->Write(wxT("language"), langs[i]);
@@ -899,7 +935,8 @@ void ConfigDialogue::OnMathBrowse(wxCommandEvent& event)
   math = wxGetFontFromUser(this, wxFont(m_mathFontSize, wxFONTFAMILY_DEFAULT,
                                         wxFONTSTYLE_NORMAL, wxFONTWEIGHT_NORMAL,
                                         false, m_mathFontName));
-
+  math.SetPointSize(m_mathFontSize);
+  
   if (math.Ok())
   {
     m_mathFontName = math.GetFaceName();
@@ -930,6 +967,8 @@ void ConfigDialogue::OnChangeFontFamily(wxCommandEvent& event)
                                         wxFONTWEIGHT_NORMAL,
                                         false, fontName,
                                         m_fontEncoding));
+  font.SetPointSize(fontsize);
+
   if (font.Ok())
   {
     if (event.GetId() == font_family)
@@ -1023,7 +1062,7 @@ void ConfigDialogue::ReadStyles(wxString file)
                    &tmp)) m_styleSelection.color.Set(tmp);
 
   // Text equal to the current selection
-  m_styleEqualsSelection.color = wxT("rgb(192,192,255)");
+  m_styleEqualsSelection.color = wxT("rgb(192,255,192)");
   if (config->Read(wxT("Style/EqualsSelection/color"),
                    &tmp)) m_styleEqualsSelection.color.Set(tmp);
 
@@ -1596,7 +1635,6 @@ void ConfigDialogue::UpdateExample()
 
 void ConfigDialogue::OnTabChange(wxBookCtrlEvent& event)
 {
-  std::cerr<<"Test\n";
   wxConfigBase* config = wxConfig::Get();
   config->Write(wxT("ConfigDialogTab"),event.GetSelection());
 }
@@ -1670,7 +1708,10 @@ END_EVENT_TABLE()
     italic = wxFONTSTYLE_SLANT;
   if (m_underlined)
     underlined = true;
-  dc.SetFont(wxFont(m_size, wxFONTFAMILY_MODERN, italic, bold, underlined, m_font));
+  wxFont font = wxFont(m_size, wxFONTFAMILY_MODERN, italic, bold, underlined, m_font);
+  font.SetPointSize(m_size);
+  dc.SetFont(font);
+
   dc.GetTextExtent(example, &text_width, &text_height);
 
   dc.DrawText(example, (panel_width - text_width) / 2,
