@@ -18,13 +18,22 @@
 //  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 //
 
+/*! \file
+  This file defines the class ConjugateCell
+
+  ConjugateCell is the MathCell type that represents the field that represents the 
+  conjugate() command.
+ */
+
 #include "ConjugateCell.h"
 #include "TextCell.h"
 
 ConjugateCell::ConjugateCell() : MathCell()
 {
   m_innerCell = NULL;
+  m_last = NULL;
   m_open = new TextCell(wxT("conjugate("));
+  m_open -> DontEscapeOpeningParenthesis();
   m_close = new TextCell(wxT(")"));
 }
 
@@ -54,7 +63,9 @@ MathCell* ConjugateCell::Copy()
   ConjugateCell* tmp = new ConjugateCell;
   CopyData(this, tmp);
   tmp->SetInner(m_innerCell->CopyList());
-
+  tmp->m_isBroken = m_isBroken;
+  tmp->m_open->DontEscapeOpeningParenthesis();
+ 
   return tmp;
 }
 
@@ -75,41 +86,47 @@ void ConjugateCell::SetInner(MathCell *inner)
   m_innerCell = inner;
 
   m_last = m_innerCell;
-  while (m_last->m_next != NULL)
-    m_last = m_last->m_next;
+  if (m_last != NULL)
+    while (m_last->m_next != NULL)
+      m_last = m_last->m_next;
 }
 
-void ConjugateCell::RecalculateWidths(CellParser& parser, int fontsize)
+void ConjugateCell::RecalculateWidths(int fontsize)
 {
-  double scale = parser.GetScale();
-  m_innerCell->RecalculateWidthsList(parser, fontsize);
+  Configuration *configuration = Configuration::Get();
+  double scale = configuration->GetScale();
+  m_innerCell->RecalculateWidthsList(fontsize);
   m_width = m_innerCell->GetFullWidth(scale) + SCALE_PX(8, scale);
-  m_open->RecalculateWidthsList(parser, fontsize);
-  m_close->RecalculateWidthsList(parser, fontsize);
+  m_open->RecalculateWidthsList(fontsize);
+  m_close->RecalculateWidthsList(fontsize);
   ResetData();
 }
 
-void ConjugateCell::RecalculateSize(CellParser& parser, int fontsize)
+void ConjugateCell::RecalculateHeight(int fontsize)
 {
-  double scale = parser.GetScale();
-  m_innerCell->RecalculateSizeList(parser, fontsize);
+  Configuration *configuration = Configuration::Get();
+  double scale = configuration->GetScale();
+  m_innerCell->RecalculateHeightList(fontsize);
   m_height = m_innerCell->GetMaxHeight() + SCALE_PX(4, scale);
   m_center = m_innerCell->GetMaxCenter() + SCALE_PX(2, scale);
-  m_open->RecalculateSizeList(parser, fontsize);
-  m_close->RecalculateSizeList(parser, fontsize);
+  m_open->RecalculateHeightList(fontsize);
+  m_close->RecalculateHeightList(fontsize);
 }
 
-void ConjugateCell::Draw(CellParser& parser, wxPoint point, int fontsize)
+void ConjugateCell::Draw(wxPoint point, int fontsize)
 {
-  double scale = parser.GetScale();
-  wxDC& dc = parser.GetDC();
-  if (DrawThisCell(parser, point))
+  Configuration *configuration = Configuration::Get();
+  MathCell::Draw(point, fontsize);
+
+  double scale = configuration->GetScale();
+  wxDC& dc = configuration->GetDC();
+  if (DrawThisCell(point) && InUpdateRegion())
   {
-    SetPen(parser);
+    SetPen();
     wxPoint in;
     in.x = point.x + SCALE_PX(4, scale);
     in.y = point.y;
-    m_innerCell->DrawList(parser, in, fontsize);
+    m_innerCell->DrawList(in, fontsize);
 
     dc.DrawLine(point.x + SCALE_PX(2, scale),
                 point.y - m_center + SCALE_PX(2, scale),
@@ -117,19 +134,24 @@ void ConjugateCell::Draw(CellParser& parser, wxPoint point, int fontsize)
                 point.y - m_center + SCALE_PX(2, scale)
 		);
 		//                point.y - m_center + m_height - SCALE_PX(2, scale));
-    UnsetPen(parser);
+    UnsetPen();
   }
-  MathCell::Draw(parser, point, fontsize);
 }
 
 wxString ConjugateCell::ToString()
 {
-  return wxT("conjugate(") + m_innerCell->ListToString() + wxT(")");
+  if (m_isBroken)
+    return wxEmptyString;
+  else
+    return wxT("conjugate(") + m_innerCell->ListToString() + wxT(")");
 }
 
 wxString ConjugateCell::ToTeX()
 {
-  return wxT("\\overline{") + m_innerCell->ListToTeX() + wxT("}");
+  if (m_isBroken)
+    return wxEmptyString;
+  else
+    return wxT("\\overline{") + m_innerCell->ListToTeX() + wxT("}");
 }
 
 wxString ConjugateCell::ToMathML()
@@ -139,6 +161,12 @@ wxString ConjugateCell::ToMathML()
     wxT("<mo>&#xaf;</mo></mover>\n");
 }
 
+wxString ConjugateCell::ToOMML()
+{
+  return wxT("<m:bar><m:barPr><m:pos m:val=\"top\"/> </m:barPr><m:e>") +
+    m_innerCell->ListToOMML() + wxT("</m:e></m:bar>");
+}
+
 wxString ConjugateCell::ToXML()
 {
   return wxT("<cj>") + m_innerCell->ListToXML() + wxT("</cj>");
@@ -166,8 +194,12 @@ bool ConjugateCell::BreakUp()
     m_isBroken = true;
     m_open->m_nextToDraw = m_innerCell;
     m_innerCell->m_previousToDraw = m_open;
-    m_last->m_nextToDraw = m_close;
-    m_close->m_previousToDraw = m_last;
+    wxASSERT_MSG(m_last != NULL,_("Bug: No last cell in an conjugateCell!"));
+    if(m_last != NULL)
+    {
+      m_last->m_nextToDraw = m_close;
+      m_close->m_previousToDraw = m_last;
+    }
     m_close->m_nextToDraw = m_nextToDraw;
     if (m_nextToDraw != NULL)
       m_nextToDraw->m_previousToDraw = m_close;
