@@ -23,6 +23,7 @@
 #define CONJUGATECELL_H
 
 #include "MathCell.h"
+#include "TextCell.h"
 
 /*! \file
 
@@ -31,6 +32,17 @@
 
 /*! A cell that represents an conjugate(x) block
 
+  In the case that this cell is broken into two lines in the order of
+  m_nextToDraw this cell is represented by the following individual 
+  cells:
+  
+   - The ConjugateCell itself
+   - The opening "conjugate("
+   - The contents
+   - The closing ")".
+   
+  If it isn't broken into multiple cells m_nextToDraw points to the 
+  cell that follows this Cell.
  */
 class ConjugateCell : public MathCell
 {
@@ -46,13 +58,15 @@ public:
   void SetParent(MathCell *parent);
 protected:
   MathCell *m_innerCell;
-  MathCell *m_open, *m_close, *m_last;
-  void RecalculateSize(CellParser& parser, int fontsize);
-  void RecalculateWidths(CellParser& parser, int fontsize);
-  void Draw(CellParser& parser, wxPoint point, int fontsize);
+  TextCell *m_open, *m_close;
+  MathCell *m_last;
+  void RecalculateHeight(int fontsize);
+  void RecalculateWidths(int fontsize);
+  void Draw(wxPoint point, int fontsize);
   wxString ToString();
   wxString ToTeX();
   wxString ToMathML();
+  wxString ToOMML();
   wxString ToXML();
 };
 
