@@ -21,6 +21,14 @@
 //  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 //
 
+
+/*! \file
+  This file defines the class DirStructure
+
+  DirStructure informs about what directory is to be found where on the current system.
+*/
+
+
 #include "Dirstructure.h"
 #include <wx/filename.h>
 
@@ -46,6 +54,9 @@ wxString Dirstructure::ResourcesDir()
 
 wxString Dirstructure::Prefix()
 {
+#ifndef PREFIX
+#define PREFIX "/usr"
+#endif
   return wxT(PREFIX);
 }
 
