@@ -27,6 +27,7 @@
 
 #include <vector>
 #include <list>
+#include <vector>
 #include <wx/tokenzr.h>
 
 /*! \file
@@ -35,32 +36,50 @@
  */
 
 /*! This class defines what the user sees as input cell
-
+  
   This class handles input cells including:
-   - The per-cell undo buffer
-   - The handling of key presses when this cell is active
+    - The per-cell undo buffer
+    - The handling of key presses when this cell is active
+  
+  Since Unicode doesn't provide us with a "soft linebreak" letter we 
+  use <code>\r</code> as a marker that this line has to be broken here if we
+  don't want it to extend beyond the right margin of the screen.
+
+  In a few places we use wxString::iterator instead of accessing individual 
+  letters within the string using the [] operator. This might look overly
+  complicated. But in UTF-8 all non-standard-ASCII-characters use more than one
+  byte making iterating over every single char of the string the only way of
+  determining which address char n is at. An iterator is the only way of not
+  having to determine the address of every single char indepently. 
  */
 class EditorCell : public MathCell
 {
 private:
-  //! Draw a box that marks the current selection
-  void MarkSelection(size_t start, size_t end,CellParser& parser,double scale, wxDC& dc, TextStyle style);
+  int ChangeNumpadToChar(int c);
+  //! A list of all potential autoComplete targets within this cell
+  wxArrayString m_wordList;
+ //! Draw a box that marks the current selection
+  void MarkSelection(long start, long end,double scale, wxDC& dc, TextStyle style,int fontsize);
   /*! The start of the current selection.
 
      - >0: the position of the cursors in characters from start
      - -1: Currently no selection is active
+
+     If the selection has been done from right to left m_selectionStart>m_selectionEnd.
    */
   long m_selectionStart;
   /*! The end of the current selection.
 
      - >0: the position of the cursors in characters from start
      - -1: Currently no selection is active
+
+     If the selection has been done from right to left m_selectionStart>m_selectionEnd.
    */
   long m_selectionEnd;
   /*! The currently selected string. 
 
     Since this string is defined statically it is available in every editor cell
-    for highlighting selected strings.
+    for highlighting other instances of the selected string.
   */
   static wxString m_selectionString;
   long m_oldSelectionStart;
@@ -72,6 +91,11 @@ private:
    */
   bool m_changeAsterisk;
 public:
+  //! Set the string that is to be highlighted as "identical to the curent selection"
+  static void SetSelectionString(wxString string)
+    {m_selectionString = string;}
+  //! A list of words that might be applicable to the autocomplete function.
+  wxArrayString GetWordList(){return m_wordList;}
   //! Has the selection changed since the last draw event?
   bool m_selectionChanged;
   //! The constructor
@@ -85,93 +109,120 @@ public:
     \param posInLine The number of characters that come before the input in the same line
     \todo Implement the actual TAB expansion
   */
-  wxString TabExpand(wxString input, size_t posInLine);
+  wxString TabExpand(wxString input, long posInLine);
   //! Escape all chars that cannot be used in HTML otherwise
   static wxString EscapeHTMLChars(wxString input);
   //! Convert all but the first of a row of multiple spaces to non-breakable
   static wxString PrependNBSP(wxString input);
-
   void Destroy();
   MathCell* Copy();
-  /*! Recalculate the widths of the current cell.
-
-      \todo If we use a centered dot instead of a * and we don't use a fixed-
-      size fonts we miscalculate the widths here.
-   */
-  void RecalculateWidths(CellParser& parser, int fontsize);
-  void Draw(CellParser& parser, wxPoint point, int fontsize);
-  //! Convert the current cell to a string
+  //! Recalculate the widths of the current cell.
+  void RecalculateWidths(int fontsize);
+  void Draw(wxPoint point, int fontsize);
   wxString ToString();
+  /*! Convert the current cell to a string
+  
+    \param dontLimitToSelection
+    - false: If text is selected return only the selected text
+    - true:  Always return all text in this text cell
+  */
+  wxString ToString(bool dontLimitToSelection);
   //! Convert the current cell to LaTeX code
   wxString ToTeX();
   //! Convert the current cell to XML code for inclusion in a .wxmx file.
   wxString ToXML();
   //! Convert the current cell to HTML code.
   wxString ToHTML();
-  void SetFont(CellParser& parser, int fontsize);
-  void SetForeground(CellParser& parser);
+  //! Convert the current cell to RTF code
+  wxString ToRTF();
+  //! Set the currently used font to the one that matches this cell's formatting
+  void SetFont(int fontsize);
+  //! Sets the current color to this cell's foreground color
+  void SetForeground();
 
   /*! Sets the text that is to be displayed.
     
     Automatically calls StyleText().
    */
-  void SetValue(wxString text);
+  void SetValue(const wxString &text);
+  /*! Returns the text contained in this cell
+
+    Naturally all soft line breaks are converted back to spaces beforehand.
+   */
   wxString GetValue()
   {
     return m_text;
   }
-  /*! Converts m_text to a list of styled text snippets that will later be used by draw().
+  /*! Converts m_text to a list of styled text snippets that will later be displayed by draw().
 
-    \todo Actually set the needed text styles.
+    This function also generates a wordlist for this EditorCell so Autocompletion can learn
+    about variable names contained in lists or cells that still haven't been evaluated.
+
+    For cells containing text instead of code this function adds a <code>\r</code> as a marker
+    that this line is to be broken here until the window's width changes.
    */
   void StyleText();
   void Reset();
   //! Decide what to do if the user pressed a key when this cell was selected
   void ProcessEvent(wxKeyEvent& event);
-  bool ActivateCell();
+  //! If the cell is activated it contains a blinking cursor.
+  bool ActivateCell(bool active);
   //! Return the index of the 1st char of the line containing the letter #pos.
-  size_t BeginningOfLine(size_t pos);
-  size_t EndOfLine(size_t pos);
+  size_t BeginningOfLine(long pos);
+  //! Return the index of the last char of the line containing the letter #pos,
+  size_t EndOfLine(long pos);
+  //! Adds a ";" to the end of the last command in this cell in case that it doesn't end in $ or ;
   bool AddEnding();
-  void PositionToXY(int pos, int* line, int* col);
+  //! Determines which line and column the pos'th char is at.
+  void PositionToXY(int pos, unsigned int* line, unsigned int* col);
+  //! Determines which index the char at the position "x chars left, y chars down" is at.
   int XYToPosition(int x, int y);
-  wxPoint PositionToPoint(CellParser& parser, int pos = -1);
+  //! The screen coordinates of the cursor
+  wxPoint PositionToPoint(int fontsize, int pos = -1);
+  //! Sets the cursor to the screen coordinate point
   void SelectPointText(wxDC &dc, wxPoint& point);
+  //! Selects the text beween the screen coordinates one and two
   void SelectRectText(wxDC &dc, wxPoint& one, wxPoint& two);
+  //! Selects the word the cursor is currently at.
   wxString SelectWordUnderCaret(bool selectParens = true, bool toRight = true);
+  //! Is the point point inside the currently selected text?
   bool IsPointInSelection(wxDC& dc, wxPoint point);
   bool CopyToClipboard();
   bool CutToClipboard();
   void PasteFromClipboard(bool primary = false);
+  //! Get the character position the selection has been started with
+  int GetSelectionStart(){return m_selectionStart;}
+  //! Get the character position the selection has been ended with
+  int GetSelectionEnd(){return m_selectionEnd;}
+  //! Select the whole text contained in this Cell
   void SelectAll()
   {
     m_selectionStart = 0;
     m_selectionEnd = m_positionOfCaret = m_text.Length();
   }
+  //! Does the selection currently span the whole cell?
+  bool AllSelected()
+    {
+      return (m_selectionStart==0)&&(m_selectionEnd == (long) m_text.Length());
+    }
+  //! Unselect everything.
   void SelectNone()
   {
     m_selectionStart = m_selectionEnd = 0;
   }
+  //! Is there any text selected right now?
   bool SelectionActive()
   {
-    return (m_selectionStart != -1)&&(m_selectionEnd != -1);
+    return (m_selectionStart >= 0)&&(m_selectionEnd >= 0);
   }
 
   bool CanCopy()
   {
     return m_selectionStart != -1;
   }
-  void SetMatchParens(bool match)
-  {
-    m_matchParens = match;
-  }
-    void SetInsertAns(bool insertAns)
-  {
-    m_insertAns = insertAns;
-  }
   bool FindMatchingQuotes();
   void FindMatchingParens();
-  int GetLineWidth(wxDC& dc, int line, int end);
+  int GetLineWidth(wxDC& dc,unsigned int line, int end);
   //! true, if this cell's width has to be recalculated.
   bool IsDirty()
   {
@@ -198,7 +249,7 @@ public:
   //! Move the cursor to the start of this cell
   void CaretToStart();
   //! Is the cursor at the end of this cell?
-  bool CaretAtEnd() { return m_positionOfCaret == m_text.Length(); }
+  bool CaretAtEnd() { return m_positionOfCaret == (long)m_text.Length(); }
   //! Move the cursor to the end of this cell
   void CaretToEnd();
   //! Move the cursor to a certain position in the cell
@@ -213,6 +264,12 @@ public:
   void Redo();
   //! Save the current contents of this cell in the undo buffer.
   void SaveValue();
+  /*! DivideAtCaret
+    Returns the string from caret to end and
+    modifies the m_text so it contains only the string
+    from beginning to caret
+    Used for 'Divide Cell', called from MathCtrl
+  */
   wxString DivideAtCaret();
   void CommentSelection();
   void ClearUndo();
@@ -305,6 +362,10 @@ private:
     wxString m_text;
     //! Do we really want to style this text portion different than the default?
     bool m_styleThisText;
+    //! By How many pixels we want to indent this line?
+    int m_indentPixels;
+    //! Chars that mark continued indentation
+    wxString m_indentChar;
   public:    
     //! Defines a piece of styled text
     StyledText(TextStyle style,wxString text)
@@ -314,18 +375,35 @@ private:
         m_styleThisText = true;
       }
 
-    //! Defines a piece of text with the default style
-    StyledText(wxString text)
+    /*! Defines a piece of text with the default style that possibly is indented
+
+      \todo Currently we access every char of the string by telling wxWidgets that we want
+      char #n. The wxWidgets docs tell that using an iterator is much more efficient.
+     */
+    StyledText(wxString text,int indentPixels = 0,wxString indentChar=wxEmptyString)
       {
         m_text = text;
+        m_style = TS_DEFAULT;
         m_styleThisText = false;
+        m_indentPixels = indentPixels;
+        m_indentChar = indentChar;
       }
     //! Returns the piece of text
     wxString GetText()
       {
         return m_text;
       }
-    //! If StyleSet() is true this function returns the color of this text portion
+    //! By how many pixels do we need to indent this line due to a bullet list or similar?
+    int GetIndentPixels()
+      {
+        return m_indentPixels;
+      }
+
+    wxString GetIndentChar()
+      {
+        return m_indentChar;
+      }
+//! If StyleSet() is true this function returns the color of this text portion
     TextStyle GetStyle()
       {
         return m_style;
@@ -337,12 +415,12 @@ private:
       }
   };
   
-  std::list<StyledText> m_styledText;
+  std::vector<StyledText> m_styledText;
 
 #if wxUSE_UNICODE
   /*! Handle ESC shortcuts for special characters
 
-    These characters can be tought to LaTeX and the html browser if neccessary in
+    These characters can be tought to LaTeX and the html browser if necessary in
     TextCell::ToTeX and EditorCell::ToTeX. They can also be
     converted to maxima strings in wxMaxima::SendMaxima.
    */
@@ -359,13 +437,16 @@ private:
   int m_caretColumn;
   long m_lastSelectionStart;
 //  long m_oldStart, m_oldEnd;
-  int m_numberOfLines;
+  unsigned int m_numberOfLines;
   bool m_isActive;
   int m_fontSize;
+  /*! The font size we were called with  the last time
+
+    We need to know this in order to be able to detect we need a full recalculation.
+   */
+  int m_fontSize_Last;
   int m_charHeight;
-  bool m_matchParens;
   int m_paren1, m_paren2;
-  bool m_insertAns;
   //! Does this cell's size have to be recalculated?
   bool m_isDirty;
   bool m_displayCaret;
