@@ -1,7 +1,7 @@
 // -*- mode: c++; c-file-style: "linux"; c-basic-offset: 2; indent-tabs-mode: nil -*-
 //
 //  Copyright (C) 2004-2015 Andrej Vodopivec <andrej.vodopivec@gmail.com>
-//            (C) 2014-2015 Gunter Königsmann <wxMaxima@physikbuch.de>
+//            (C) 2014-2016 Gunter Königsmann <wxMaxima@physikbuch.de>
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
@@ -23,7 +23,31 @@
 #define FUNCELL_H
 
 #include "MathCell.h"
+/*! \file
 
+  This file declares the class FunCell() that represents a maxima function.
+ */
+
+/*! FunCell represents a maxiam function no special visual representation exists for 
+
+  Examples of functions with special visual representation would be:
+   - SqurtCell
+   - ExptCell
+   - AbsCell
+   - ConjugateCell
+
+  In the case that this cell is broken into two lines in the order of
+  m_nextToDraw this cell is represented by the following individual 
+  cells:
+  
+   - The FunCell itself
+   - The function name"
+   - The ParenCell containing its contents
+   - The closing ")".
+   
+  If it isn't broken into multiple cells m_nextToDraw points to the 
+  cell that follows this Cell. 
+*/
 class FunCell : public MathCell
 {
 public:
@@ -33,12 +57,14 @@ public:
   void Destroy();
   void SetName(MathCell *base);
   void SetArg(MathCell *index);
-  void RecalculateSize(CellParser& parser, int fontsize);
-  void RecalculateWidths(CellParser& parser, int fontsize);
-  void Draw(CellParser& parser, wxPoint point, int fontsize);
+  void RecalculateHeight(int fontsize);
+  void RecalculateWidths(int fontsize);
+  void Draw(wxPoint point, int fontsize);
   wxString ToString();
   wxString ToTeX();
+  wxString ToMathML();
   wxString ToXML();
+  wxString ToOMML();
   void SelectInner(wxRect& rect, MathCell** first, MathCell** last);
   bool BreakUp();
   void Unbreak();
