@@ -21,6 +21,13 @@
 //  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 //
 
+/*! \file
+  This file defines the class GroupCell
+
+  GroupCell is the MathCell type that bundles user input with eventual images or 
+  output from maxima that belongs to it.
+*/
+
 #include <wx/config.h>
 #include <wx/clipbrd.h>
 #include "MarkDown.h"
@@ -43,6 +50,8 @@ GroupCell::GroupCell(int groupType, wxString initString) : MathCell()
   m_outputRect.width = 0;
   m_outputRect.height = 0;
   m_group = this;
+  m_fontSize = 10;
+  m_mathFontSize = 10;
   m_forceBreakLine = true;
   m_breakLine = true;
   m_type = MC_TYPE_GROUP;
@@ -63,13 +72,7 @@ GroupCell::GroupCell(int groupType, wxString initString) : MathCell()
     m_input->SetType(MC_TYPE_MAIN_PROMPT);
   }
 
-  bool match = true;
-  bool insertAns = false;
-  wxConfig::Get()->Read(wxT("matchParens"), &match);
-  wxConfig::Get()->Read(wxT("insertAns"), &insertAns);
   EditorCell *editor = new EditorCell();
-  editor->SetMatchParens(match);
-  editor->SetInsertAns(insertAns);
 
   switch (groupType) {
     case GC_TYPE_CODE:
@@ -279,6 +282,7 @@ void GroupCell::AppendInput(MathCell *cell)
   }
 }
 
+
 void GroupCell::SetOutput(MathCell *output)
 {
   if (output == NULL)
@@ -294,6 +298,7 @@ void GroupCell::SetOutput(MathCell *output)
   while (m_lastInOutput->m_next != NULL)
     m_lastInOutput = m_lastInOutput->m_next;
 
+ // ResetSize();
   //m_appendedCells = output;
 }
 
@@ -304,9 +309,11 @@ void GroupCell::RemoveOutput()
     return;
   
   DestroyOutput(!(GetGroupType() == GC_TYPE_IMAGE));
+
+  // Calculate the new cell height.
+
   ResetSize();
-  if(GetGroupType() != GC_TYPE_IMAGE)
-    m_height = GetEditable()->GetHeight();
+  RecalculateHeight(Configuration::Get()->GetDefaultFontSize());
   m_hide = false;
 }
 
@@ -343,20 +350,25 @@ void GroupCell::AppendOutput(MathCell *cell)
 
   if (m_appendedCells == NULL)
     m_appendedCells = cell;
+ // ResetSize();
 }
 
-void GroupCell::Recalculate(CellParser& parser, int d_fontsize, int m_fontsize)
+void GroupCell::Recalculate()
 {
+  int d_fontsize = Configuration::Get()->GetDefaultFontSize();
+  int m_fontsize = Configuration::Get()->GetMathFontSize();
+
   m_fontSize = d_fontsize;
   m_mathFontSize = m_fontsize;
 
-  RecalculateWidths(parser, d_fontsize);
-  RecalculateSize(parser, d_fontsize);
+  RecalculateWidths(d_fontsize);
+  RecalculateHeight(d_fontsize);
 }
 
-void GroupCell::RecalculateWidths(CellParser& parser, int fontsize)
+void GroupCell::RecalculateWidths(int fontsize)
 {
-  if (m_width == -1 || m_height == -1 || parser.ForceUpdate())
+  Configuration *configuration = Configuration::Get();
+  if (m_width == -1 || m_height == -1 || configuration->ForceUpdate())
   {
     // special case of 'line cell'
     if (m_groupType == GC_TYPE_PAGEBREAK) {
@@ -368,37 +380,53 @@ void GroupCell::RecalculateWidths(CellParser& parser, int fontsize)
 
     UnBreakUpCells();
 
-    double scale = parser.GetScale();
-    m_input->RecalculateWidthsList(parser, fontsize);
+    double scale = configuration->GetScale();
+
+    m_input->RecalculateWidthsList(fontsize);
 
     // recalculate the position of input in ReEvaluateSelection!
-    if (m_input->m_next != NULL) {
+    if (m_input->m_next != NULL)
+    {
       m_input->m_next->m_currentPoint.x = m_currentPoint.x + m_input->GetWidth() + MC_CELL_SKIP;
     }
 
-    if (m_output == NULL || m_hide) {
-      m_width = m_input->GetFullWidth(scale);
+    if (m_output == NULL || m_hide)
+    {
+      if((configuration->ShowCodeCells()) ||
+         (m_groupType != GC_TYPE_CODE))
+      {
+        m_width = m_input->GetFullWidth(scale);
+      }
+      else
+        m_width = 0;
     }
-
-    else {
+    else
+    {
       MathCell *tmp = m_output;
       while (tmp != NULL) {
-        tmp->RecalculateWidths(parser, tmp->IsMath() ? m_mathFontSize : m_fontSize);
+        tmp->RecalculateWidths(tmp->IsMath() ? m_mathFontSize : m_fontSize);
         tmp = tmp->m_next;
       }
-      // This is not correct, m_width will be computed correctly in RecalculateSize!
-      m_width = m_input->GetFullWidth(scale);
+      // This is not correct, m_width will be computed correctly in RecalculateHeight!
+      if((configuration->ShowCodeCells()) ||
+         (m_groupType != GC_TYPE_CODE))
+      {
+        m_width = m_input->GetFullWidth(scale);
+      }
     }
 
-    BreakUpCells(parser, m_fontSize, parser.GetClientWidth());
-    BreakLines(parser.GetClientWidth());
+    BreakUpCells(m_fontSize, configuration->GetClientWidth());
+    BreakLines(configuration->GetClientWidth());
   }
   ResetData();
 }
 
-void GroupCell::RecalculateSize(CellParser& parser, int fontsize)
+void GroupCell::RecalculateHeight(int fontsize)
 {
-  if (m_width == -1 || m_height == -1 || parser.ForceUpdate())
+  Configuration *configuration = Configuration::Get();
+  double scale = configuration->GetScale();
+
+  if (m_width == -1 || m_height == -1 || configuration->ForceUpdate())
   {
     // special case
     if (m_groupType == GC_TYPE_PAGEBREAK) {
@@ -406,29 +434,60 @@ void GroupCell::RecalculateSize(CellParser& parser, int fontsize)
       m_height = 2;
       m_center = 0;
       m_indent = 0;
-      MathCell::RecalculateWidthsList(parser, fontsize);
+      MathCell::RecalculateWidthsList(fontsize);
       return;
     }
 
-    double scale = parser.GetScale();
-    m_input->RecalculateSizeList(parser, fontsize);
-    m_center = m_input->GetMaxCenter();
-    m_height = m_input->GetMaxHeight();
-    m_indent = parser.GetIndent();
+    if((configuration->ShowCodeCells()) ||
+       (m_groupType != GC_TYPE_CODE))
+    {
+      m_input->RecalculateHeightList(fontsize);
+      m_center = m_input->GetMaxCenter();
+      m_height = m_input->GetMaxHeight();
+    }
+    else
+    {
+      m_center = 0;
+      m_height = 0;
+    }
+    
+    m_indent = configuration->GetIndent();
 
-    if (m_output != NULL && !m_hide) {
+    if (m_output != NULL && !m_hide)
+    {
       MathCell *tmp = m_output;
       while (tmp != NULL) {
-        tmp->RecalculateSize(parser,  tmp->IsMath() ? m_mathFontSize : m_fontSize);
+        tmp->RecalculateHeight(tmp->IsMath() ? m_mathFontSize : m_fontSize);
         tmp = tmp->m_next;
       }
-
+      
+      if(m_previous == NULL)
+      {
+        m_currentPoint.x = MC_GROUP_LEFT_INDENT;
+        m_currentPoint.y = Configuration::Get()->GetBaseIndent() + GetMaxCenter();
+      }
+      else
+      {
+      m_currentPoint.x = MC_GROUP_LEFT_INDENT;
+      m_currentPoint.y = dynamic_cast<GroupCell*>(m_previous) -> m_currentPoint.y +
+        dynamic_cast<GroupCell*>(m_previous) -> GetMaxDrop() + GetMaxCenter() + Configuration::Get()->GetGroupSkip();
+      }
+      
       m_outputRect.x = m_currentPoint.x;
       m_outputRect.y = m_currentPoint.y - m_output->GetMaxCenter();
       m_outputRect.width = 0;
       m_outputRect.height = 0;
-      m_height = m_input->GetMaxHeight();
-      m_width = m_input->GetFullWidth(scale);
+      if((configuration->ShowCodeCells()) ||
+         (m_groupType != GC_TYPE_CODE))
+      {
+        m_height = m_input->GetMaxHeight();
+        m_width = m_input->GetFullWidth(scale);
+      }
+      else
+      {
+        m_height = 0;
+        m_width  = 0;
+      }
 
       tmp = m_output;
       while (tmp != NULL) {
@@ -443,50 +502,58 @@ void GroupCell::RecalculateSize(CellParser& parser, int fontsize)
         tmp = tmp->m_nextToDraw;
       }
     }
+    else
+    {
+      if(m_previous == NULL)
+      {
+        m_currentPoint.x = MC_GROUP_LEFT_INDENT;
+        m_currentPoint.y = Configuration::Get()->GetBaseIndent() + GetMaxCenter();
+      }
+      else
+      {
+        m_currentPoint.x = MC_GROUP_LEFT_INDENT;
+        m_currentPoint.y = dynamic_cast<GroupCell*>(m_previous) -> m_currentPoint.y +
+          dynamic_cast<GroupCell*>(m_previous) -> GetMaxDrop() + GetMaxCenter() + Configuration::Get()->GetGroupSkip();
+      }
+    }
+    if(m_height < SCALE_PX(10, scale))
+      m_height = SCALE_PX(10, scale);
   }
-
+    
   m_appendedCells = NULL;
 
-  if(m_previous == NULL)
-  {
-    m_currentPoint.x = MC_GROUP_LEFT_INDENT;
-    m_currentPoint.y = MC_BASE_INDENT + GetMaxCenter();
-  }
-  else
-  {
-    m_currentPoint.x = MC_GROUP_LEFT_INDENT;
-    m_currentPoint.y = dynamic_cast<GroupCell*>(m_previous) -> m_currentPoint.y +
-      dynamic_cast<GroupCell*>(m_previous) -> GetMaxDrop() + GetMaxCenter() + MC_GROUP_SKIP;
-  }
-  
   if(m_input)
     m_input->m_currentPoint = m_currentPoint;
+  if(GetEditable())
+    GetEditable()->m_currentPoint = m_currentPoint;
 }
 
 // We assume that appended cells will be in a new line!
-void GroupCell::RecalculateAppended(CellParser& parser)
+void GroupCell::RecalculateAppended()
 {
+  Configuration *configuration = Configuration::Get();
   if (m_appendedCells == NULL)
     return;
 
   MathCell *tmp = m_appendedCells;
-  int fontsize = m_fontSize;
-  double scale = parser.GetScale();
-
+  m_fontSize = configuration->GetFontSize(TS_TEXT);
+  double scale = configuration->GetScale();
+  m_mathFontSize = configuration->GetMathFontSize();
+  
   // Recalculate widths of cells
   while (tmp != NULL) {
-    tmp->RecalculateWidths(parser, tmp->IsMath() ? m_mathFontSize : m_fontSize);
+    tmp->RecalculateWidths(tmp->IsMath() ? m_mathFontSize : m_fontSize);
     tmp = tmp->m_next;
   }
 
   // Breakup cells and break lines
-  BreakUpCells(m_appendedCells, parser, fontsize, parser.GetClientWidth());
-  BreakLines(m_appendedCells, parser.GetClientWidth());
+  BreakUpCells(m_appendedCells, m_fontSize, configuration->GetClientWidth());
+  BreakLines(m_appendedCells, configuration->GetClientWidth());
 
   // Recalculate size of cells
   tmp = m_appendedCells;
   while (tmp != NULL) {
-    tmp->RecalculateSize(parser,  tmp->IsMath() ? m_mathFontSize : m_fontSize);
+    tmp->RecalculateHeight(tmp->IsMath() ? m_mathFontSize : m_fontSize);
     tmp = tmp->m_next;
   }
 
@@ -507,67 +574,80 @@ void GroupCell::RecalculateAppended(CellParser& parser)
   m_appendedCells = NULL;
 }
 
-void GroupCell::Draw(CellParser& parser, wxPoint point, int fontsize)
+void GroupCell::Draw(wxPoint point, int fontsize)
 {
-  double scale = parser.GetScale();
-  wxDC& dc = parser.GetDC();
+  MathCell::Draw(point, fontsize);
+
+  Configuration *configuration = Configuration::Get();
+  double scale = configuration->GetScale();
+  wxDC& dc = configuration->GetDC();
   if (m_width == -1 || m_height == -1) {
-    RecalculateWidths(parser, fontsize);
-    RecalculateSize(parser, fontsize);
+    RecalculateWidths(fontsize);
+    RecalculateHeight(fontsize);
   }
-  if (DrawThisCell(parser, point))
+  if (DrawThisCell(point))
   {
     // draw a thick line for 'page break'
     // and return
     if (m_groupType == GC_TYPE_PAGEBREAK) {
       wxRect rect = GetRect(false);
       int y = rect.GetY();
-      wxPen pen(parser.GetColor(TS_CURSOR), 1, wxPENSTYLE_DOT);
-      dc.SetPen(pen);
-      dc.DrawLine(0, y , 10000, y);
-      MathCell::Draw(parser, point, fontsize);
+      wxPen pen(configuration->GetColor(TS_CURSOR), 1, wxPENSTYLE_DOT);
+      dc.SetPen(pen);      
+      dc.DrawLine(0,y,m_canvasSize.GetWidth(),y);
+      MathCell::Draw(point, fontsize);
       return;
     }
 
     //
     // Paint background if we have a text cell
     //
-    if (m_groupType == GC_TYPE_TEXT && !parser.GetPrinter()) {
+    if (m_groupType == GC_TYPE_TEXT && !configuration->GetPrinter()) {
       wxRect rect = GetRect(false);
       int y = rect.GetY();
 
       if (m_height > 0 && m_width > 0 && y>=0) {
-        wxBrush br(parser.GetColor(TS_TEXT_BACKGROUND));
+        wxBrush br(configuration->GetColor(TS_TEXT_BACKGROUND));
         dc.SetBrush(br);
-        wxPen pen(parser.GetColor(TS_TEXT_BACKGROUND));
+        wxPen pen(configuration->GetColor(TS_TEXT_BACKGROUND));
         dc.SetPen(pen);
-        dc.DrawRectangle(0, y , 10000, rect.GetHeight());
+        rect.SetWidth(m_canvasSize.GetWidth());
+        if(InUpdateRegion(rect))
+          dc.DrawRectangle(CropToUpdateRegion(rect));
       }
     }
     //
     // Draw input and output
     //
-    SetPen(parser);
+    SetPen();
     wxPoint in(point);
-    parser.Outdated(false);
-    m_input->DrawList(parser, in, fontsize);
-    if (m_groupType == GC_TYPE_CODE && m_input->m_next)
-      parser.Outdated(((EditorCell *)(m_input->m_next))->ContainsChanges());
 
+    if((configuration->ShowCodeCells()) ||
+       (m_groupType != GC_TYPE_CODE))
+    {
+      configuration->Outdated(false);
+      m_input->DrawList(in, fontsize);
+      if (m_groupType == GC_TYPE_CODE && m_input->m_next)
+        configuration->Outdated(((EditorCell *)(m_input->m_next))->ContainsChanges());
+    }
+    
     if (m_output != NULL && !m_hide) {
       MathCell *tmp = m_output;
       int drop = tmp->GetMaxDrop();
-      in.y += m_input->GetMaxDrop() + m_output->GetMaxCenter();
+      if((configuration->ShowCodeCells()) ||
+         (m_groupType != GC_TYPE_CODE))
+      {
+        in.y += m_input->GetMaxDrop();
+      }
+      in.y += m_output->GetMaxCenter();
       m_outputRect.y = in.y - m_output->GetMaxCenter();
       m_outputRect.x = in.x;
 
       while (tmp != NULL) {
-
+        tmp->m_currentPoint = in;
         if (!tmp->m_isBroken) {
-          tmp->m_currentPoint.x = in.x;
-          tmp->m_currentPoint.y = in.y;
-          if (tmp->DrawThisCell(parser, in))
-            tmp->Draw(parser, in, MAX(tmp->IsMath() ? m_mathFontSize : m_fontSize, MC_MIN_SIZE));
+          if (tmp->DrawThisCell(in))
+            tmp->Draw(in, MAX(tmp->IsMath() ? m_mathFontSize : m_fontSize, MC_MIN_SIZE));
           if (tmp->m_nextToDraw != NULL) {
             if (tmp->m_nextToDraw->BreakLineHere()) {
               in.x = m_indent;
@@ -593,15 +673,15 @@ void GroupCell::Draw(CellParser& parser, wxPoint point, int fontsize)
       }
     }
 
-    parser.Outdated(false);
+    configuration->Outdated(false);
     MathCell *editable = GetEditable();
     if (editable != NULL && editable->IsActive()) {
-      dc.SetPen( *(wxThePenList->FindOrCreatePen(parser.GetColor(TS_ACTIVE_CELL_BRACKET), 2, wxPENSTYLE_SOLID))); // window linux, set a pen
-      dc.SetBrush( *(wxTheBrushList->FindOrCreateBrush(parser.GetColor(TS_ACTIVE_CELL_BRACKET)))); //highlight c.
+      dc.SetPen( *(wxThePenList->FindOrCreatePen(configuration->GetColor(TS_ACTIVE_CELL_BRACKET), 2, wxPENSTYLE_SOLID))); // window linux, set a pen
+      dc.SetBrush( *(wxTheBrushList->FindOrCreateBrush(configuration->GetColor(TS_ACTIVE_CELL_BRACKET)))); //highlight c.
     }
     else {
-      dc.SetPen( *(wxThePenList->FindOrCreatePen(parser.GetColor(TS_CELL_BRACKET), 1, wxPENSTYLE_SOLID))); // window linux, set a pen
-      dc.SetBrush( *(wxTheBrushList->FindOrCreateBrush(parser.GetColor(TS_CELL_BRACKET)))); //highlight c.
+      dc.SetPen( *(wxThePenList->FindOrCreatePen(configuration->GetColor(TS_CELL_BRACKET), 1, wxPENSTYLE_SOLID))); // window linux, set a pen
+      dc.SetBrush( *(wxTheBrushList->FindOrCreateBrush(configuration->GetColor(TS_CELL_BRACKET)))); //highlight c.
     }
 
     if ((!m_hide) && (!m_hiddenTree)) {
@@ -639,17 +719,16 @@ void GroupCell::Draw(CellParser& parser, wxPoint point, int fontsize)
       dc.DrawLine(point.x - SCALE_PX(10, scale), point.y - m_center + m_height,
           point.x - SCALE_PX(2, scale) , point.y - m_center + m_height);
       // middle horizontal
-      if (m_groupType == GC_TYPE_CODE && m_output != NULL && !m_hide) {
+      if (configuration->ShowCodeCells() && m_groupType == GC_TYPE_CODE && m_output != NULL && !m_hide) {
         dc.DrawLine(point.x - SCALE_PX(6, scale),
-            point.y - m_center + m_input->GetMaxHeight(),
-            point.x - SCALE_PX(10, scale),
-            point.y - m_center + m_input->GetMaxHeight());
+                    point.y - m_center + m_input->GetMaxHeight(),
+                    point.x - SCALE_PX(10, scale),
+                    point.y - m_center + m_input->GetMaxHeight());
       }
     }
 
-    UnsetPen(parser);
+    UnsetPen();
   }
-  MathCell::Draw(parser, point, fontsize);
 }
 
 wxRect GroupCell::HideRect()
@@ -660,16 +739,45 @@ wxRect GroupCell::HideRect()
 wxString GroupCell::ToString()
 {
   wxString str;
-  if (GetEditable()) {
-    str = m_input->ListToString();
-    if (m_output != NULL && !m_hide) {
-      MathCell *tmp = m_output;
-      while (tmp != NULL) {
-        if (tmp->ForceBreakLineHere() && str.Length()>0)
+  Configuration *configuration = Configuration::Get();
+  
+  if(m_input != NULL)
+  {
+    if((configuration->ShowCodeCells()) ||
+       (m_groupType != GC_TYPE_CODE))
+    {
+      str = m_input->ToString();
+      
+      if (GetEditable() != NULL)
+        str += GetEditable()->ToString();
+      
+      str.Replace(wxT("\n"),wxT("\n\t"));
+    }
+  }
+
+  if (m_output != NULL && !m_hide)
+  {
+    MathCell *tmp = m_output;
+    bool firstCell = true;
+    while (tmp != NULL) {
+      if (firstCell || (tmp->ForceBreakLineHere() && str.Length()>0))
+      {
+        if(firstCell)
           str += wxT("\n");
-        str += tmp->ToString();
-        tmp = tmp->m_nextToDraw;
+        else
+        {
+          str += wxT("\n");
+          if(
+            (tmp->m_nextToDraw != NULL ) &&
+              (tmp->GetStyle()!=TS_LABEL) &&
+            (tmp->GetStyle()!=TS_USERLABEL)
+            )
+            str += wxT("\t");
+        }
       }
+      str += tmp->ToString();
+      firstCell = false;
+      tmp = tmp->m_nextToDraw;
     }
   }
   return str;
@@ -680,19 +788,63 @@ wxString GroupCell::ToTeX()
   return ToTeX(wxEmptyString, wxEmptyString, NULL);
 }
 
+wxString GroupCell::ToRTF()
+{
+  Configuration *configuration = Configuration::Get();
+  if(m_groupType == GC_TYPE_PAGEBREAK)
+    return(wxT("\\page "));
+  
+  wxString retval;
+  if(m_groupType == GC_TYPE_CODE)
+  {
+    if(m_input != NULL &&
+       (
+         (configuration->ShowCodeCells()) ||
+         (m_groupType != GC_TYPE_CODE)
+         )
+      )
+    {
+      if(m_previous != NULL)
+        retval = wxT("\\par}{\\pard\\s22\\li1105\\lin1105\\fi-1105\\f0\\fs24 \n");
+      else
+        retval += wxT("\\pard\\s22\\li1105\\lin1105\\fi-1105\\f0\\fs24 ");
+      retval += RTFescape(m_input->ToString());
+      retval += wxT("\\tab\n");
+    }
+    else
+    {
+      if(m_previous != NULL)
+        retval = wxT("\\par}\n{\\pard\\s21\\li1105\\lin1105\\f0\\fs24 ");
+      else
+        retval = wxT("\\pard\\s21\\li1105\\lin1105\\f0\\fs24 ");        
+    }
+  }
+  else
+    retval = wxT("\\par}\n{");
+
+  if(GetEditable() != NULL)
+    retval += GetEditable()->ToRTF();
+
+  MathCell *out = GetLabel();
+  if(out != NULL)
+  {
+    retval += out->ListToRTF(true);
+  }
+  return retval;
+}
 
 wxString GroupCell::ToTeX(wxString imgDir, wxString filename, int *imgCounter)
 {
+  wxASSERT_MSG((imgCounter != NULL),_("Bug: No image counter to write to!"));
+  if (imgCounter == NULL) return wxEmptyString;
   wxString str;
-  bool SuppressLeadingNewlines = true;
   // Now we might want to introduce some markdown:
-  MarkDownTeX MarkDownParser;
+  MarkDownTeX MarkDown;
 
   switch (m_groupType)
   {
   case GC_TYPE_PAGEBREAK:
     str = wxT("\\pagebreak\n");
-    SuppressLeadingNewlines = true;
     break;
 
   case GC_TYPE_IMAGE:
@@ -748,7 +900,7 @@ wxString GroupCell::ToTeX(wxString imgDir, wxString filename, int *imgCounter)
           str = str.Mid(5, str.Length());
         }
         else {
-          str = MarkDownParser.MarkDown(str);
+          str = MarkDown.MarkDown(str);
         }
         break;
       }
@@ -762,11 +914,9 @@ wxString GroupCell::ToTeX(wxString imgDir, wxString filename, int *imgCounter)
 wxString GroupCell::ToTeXCodeCell(wxString imgDir, wxString filename, int *imgCounter)
 {
   wxString str;
-  bool exportInput = true;
-  wxConfig::Get()->Read(wxT("exportInput"), &exportInput);
 
   // Input cells
-  if(exportInput)
+  if(Configuration::Get()->ShowCodeCells())
   {
     str = wxT("\n\n\\noindent\n%%%%%%%%%%%%%%%\n")
       wxT("%%% INPUT:\n")
@@ -854,6 +1004,9 @@ wxString GroupCell::ToTeXCodeCell(wxString imgDir, wxString filename, int *imgCo
 
 wxString GroupCell::ToTeXImage(MathCell *tmp, wxString imgDir, wxString filename, int *imgCounter)
 {
+  wxASSERT_MSG((imgCounter != NULL),_("Bug: No image counter to write to!"));
+  if (imgCounter == NULL) return wxEmptyString;
+
   wxString str;
   
   if (imgDir != wxEmptyString)
@@ -1001,10 +1154,19 @@ wxString GroupCell::ToXML()
 void GroupCell::SelectRectGroup(wxRect& rect, wxPoint& one, wxPoint& two,
     MathCell **first, MathCell **last)
 {
+  Configuration *configuration = Configuration::Get();
+
   *first = NULL;
   *last = NULL;
 
-  if ((m_input) && (m_input->ContainsRect(rect)))
+    
+  if ((m_input) &&
+      (
+        (configuration->ShowCodeCells()) ||
+        (m_groupType != GC_TYPE_CODE)
+        ) &&
+      (m_input->ContainsRect(rect))
+    )
     m_input->SelectRect(rect, first, last);
   else if (m_output != NULL && !m_hide && m_outputRect.Contains(rect))
     SelectRectInOutput(rect, one, two, first, last);
@@ -1109,7 +1271,8 @@ void GroupCell::SelectRectInOutput(wxRect& rect, wxPoint& one, wxPoint& two,
 
 bool GroupCell::SetEditableContent(wxString text)
 {
-  if (GetEditable()) {
+  if (GetEditable())
+  {
     GetEditable()->SetValue(text);
     return true;
   }
@@ -1119,18 +1282,19 @@ bool GroupCell::SetEditableContent(wxString text)
 
 EditorCell *GroupCell::GetEditable()
 {
-  switch (m_groupType) {
-    case GC_TYPE_CODE:
-    case GC_TYPE_IMAGE:
-    case GC_TYPE_TEXT:
-    case GC_TYPE_TITLE:
-    case GC_TYPE_SECTION:
-    case GC_TYPE_SUBSECTION:
-    case GC_TYPE_SUBSUBSECTION:
-      return GetInput();
-    case GC_TYPE_PAGEBREAK:
-    default:
-      return NULL;
+  switch (m_groupType)
+  {
+  case GC_TYPE_PAGEBREAK:
+    return NULL;
+  case GC_TYPE_CODE:
+  case GC_TYPE_IMAGE:
+  case GC_TYPE_TEXT:
+  case GC_TYPE_TITLE:
+  case GC_TYPE_SECTION:
+  case GC_TYPE_SUBSECTION:
+  case GC_TYPE_SUBSUBSECTION:
+  default:
+    return GetInput();
   }
 }
 
@@ -1183,20 +1347,20 @@ void GroupCell::SelectOutput(MathCell **start, MathCell **end)
     *end = *start = NULL;
 }
 
-void GroupCell::BreakUpCells(CellParser parser, int fontsize, int clientWidth)
+void GroupCell::BreakUpCells(int fontsize, int clientWidth)
 {
-  BreakUpCells(m_output, parser, fontsize, clientWidth);
+  BreakUpCells(m_output, fontsize, clientWidth);
 }
 
-void GroupCell::BreakUpCells(MathCell *cell, CellParser parser, int fontsize, int clientWidth)
+void GroupCell::BreakUpCells(MathCell *cell, int fontsize, int clientWidth)
 {
   MathCell *tmp = cell;
 
   while (tmp != NULL && !m_hide) {
     if (tmp->GetWidth() > clientWidth) {
       if (tmp->BreakUp()) {
-        tmp->RecalculateWidths(parser,  tmp->IsMath() ? m_mathFontSize : m_fontSize);
-        tmp->RecalculateSize(parser,  tmp->IsMath() ? m_mathFontSize : m_fontSize);
+        tmp->RecalculateWidths(tmp->IsMath() ? m_mathFontSize : m_fontSize);
+        tmp->RecalculateHeight(tmp->IsMath() ? m_mathFontSize : m_fontSize);
       }
     }
     tmp = tmp->m_nextToDraw;
@@ -1228,7 +1392,8 @@ void GroupCell::Hide(bool hide) {
     GetEditable()->SetFirstLineOnly(m_hide);
 
   // Don't keep cached versions of scaled images around if they aren't visible at all.
-  GetLabel()->ClearCacheList();
+  if(GetLabel())
+    GetLabel()->ClearCacheList();
 
   ResetSize();
   GetEditable()->ResetSize();
@@ -1252,7 +1417,8 @@ bool GroupCell::HideTree(GroupCell *tree)
   GroupCell *tmp = m_hiddenTree;
   while(tmp)
   {
-    tmp->GetLabel()->ClearCacheList();
+    if(tmp->GetLabel())
+      tmp->GetLabel()->ClearCacheList();
     tmp = dynamic_cast<GroupCell *>(tmp->m_next);
   }
   
@@ -1307,8 +1473,10 @@ GroupCell *GroupCell::Fold() {
   GroupCell *end = dynamic_cast<GroupCell*>(m_next);
   GroupCell *start = end; // first to fold
 
-  while (end) {
-    end->GetLabel()->ClearCacheList();
+  while (end)
+  {
+    if(end->GetLabel())
+      end->GetLabel()->ClearCacheList();
  
     GroupCell *tmp = dynamic_cast<GroupCell*>(end->m_next);
     if (tmp == NULL)
@@ -1431,8 +1599,13 @@ bool GroupCell::IsLesserGCType(int comparedTo) {
   }
 }
 
-void GroupCell::Number(int &section, int &subsection, int &subsubsection, int &image) {
-  switch (m_groupType) {
+void GroupCell::Number(int &section, int &subsection, int &subsubsection, int &image)
+{
+  GroupCell *tmp = this;
+
+  while(tmp != NULL)
+  {
+    switch (tmp->m_groupType) {
     case GC_TYPE_TITLE:
       section = subsection = subsubsection = 0;
       break;
@@ -1442,7 +1615,7 @@ void GroupCell::Number(int &section, int &subsection, int &subsubsection, int &i
       {
         wxString num = wxT(" ");
         num << section << wxT(" ");
-        ((TextCell*)m_input)->SetValue(num);
+        ((TextCell*)tmp->m_input)->SetValue(num);
       }
       break;
     case GC_TYPE_SUBSECTION:
@@ -1451,7 +1624,7 @@ void GroupCell::Number(int &section, int &subsection, int &subsubsection, int &i
       {
         wxString num = wxT("  ");
         num << section << wxT(".") << subsection << wxT(" ");
-        ((TextCell*)m_input)->SetValue(num);
+        ((TextCell*)tmp->m_input)->SetValue(num);
       }
       break;
     case GC_TYPE_SUBSUBSECTION:
@@ -1459,25 +1632,25 @@ void GroupCell::Number(int &section, int &subsection, int &subsubsection, int &i
       {
         wxString num = wxT("  ");
         num << section << wxT(".") << subsection << wxT(".") << subsubsection << wxT(" ");
-        ((TextCell*)m_input)->SetValue(num);
+        ((TextCell*)tmp->m_input)->SetValue(num);
       }
       break;
     case GC_TYPE_IMAGE:
       image++;
       {
         wxString num = wxString::Format(_("Figure %d:"), image);
-        ((TextCell*)m_input)->SetValue(num);
+        ((TextCell*)tmp->m_input)->SetValue(num);
       }
       break;
     default:
       break;
+    }
+
+    if (IsFoldable() && tmp->m_hiddenTree)
+      tmp->m_hiddenTree->Number(section, subsection, subsubsection, image);
+
+    tmp = dynamic_cast<GroupCell*>(tmp->m_next);
   }
-
-  if (IsFoldable() && m_hiddenTree)
-    m_hiddenTree->Number(section, subsection, subsubsection, image);
-
-  if (m_next)
-    dynamic_cast<GroupCell*>(m_next)->Number(section, subsection, subsubsection, image);
 }
 
 bool GroupCell::IsMainInput(MathCell *active)
@@ -1493,21 +1666,22 @@ bool GroupCell::Contains(GroupCell *cell)
   GroupCell *tmp = this;
 
   // Iterate through all cells
-  while(tmp)
+  while(tmp != NULL)
   {
     // If this is the cell we search for we can end the search.
     if(tmp == cell)
       return true;
 
     // If this cell contains a hidden tree we have to search that at well.
-    if ((tmp->IsFoldable()) && (tmp->GetHiddenTree()) )
-      if (this->GetHiddenTree()->Contains(cell))
+    if ((tmp->IsFoldable()) && (tmp->GetHiddenTree()) != NULL)
+    {
+      if (tmp->GetHiddenTree()->Contains(cell))
         return true;
-
+    }
+    
     // Step to the next cell.
-    tmp= dynamic_cast<GroupCell*>(tmp->m_next);
+    tmp=dynamic_cast<GroupCell*>(tmp->m_next);
   }
   
   return false;
 }
-
