@@ -1,7 +1,7 @@
 // -*- mode: c++; c-file-style: "linux"; c-basic-offset: 2; indent-tabs-mode: nil -*-
 //
 //  Copyright (C) 2004-2015 Andrej Vodopivec <andrej.vodopivec@gmail.com>
-//            (C) 2014-2015 Gunter Königsmann <wxMaxima@physikbuch.de>
+//            (C) 2014-2016 Gunter Königsmann <wxMaxima@physikbuch.de>
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
@@ -19,6 +19,13 @@
 //  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 //
 
+/*! \file
+  This file defines the class ImgCell
+
+  ImgCell is the MathCell type that represents still images in maxima's output
+  or in user-provided images.
+ */
+
 #include "ImgCell.h"
 
 #include <wx/file.h>
@@ -26,6 +33,7 @@
 #include <wx/filesys.h>
 #include <wx/fs_mem.h>
 #include <wx/clipbrd.h>
+#include <wx/mstream.h>
 
 ImgCell::ImgCell() : MathCell()
 {
@@ -33,6 +41,16 @@ ImgCell::ImgCell() : MathCell()
   m_type = MC_TYPE_IMAGE;
   m_drawRectangle = true;
   m_imageBorderWidth = 1;
+  m_drawBoundingBox = false;
+}
+
+ImgCell::ImgCell(wxMemoryBuffer image,wxString type) : MathCell()
+{
+  m_image = new Image(image,type);
+  m_type = MC_TYPE_IMAGE;
+  m_drawRectangle = true;
+  m_imageBorderWidth = 1;
+  m_drawBoundingBox = false;
 }
 
 ImgCell::ImgCell(const wxBitmap &bitmap) : MathCell()
@@ -41,6 +59,7 @@ ImgCell::ImgCell(const wxBitmap &bitmap) : MathCell()
   m_type = MC_TYPE_IMAGE;
   m_drawRectangle = true;
   m_imageBorderWidth = 1;
+  m_drawBoundingBox = false;
 }
 
 int ImgCell::s_counter = 0;
@@ -54,6 +73,7 @@ ImgCell::ImgCell(wxString image, bool remove, wxFileSystem *filesystem) : MathCe
     m_image = new Image(image,remove,filesystem);
   else
     m_image = new Image();
+  m_drawBoundingBox = false;
 }
 
 ImgCell::~ImgCell()
@@ -96,54 +116,66 @@ void ImgCell::Destroy()
   m_next = NULL;
 }
 
-void ImgCell::RecalculateWidths(CellParser& parser, int fontsize)
+void ImgCell::RecalculateWidths(int fontsize)
 {
-  double scale = parser.GetScale();
+  // Here we recalculate the height, as well:
+  //  - This doesn't cost much time and
+  //  - as image cell's sizes might change when the resolution does
+  //    we might have intermittent calculation issues otherwise
+  Configuration *configuration = Configuration::Get();
+  double scale = configuration->GetScale();
   m_image->ViewportSize(m_canvasSize.x,m_canvasSize.y,scale);
   
-  m_width = (scale * m_image->m_width) + 2 * m_imageBorderWidth;
-  ResetData();
-}
-
-void ImgCell::RecalculateSize(CellParser& parser, int fontsize)
-{
-  double scale = parser.GetScale();
-  m_image->ViewportSize(m_canvasSize.x,m_canvasSize.y,scale);
-  
-  m_height = (scale * m_image->m_height) + 2 * m_imageBorderWidth;
-  ResetData();
-
+  m_width  = m_image->m_width  + 2 * m_imageBorderWidth;
+  m_height = m_image->m_height + 2 * m_imageBorderWidth;
   m_center = m_height / 2;
 }
 
-void ImgCell::Draw(CellParser& parser, wxPoint point, int fontsize)
+void ImgCell::RecalculateHeight(int fontsize)
 {
-  wxDC& dc = parser.GetDC();
+  // Here we recalculate the width, as well:
+  //  - This doesn't cost much time and
+  //  - as image cell's sizes might change when the resolution does
+  //    we might have intermittent calculation issues otherwise
+  RecalculateWidths(fontsize);
+}
 
-  if (DrawThisCell(parser, point) && m_image != NULL)
+void ImgCell::Draw(wxPoint point, int fontsize)
+{
+  MathCell::Draw(point, fontsize);
+  m_image->ViewportSize(m_canvasSize.x,m_canvasSize.y,Configuration::Get()->GetScale());
+
+  // TODO: Enable this when unselecting text updates the right region.
+  //if (!InUpdateRegion()) return;
+
+  Configuration *configuration = Configuration::Get();
+  wxDC& dc = configuration->GetDC();
+  if (DrawThisCell(point) && (m_image != NULL))
   {
     wxMemoryDC bitmapDC;
-    double scale = parser.GetScale();
-    m_image->ViewportSize(m_canvasSize.x,m_canvasSize.y,scale);
-  
-    m_height = (m_image->m_height) + 2 * m_imageBorderWidth;
-    m_width  = (m_image->m_width)  + 2 * m_imageBorderWidth;
-
-    SetPen(parser);
-    if (m_drawRectangle)
-      
-      dc.DrawRectangle(wxRect(point.x, point.y - m_center, m_width, m_height));  
 
+    if(m_drawBoundingBox)
+      dc.SetBrush( *(wxTheBrushList->FindOrCreateBrush(configuration->GetColor(TS_SELECTION))));
+    else
+      SetPen();
+    
+    if (m_drawRectangle || m_drawBoundingBox)
+      dc.DrawRectangle(wxRect(point.x, point.y - m_center, m_width, m_height));
+    
     wxBitmap bitmap = m_image->GetBitmap();
     bitmapDC.SelectObject(bitmap);
-      
-    dc.Blit(point.x + m_imageBorderWidth, point.y - m_center + m_imageBorderWidth, m_width - 2 * m_imageBorderWidth, m_height - 2 * m_imageBorderWidth, &bitmapDC, 0, 0);
+
+    if ((m_drawBoundingBox == false) || (m_imageBorderWidth > 0))
+      dc.Blit(point.x + m_imageBorderWidth, point.y - m_center + m_imageBorderWidth, m_width - 2 * m_imageBorderWidth, m_height - 2 * m_imageBorderWidth, &bitmapDC, 0, 0);
+    else
+      dc.StretchBlit(point.x + 5, point.y - m_center + 5, m_width - 2 * 5, m_height - 2 * 5, &bitmapDC, 0, 0, m_width, m_height);
   }
   else
     // The cell isn't drawn => No need to keep it's image cache for now.
     ClearCache();
-
-  MathCell::Draw(parser, point, fontsize);
+  
+  // The next time we need to draw a bounding box we will be informed again.
+  m_drawBoundingBox = false;
 }
 
 wxString ImgCell::ToString()
@@ -161,6 +193,50 @@ wxSize ImgCell::ToImageFile(wxString file)
   return m_image->ToImageFile(file);
 }
 
+wxString ImgCell::ToRTF()
+{
+  // Lines that are common to all types of images
+  wxString header=wxT("{\\pict");
+  wxString footer=wxT("}\n");
+  
+  // Extract the description of the image data
+  wxString image;
+  wxMemoryBuffer imgdata;
+  if(m_image->GetExtension().Lower() == wxT("png"))
+  {
+    imgdata = GetCompressedImage();
+    image=wxT("\\pngblip");
+  } else if(
+    (m_image->GetExtension().Lower() == wxT("jpg"))||
+    (m_image->GetExtension().Lower() == wxT("jpeg"))
+    )
+  {
+    imgdata = GetCompressedImage();
+    image=wxT("\\jpegblip");
+  }
+    else
+    {
+      // Convert any non-rtf-enabled format to .png before adding it to the .rtf file.
+      image = wxT("\\pngblip");
+      wxImage imagedata = m_image->GetUnscaledBitmap().ConvertToImage();
+      wxMemoryOutputStream stream;
+      imagedata.SaveFile(stream,wxBITMAP_TYPE_PNG);
+      imgdata.AppendData(stream.GetOutputStreamBuffer()->GetBufferStart(),
+                         stream.GetOutputStreamBuffer()->GetBufferSize());
+    }
+
+  image +=wxString::Format(wxT("\\picw%li\\pich%li "),
+                           m_image->GetOriginalWidth(),
+                           m_image->GetOriginalHeight()
+    );
+  
+  // Convert the data into a hexadecimal string
+  for(size_t i=0;i<= imgdata.GetDataLen();i++)
+    image += wxString::Format("%02x",((unsigned char *)imgdata.GetData())[i]);
+
+  return header+image+footer;
+}
+
 wxString ImgCell::ToXML()
 {
   wxString basename = ImgCell::WXMXGetNewFileName();
