@@ -19,6 +19,12 @@
 //  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 //
 
+/*! \file
+  This file declares the class LimitCell
+
+  LimitCell is the MathCell type that represents maxima's <code>limit()</code> command.
+*/
+
 #ifndef LIMITCELL_H
 #define LIMITCELL_H
 
@@ -31,15 +37,16 @@ public:
   ~LimitCell();
   void Destroy();
   MathCell* Copy();
-  void RecalculateSize(CellParser& parser, int fontsize);
-  void RecalculateWidths(CellParser& parser, int fontsize);
-  void Draw(CellParser& parser, wxPoint point, int fontsize);
+  void RecalculateHeight(int fontsize);
+  void RecalculateWidths(int fontsize);
+  void Draw(wxPoint point, int fontsize);
   void SetBase(MathCell* base);
   void SetUnder(MathCell* under);
   void SetName(MathCell* name);
   wxString ToString();
   wxString ToTeX();
   wxString ToXML();
+  wxString ToOMML();
   wxString ToMathML();
   void SelectInner(wxRect& rect, MathCell** first, MathCell** last);
   void SetParent(MathCell *parent);
