@@ -1,6 +1,6 @@
 // -*- mode: c++; c-file-style: "linux"; c-basic-offset: 2; indent-tabs-mode: nil -*-
 //
-//  Copyright (C) 2015      Gunter Königsmann <wxMaxima@physikbuch.de>
+//  Copyright (C) 2015-2016 Gunter Königsmann <wxMaxima@physikbuch.de>
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
@@ -18,6 +18,11 @@
 //  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 //
 
+/*!\file This file contains the code for MarkDownParser.
+
+MarkDownParser is the class that handles the markdown syntax
+*/
+
 #ifndef MARKDOWN_H
 #define MARKDOWN_H
 
@@ -62,9 +67,11 @@ public:
   //! A list of things we want to replace.
   std::list<RegexReplacer *> RegexReplaceList(){return regexReplaceList;}
  private:    
-  bool m_flowedTextRequested;             //!< For HTML: Do we want to pass all newlines to the output?
   virtual wxString itemizeBegin()=0;      //!< The marker for the begin of an item list
   virtual wxString itemizeEnd()=0;        //!< The marker for the end of an item list
+  virtual wxString quoteChar()=0;         //!< The marker for an quote
+  virtual wxString quoteBegin()=0;        //!< The marker that says we want to start quote
+  virtual wxString quoteEnd  ()=0;        //!< The marker that says we want to end quote
   virtual wxString itemizeItem()=0;       //!< The marker for the begin of an item
   virtual wxString itemizeEndItem()=0;    //!< The marker for the end of an item
   virtual wxString NewLine()=0;           //!< The marker for the beginning of a new line
@@ -77,6 +84,9 @@ class MarkDownTeX: public MarkDownParser
 public:
   MarkDownTeX();
  private:
+  virtual wxString quoteBegin(){return wxT("\\begin{quote}\n");}
+  virtual wxString quoteEnd  (){return wxT("\\end{quote}\n");}
+  virtual wxString quoteChar (){return wxT("\\ensuremath{>}");}
   virtual wxString itemizeBegin(){return wxT("\\begin{itemize}\n");}
   virtual wxString itemizeEnd(){return wxT("\\end{itemize}\n");}
   virtual wxString itemizeItem(){return wxT("\\item ");}
@@ -91,6 +101,9 @@ class MarkDownHTML: public MarkDownParser
 public:
   MarkDownHTML();
  private:
+  virtual wxString quoteChar (){return wxT("&gt;");}
+  virtual wxString quoteBegin(){return wxT("<blockquote>\n");}
+  virtual wxString quoteEnd  (){return wxT("</blockquote>\n");}
   virtual wxString itemizeBegin(){return wxT("<UL>\n");}
   virtual wxString itemizeEnd(){return wxT("</UL>\n");}
   virtual wxString itemizeItem(){return wxT("<LI>");}
