@@ -19,7 +19,15 @@
 //  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 //
 
+/*! \file
+  This file defines the class MathCell
+
+  MathCell is the base class for all cell- or list-type elements.
+*/
+
 #include "MathCell.h"
+#include <wx/regex.h>
+#include <wx/sstream.h>
 
 MathCell::MathCell()
 {
@@ -96,6 +104,7 @@ void MathCell::SetType(int type)
     m_textStyle = TS_DEFAULT;
     break;
   }
+  ResetSize();
 }
 
 MathCell *MathCell::CopyList()
@@ -199,8 +208,6 @@ int MathCell::GetMaxCenter()
 
 /***
  * Get the maximum drop of cell.
-
-\todo Convert this function to not using recursive function calls any more.
  */
 int MathCell::GetMaxDrop()
 {
@@ -228,8 +235,6 @@ int MathCell::GetMaxHeight()
 }
 
 /*! Get full width of this group.
-
-  \todo Change not to use recursive function calls any more.
  */
 int MathCell::GetFullWidth(double scale)
 {
@@ -275,31 +280,55 @@ int MathCell::GetLineWidth(double scale)
  To make this work each derived class must draw the content of the cell
  and then call MathCall::Draw(...).
  */
-void MathCell::Draw(CellParser& parser, wxPoint point, int fontsize)
+void MathCell::Draw(wxPoint point, int fontsize)
 {
-  m_currentPoint.x = point.x;
-  m_currentPoint.y = point.y;
+  m_currentPoint = point;
 }
 
-void MathCell::DrawList(CellParser& parser, wxPoint point, int fontsize)
+void MathCell::DrawList(wxPoint point, int fontsize)
 {
+  Configuration *configuration = Configuration::Get();
   MathCell *tmp=this;
   while(tmp!=NULL)
   {
-    tmp->Draw(parser,point,fontsize);
-    double scale = parser.GetScale();
+    tmp->Draw(point,fontsize);
+    double scale = configuration->GetScale();
     point.x += tmp->m_width + SCALE_PX(MC_CELL_SKIP, scale);
     tmp=tmp->m_nextToDraw;
   }
 }
 
-void MathCell::RecalculateSizeList(CellParser& parser, int fontsize)
+void MathCell::RecalculateList(int fontsize)
 {
   MathCell *tmp=this;
 
   while(tmp!=NULL)
     {
-      tmp->RecalculateSize(parser, fontsize);
+      tmp->RecalculateWidths(fontsize);
+      tmp->RecalculateHeight(fontsize);
+      tmp=tmp->m_next;
+    }  
+}
+
+void MathCell::ResetSizeList()
+{
+  MathCell *tmp=this;
+
+  while(tmp!=NULL)
+    {
+      tmp->ResetSize();
+      tmp=tmp->m_next;
+    }
+}
+
+
+void MathCell::RecalculateHeightList(int fontsize)
+{
+  MathCell *tmp=this;
+
+  while(tmp!=NULL)
+    {
+      tmp->RecalculateHeight(fontsize);
       tmp=tmp->m_next;
     }  
 }
@@ -311,28 +340,29 @@ void MathCell::RecalculateSizeList(CellParser& parser, int fontsize)
   
   Should set: set m_width.
 */
-void MathCell::RecalculateWidthsList(CellParser& parser, int fontsize)
+void MathCell::RecalculateWidthsList(int fontsize)
 {
   MathCell *tmp=this;
 
   while(tmp!=NULL)
     {
-      tmp->RecalculateWidths(parser, fontsize);
+      tmp->RecalculateWidths(fontsize);
       tmp=tmp->m_next;
     }
 }
 
-void MathCell::RecalculateWidths(CellParser& parser, int fontsize)
+void MathCell::RecalculateWidths(int fontsize)
 {
   ResetData();
 }
 
 /*! Is this cell currently visible in the window?.
  */
-bool MathCell::DrawThisCell(CellParser& parser, wxPoint point)
+bool MathCell::DrawThisCell(wxPoint point)
 {
-  int top = parser.GetTop();
-  int bottom = parser.GetBottom();
+  Configuration *configuration = Configuration::Get();
+  int top = configuration->GetTop();
+  int bottom = configuration->GetBottom();
   if (top == -1 || bottom == -1)
     return true;
   if (point.y - GetMaxCenter() > bottom || point.y + GetMaxDrop() < top)
@@ -357,16 +387,40 @@ wxRect MathCell::GetRect(bool all)
                 m_width, m_height);
 }
 
-/***
- * Draws a box around this cell - if all is true draws a box around the whole
- * line.
- */
-void MathCell::DrawBoundingBox(wxDC& dc, bool all, int border)
+bool MathCell::InUpdateRegion(const wxRect &rect)
+{
+  if (m_clipToDrawRegion) return true;
+  if (rect.GetLeft()   > m_updateRegion.GetRight())  return false;
+  if (rect.GetRight()  < m_updateRegion.GetLeft())   return false;
+  if (rect.GetBottom() < m_updateRegion.GetTop())    return false;
+  if (rect.GetTop()    > m_updateRegion.GetBottom()) return false;
+  return true;
+}
+
+wxRect MathCell::CropToUpdateRegion(const wxRect &rect)
+{
+  if(m_clipToDrawRegion) return rect;
+  
+  int left  =rect.GetLeft();
+  int top   =rect.GetTop ();
+  int right =rect.GetRight();
+  int bottom=rect.GetBottom();
+  if (left   < m_updateRegion.GetLeft())   left   = m_updateRegion.GetLeft();
+  if (right  > m_updateRegion.GetRight())  right  = m_updateRegion.GetRight();
+  if (top    < m_updateRegion.GetTop())    top    = m_updateRegion.GetTop();
+  if (bottom > m_updateRegion.GetBottom()) bottom = m_updateRegion.GetBottom();
+
+  // Windows seems to utterly dislike rectangles with the width or height 0.
+  if(bottom == top) bottom++;
+  if(left == right) right++;
+  return wxRect(wxPoint(left,top),wxPoint(right,bottom));
+}
+
+void MathCell::DrawBoundingBox(wxDC& dc, bool all)
 {
   wxRect rect = GetRect(all);
-  int x = rect.GetX(), y = rect.GetY();
-  int width = rect.GetWidth(), height = rect.GetHeight();
-  dc.DrawRectangle(x - border, y - border, width + 2*border, height + 2*border);
+  if(InUpdateRegion())
+    dc.DrawRectangle(CropToUpdateRegion(rect));
 }
 
 /***
@@ -404,16 +458,18 @@ wxString MathCell::ListToString()
   bool firstline = true;
   
   while(tmp!=NULL)
-    {
-      retval+=tmp->ToString();
-      if((!firstline)&&(tmp->m_forceBreakLine))
-        retval+=wxT("\n");
+  {
+    retval += tmp->ToString();
 
-      firstline = false;
-      tmp=tmp->m_next;
-    }
+    
+    if((!firstline)&&(tmp->m_forceBreakLine))
+      retval += wxT("\n");
+    
+    firstline = false;
+    tmp = tmp->m_nextToDraw;
+  }
   
-  return retval;
+ return retval;
 }
 
 wxString MathCell::ToTeX()
@@ -434,7 +490,14 @@ wxString MathCell::ListToTeX()
     retval += tmp->ToTeX();
     tmp = tmp->m_next;
   }
-  
+
+  // TODO: Things like {a}_{b} make the LaTeX code harder to read. But things like
+  // \sqrt{a} need us to use braces from time to time.
+  //
+  // How far I got was:
+  //
+  //  wxRegEx removeUnneededBraces1(wxT("{([a-zA-Z0-9])}([{}_a-zA-Z0-9 \\\\^_])"));
+  //  removeUnneededBraces1.Replace(&retval,wxT(" \\1\\2"),true);
   return retval;
 }
 
@@ -513,6 +576,215 @@ wxString MathCell::ListToMathML(bool startofline)
   return retval;
 }
 
+wxString MathCell::OMML2RTF(wxXmlNode *node)
+{
+  wxString result;
+
+  while(node != NULL)
+  {
+    if (node->GetType() == wxXML_ELEMENT_NODE)
+    {
+      wxString ommlname = node->GetName();
+      result += wxT("{\\m")+ommlname.Right(ommlname.Length()-2);
+
+      // Convert the attributes
+      wxXmlAttribute* attributes = node->GetAttributes();
+      while(attributes != NULL)
+      {
+        wxString ommlatt = attributes -> GetName();
+        result += wxT("{\\m")+ommlatt.Right(ommlatt.Length()-2) +
+          wxT(" ")+attributes->GetValue()+wxT("}");
+        attributes = attributes -> GetNext();
+      }
+
+      // Convert all child nodes
+      if(node->GetChildren() != NULL)
+      {
+        result += OMML2RTF(node->GetChildren());
+      }
+      result += wxT("}");
+    }
+    else
+      result += wxT(" ")+RTFescape(node->GetContent());
+
+    node = node->GetNext();
+  }
+  return result;
+}
+  
+wxString MathCell::OMML2RTF(wxString ommltext)
+{
+  if(ommltext == wxEmptyString)
+    return wxEmptyString;
+  
+  wxString result;
+  wxXmlDocument ommldoc;
+  ommltext = wxT("<m:r>")+ommltext+wxT("</m:r>");
+
+  wxStringInputStream ommlStream(ommltext);
+  
+  ommldoc.Load(ommlStream,wxT("UTF-8"));
+
+  wxXmlNode *node = ommldoc.GetRoot();
+  result += OMML2RTF(node);
+
+  if((result != wxEmptyString) && (result != wxT("\\mr")))
+  {
+    result = wxT("{\\mmath {\\*\\moMath") + result + wxT("}}");
+  }
+  return result;
+}
+
+wxString MathCell::XMLescape(wxString input)
+{
+  input.Replace(wxT("&"),  wxT("&amp;"));
+  input.Replace(wxT("<"),  wxT("&lt;"));
+  input.Replace(wxT(">"),  wxT("&gt;"));
+  input.Replace(wxT("'"),  wxT("&apos;"));
+  input.Replace(wxT("\""), wxT("&quot;"));
+  return input;
+}
+
+wxString MathCell::RTFescape(wxString input,bool MarkDown)
+{
+  // Characters with a special meaning in RTF
+  input.Replace("\\","\\\\");
+  input.Replace("{","\\{");
+  input.Replace("}","\\}");
+
+  // The Character we will use as a soft line break
+  input.Replace("\r",wxEmptyString);
+
+  // Encode unicode characters in a rather mind-boggling way
+  wxString output;
+  for(size_t i=0;i<input.Length();i++)
+  {
+    wxChar ch = input[i];
+    if (ch == wxT('\n'))
+    {
+      if(((i>0) && (input[i-1] == wxT('\n'))) || !MarkDown)
+        output += wxT("\\par}\n{\\pard ");
+      else
+        output += wxT("\n");
+    }
+    else
+    {
+      if((ch < 128) && (ch > 0))
+      {
+        output += ch;
+      }
+      else
+      {
+        if(ch < 32768)
+        {
+          output += wxString::Format("\\u%i?",int(ch));
+        }
+        else
+        {
+          output += wxString::Format("\\u%i?",int(ch)-65536);
+        }
+      }
+    }
+  }
+  return(output);
+}
+
+wxString MathCell::ListToOMML(bool startofline)
+{
+  bool multiCell = (m_next != NULL);
+
+  wxString retval;
+
+  // If the region to export contains linebreaks or labels we put it into a table.
+  // Export all cells
+
+  MathCell *tmp = this;
+  while(tmp!=NULL)
+  {
+    wxString token = tmp->ToOMML();
+
+    // End exporting the equation if we reached the end of the equation.
+    if(token == wxEmptyString)
+      break;
+    
+    retval += token;
+
+    // Hard linebreaks aren't supported by OMML and therefore need a new equation object
+    if(tmp->ForceBreakLineHere())
+      break;
+    
+    tmp=tmp->m_next;
+  }
+
+  if((multiCell) && (retval != wxEmptyString))
+    return wxT("<m:r>")+retval+wxT("</m:r>");
+  else
+    return retval;
+}
+
+wxString MathCell::ListToRTF(bool startofline)
+{
+  wxString retval;
+  MathCell *tmp = this;
+  
+  while(tmp != NULL)
+  {    
+    wxString rtf = tmp->ToRTF();
+    if(rtf != wxEmptyString)
+    {
+      if((GetStyle() == TS_LABEL) || ((GetStyle() == TS_USERLABEL)))
+      {
+        retval += wxT("\\par}\n{\\pard\\s22\\li1105\\lin1105\\fi-1105\\f0\\fs24 ") + rtf + wxT("\\tab");
+        startofline = false;
+      }
+      else 
+      {
+        if(startofline)
+          retval += wxT("\\par}\n{\\pard\\s21\\li1105\\lin1105\\f0\\fs24 ") + rtf+wxT("\\n");
+        startofline = true;
+      }
+      tmp = tmp->m_next;
+    }
+    else
+    {
+      if(tmp->ListToOMML() != wxEmptyString)
+      {
+        // Math!
+
+        // set the style for this line.
+        if(startofline)
+          retval += wxT("\\pard\\s21\\li1105\\lin1105\\f0\\fs24 ");
+        
+        retval += OMML2RTF(tmp->ListToOMML());
+        
+        startofline = true;
+        
+        // Skip the rest of this equation
+        while(tmp != NULL)
+        {
+          // A non-equation item starts a new rtf item
+          if (tmp->ToOMML() == wxEmptyString)
+            break;
+          
+          // A newline starts a new equation
+        if(tmp->ForceBreakLineHere())
+        {
+          tmp = tmp->m_next;
+          break;
+        }
+        
+        tmp = tmp->m_next;
+        }
+      }
+      else
+      {
+        tmp = tmp->m_next;
+      }
+    }
+  }
+  return retval;
+}
+
 wxString MathCell::ListToXML()
 {
   bool highlight=false;
@@ -609,7 +881,7 @@ bool MathCell::BreakLineHere()
   return (((!m_isBroken) && m_breakLine) || m_forceBreakLine);
 }
 
-bool MathCell::ContainsRect(wxRect& sm, bool all)
+bool MathCell::ContainsRect(const wxRect& sm, bool all)
 {
   wxRect big = GetRect(all);
   if (big.x <= sm.x &&
@@ -686,41 +958,58 @@ void MathCell::DestroyList()
     tmp = next;
   }
 }
-/***
- * Set the pen in device context accordint to the style of the cell.
- */
-void MathCell::SetPen(CellParser& parser)
+
+int MathCell::GetDefaultLineWidth()
 {
-  wxDC& dc = parser.GetDC();
-  if (m_highlight)
-    dc.SetPen(*(wxThePenList->FindOrCreatePen(parser.GetColor(TS_HIGHLIGHT),
-                1, wxPENSTYLE_SOLID)));
-  else if (m_type == MC_TYPE_PROMPT)
-    dc.SetPen(*(wxThePenList->FindOrCreatePen(parser.GetColor(TS_OTHER_PROMPT),
-                1, wxPENSTYLE_SOLID)));
-  else if (m_type == MC_TYPE_INPUT)
-    dc.SetPen(*(wxThePenList->FindOrCreatePen(parser.GetColor(TS_INPUT),
-                1, wxPENSTYLE_SOLID)));
+  int linewidth;
+  if(Printing())
+    linewidth = Configuration::Get()->GetScale();
   else
-    dc.SetPen(*(wxThePenList->FindOrCreatePen(parser.GetColor(TS_DEFAULT),
-                    1, wxPENSTYLE_SOLID)));
+    linewidth = Configuration::Get()->GetZoomFactor();
+  if (linewidth < 1)
+    linewidth = 1;
+
+  return linewidth;
+}
+
+/*!
+  Set the pen in device context according to the style of the cell.
+*/
+void MathCell::SetPen()
+{
+  Configuration *configuration = Configuration::Get();
+  wxDC& dc = configuration->GetDC();
+  
+  if (m_highlight)
+    dc.SetPen(*(wxThePenList->FindOrCreatePen(configuration->GetColor(TS_HIGHLIGHT),
+                                              GetDefaultLineWidth(), wxPENSTYLE_SOLID)));
+  else if (m_type == MC_TYPE_PROMPT)
+    dc.SetPen(*(wxThePenList->FindOrCreatePen(configuration->GetColor(TS_OTHER_PROMPT),
+                                              GetDefaultLineWidth(), wxPENSTYLE_SOLID)));
+  else if (m_type == MC_TYPE_INPUT)
+    dc.SetPen(*(wxThePenList->FindOrCreatePen(configuration->GetColor(TS_INPUT),
+                                              GetDefaultLineWidth(), wxPENSTYLE_SOLID)));
+  else
+    dc.SetPen(*(wxThePenList->FindOrCreatePen(configuration->GetColor(TS_DEFAULT),
+                                              GetDefaultLineWidth(), wxPENSTYLE_SOLID)));
 }
 
 /***
  * Reset the pen in the device context.
  */
-void MathCell::UnsetPen(CellParser& parser)
+void MathCell::UnsetPen()
 {
-  wxDC& dc = parser.GetDC();
+  Configuration *configuration = Configuration::Get();
+  wxDC& dc = configuration->GetDC();
   if (m_type == MC_TYPE_PROMPT || m_type == MC_TYPE_INPUT || m_highlight)
-    dc.SetPen(*(wxThePenList->FindOrCreatePen(parser.GetColor(TS_DEFAULT),
+    dc.SetPen(*(wxThePenList->FindOrCreatePen(configuration->GetColor(TS_DEFAULT),
                 1, wxPENSTYLE_SOLID)));
 }
 
 /***
  * Copy all important data from s to t
  */
-void MathCell::CopyData(MathCell* s, MathCell* t)
+void MathCell::CopyData(MathCell *s, MathCell *t)
 {
   t->m_altCopyText = s->m_altCopyText;
   t->m_forceBreakLine = s->m_forceBreakLine;
@@ -728,31 +1017,32 @@ void MathCell::CopyData(MathCell* s, MathCell* t)
   t->m_textStyle = s->m_textStyle;
 }
 
-void MathCell::SetForeground(CellParser& parser)
+void MathCell::SetForeground()
 {
+  Configuration *configuration = Configuration::Get();
   wxColour color;
-  wxDC& dc = parser.GetDC();
+  wxDC& dc = configuration->GetDC();
   if (m_highlight)
   {
-    color = parser.GetColor(TS_HIGHLIGHT);
+    color = configuration->GetColor(TS_HIGHLIGHT);
   }
   else {
     switch (m_type)
     {
     case MC_TYPE_PROMPT:
-      color = parser.GetColor(TS_OTHER_PROMPT);
+      color = configuration->GetColor(TS_OTHER_PROMPT);
       break;
     case MC_TYPE_MAIN_PROMPT:
-      color = parser.GetColor(TS_MAIN_PROMPT);
+      color = configuration->GetColor(TS_MAIN_PROMPT);
       break;
     case MC_TYPE_ERROR:
       color = wxColour(wxT("red"));
       break;
     case MC_TYPE_LABEL:
-      color = parser.GetColor(TS_LABEL);
+      color = configuration->GetColor(TS_LABEL);
       break;
     default:
-      color = parser.GetColor(m_textStyle);
+      color = configuration->GetColor(m_textStyle);
       break;
     }
   }
@@ -768,3 +1058,5 @@ bool MathCell::IsMath()
 }
 
 wxSize MathCell::m_canvasSize;
+wxRect MathCell::m_updateRegion;
+bool   MathCell::m_clipToDrawRegion = true;
