@@ -20,6 +20,12 @@
 //  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 //
 
+/*! \file
+  This file defines the class MathCtrl
+
+  MathCtrl represents the worksheet.
+ */
+
 #ifndef MATHCTRL_H
 #define MATHCTRL_H
 
@@ -28,29 +34,146 @@
 #include <wx/textfile.h>
 #include <wx/fdrepdlg.h>
 #include <list>
+#include "ContentAssistantPopup.h"
 
 #include "MathCell.h"
 #include "EditorCell.h"
 #include "GroupCell.h"
 #include "EvaluationQueue.h"
+#include "FindReplaceDialog.h"
 #include "Autocomplete.h"
 #include "AutocompletePopup.h"
-#include "Structure.h"
+#include "TableOfContents.h"
 #include "ToolBar.h"
 
 /*! The canvas that contains the spreadsheet the whole program is about.
 
-This canvas contains all the math, title, image etc.- cells of the current session.
- */
+This canvas contains all the math-, title-, image- input- ("editor-")- etc.- 
+cells of the current session.
+
+Most of the logic that handles that this canvas might be larger than the screen
+(and could even be larger than the computer's RAM would it have been handled as
+a big bitmap) is provided by wxWidgets:
+ - If part of the screen needs to be drawn it calls the OnPaint() method. 
+ - If the canvas is scrolled wxWidgets the parts that are already decides how much 
+   of the off-screen part of the canvas is cached in a backing store
+ - Also if the canvas is scrolled wxWidgets is intelligent enough to only request
+   the parts of the new view that aren't available on the screen or in the backing 
+   store
+ - If we call the Refresh() method wxWidgets requests us to draw the entire visible
+   part of the worksheet anew and invalidates all cached areas it might have.
+ - and the RefreshRect() method notifies wxWidgets that a rectangular region
+   contains changes that need to be redrawn.
+
+The worksheet isn't immediately redrawn on a key press, a mouse klick or on
+maxima outputting new data. Instead all such events are processed in order until
+wxMaxima has caught up with all the events (which causes wxWidgets to send a Idle
+event to the wxMaxima object) or until we reach a timeout. If the user manages
+to type faster than wxMaxima redraws the screen this allows us to skip
+a few redraws in order to process the keypresses as fast as the user types.
+Also this keeps us responsive even if maxima outputs data faster than 
+wxMaxima can display it.
+*/
 class MathCtrl: public wxScrolledCanvas
 {
 private:
-  //! true, if we have the current focus.
+  /*! The size of a scroll step
+    
+    Defines the size of a 
+    scroll step, but besides that also the accuracy wxScrolledCanvas
+    calculates some widths in.
+  */
+  int m_scrollUnit;
+  /*! The drawing contect used for calculating sizes. 
+
+    Drawing is done from a wxPaintDC in OnPaint() instead.
+  */
+  wxClientDC *m_dc;
+  //! The central settings storage
+  Configuration *m_configuration;
+  //! Where do we need to start the repainting of the worksheet?
+  GroupCell *m_redrawStart;
+  //! Do we need to redraw the worksheet?
+  bool m_redrawRequested;
+  //! Which groupCell incremental search has started in?
+  GroupCell *m_cellSearchStartedIn;
+  //! Which cursor position incremental search has started at?
+  int m_indexSearchStartedAt;
+  
+  
+  //! The clipboard format "mathML"
+
+  //! A class that publishes wxm data to the clipboard
+  static wxDataFormat m_wxmFormat;
+  //! A class that publishes MathML data to the clipboard
+  static wxDataFormat m_mathmlFormat;
+  //! A second way to publish MathML data for the cipboard
+  static wxDataFormat m_mathmlFormat2;
+  //! A class that publishes RTF data to the clipboard
+  static wxDataFormat m_rtfFormat;
+  //! A second way to publish RTF data on the clipboard
+  static wxDataFormat m_rtfFormat2;
+
+  /*! An object that can be filled with MathML data for the clipboard
+
+    \todo Is there a way to make this object share its data with MathMLDataObject2?
+   */
+  class MathMLDataObject:public wxCustomDataObject
+  {
+  public:    
+    MathMLDataObject(wxString data);
+    MathMLDataObject();
+  private:
+    wxCharBuffer m_databuf;
+  };
+
+  //! An object that can be filled with wxm data for the clipboard
+  class wxmDataObject:public wxCustomDataObject
+  {
+  public:
+    wxmDataObject(wxString data);
+    wxmDataObject();
+  private:
+    wxCharBuffer m_databuf;
+  };
+
+  class MathMLDataObject2:public wxCustomDataObject
+  {
+  public:
+    MathMLDataObject2(wxString data);
+    MathMLDataObject2();
+  private:
+    wxCharBuffer m_databuf;
+  };
+
+  /*! An object that can be filled with MathML data for the clipboard
+
+    \todo Is there a way to make this object share its data with RTFDataObject2?
+   */
+  class RtfDataObject:public wxCustomDataObject
+  {
+  public:
+    RtfDataObject(wxString data);
+    RtfDataObject();
+  private:
+    wxCharBuffer m_databuf;
+  };
+
+  class RtfDataObject2:public wxCustomDataObject
+  {
+  public:
+    RtfDataObject2(wxString data);
+    RtfDataObject2();
+  private:
+    wxCharBuffer m_databuf;
+  };
+
+//! true, if we have the current focus.
   bool m_hasFocus;
   //! The last beginning for the area being drawn
-  size_t m_lastTop;
+  long m_lastTop;
   //! The last ending for the area being drawn
-  size_t m_lastBottom;
+  long m_lastBottom;
   /*! \defgroup UndoBufferFill
 
     These methods and classes contain the undo functionality for tree changes:
@@ -128,6 +251,9 @@ private:
   //! Clear the list of actions for which an undo can be undone
   void TreeUndo_ClearRedoActionList();
 
+  //! Clear the list of actions for which undo can undo
+  void TreeUndo_ClearUndoActionList();
+
   //! Remove one action ftom the action list
   void TreeUndo_DiscardAction(std::list <TreeUndoAction *> *actionList);
 
@@ -259,19 +385,23 @@ private:
   //! Add a line to a file.
   void AddLineToFile(wxTextFile& output, wxString s, bool unicode = true);
   //! Copy the currently selected cells
-  MathCell* CopySelection();
+  MathCell* CopySelection(bool asData = false);
   /*! Copy the currently given list of cells
 
     \param start The cell to start copying at
     \param end   The cell the copy has to end with
     \param asdata 
-      - true:  The cells are copied in the order they are stored
-      - false: The cells are copied in the order they are displayed: Sometimes (for 
-               example with fractions that can be displayed in 2d or flat) these
-               two orders differ.
-   */
+      - true:  The cells are copied in the order they are stored. m_next and m_previous
+               therefore point to the right places. But m_nextToDraw and m_previousToDraw
+               will be treated as aliasses of m_next and m_previous.
+      - false: If a cell is broken into individual lines m_nextToDraw won't point to the
+               next cell that is to be displayed. It will point to the cell containing the
+               function name instead that is followed by the cell containing its contents. 
+               This is accurately copied if asdata=false. But m_next and m_previous are
+               treated as mere aliasses of m_nextToDraw and m_previousToDraw in this case.
+  */
   MathCell* CopySelection(MathCell* start, MathCell* end, bool asData = false);
-  
+
   void GetMaxPoint(int* width, int* height);
   //! Is executed if a timer associated with MathCtrl has expired.
   void OnTimer(wxTimerEvent& event);
@@ -280,6 +410,8 @@ private:
     True means: A save will be issued after the user stops typing.
    */
   bool m_autoSaveIntervalExpired;
+  //! Handle pinch-to-zoom-events
+  void OnMouseMagnify(wxMouseEvent& event);
   void OnMouseExit(wxMouseEvent& event);
   void OnMouseEnter(wxMouseEvent& event);
   /*! Is called by wxWidgets when it wants to redraw the console.
@@ -291,6 +423,8 @@ private:
   void OnSize(wxSizeEvent& event);
   void OnMouseRightDown(wxMouseEvent& event);
   void OnMouseLeftUp(wxMouseEvent& event);
+  //! We lost the mouse connection during drag-and-drop
+  void OnMouseCaptureLost(wxMouseCaptureLostEvent& event);
   void OnMouseLeftDown(wxMouseEvent& event);
   void OnMouseLeftInGcCell(wxMouseEvent& event, GroupCell *clickedInGC);
   void OnMouseLeftInGcLeft(wxMouseEvent& event, GroupCell *clickedInGC);
@@ -369,14 +503,14 @@ private:
   GroupCell *m_hCaretPosition;
   /*! The start for the selection when selecting group with the horizontally drawn cursor
 
-    This cell does actually define weree the selection was started and therefore does not need 
-    to be above m_hCaretPositionEnd in the worksheet.
+    This cell does define were the selection was actually started and therefore does not need 
+    to be above m_hCaretPositionEnd in the worksheet. See also m_selectionStart.
    */
   GroupCell *m_hCaretPositionStart;
   /*! The end of the selection when selecting group with the horizontally drawn cursor
 
-    This cell does actually define where the selection was ended and therefore does not need 
-    to be below m_hCaretPositionEnd in the worksheet.
+    This cell does define where the selection was actually ended and therefore does not need 
+    to be below m_hCaretPositionEnd in the worksheet. See also m_selectionEnd.
    */
   GroupCell *m_hCaretPositionEnd;
   bool m_leftDown;
@@ -394,7 +528,23 @@ private:
   GroupCell *m_workingGroup;
   //! The last group cell maxima was working on.
   GroupCell *m_lastWorkingGroup;
+  /*! The first cell of the currently selected range of groupCells.
+    
+    NULL, when no GroupCells are selected and NULL, if only stuff inside a GroupCell
+    is selected and therefore the selection is handled by EditorCell; This cell is 
+    always above m_selectionEnd.
+
+    See also m_hCaretPositionStart
+   */
   MathCell *m_selectionStart;
+  /*! The last cell of the currently selected range of groupCells.
+    
+    NULL, when no GroupCells are selected and NULL, if only stuff inside a GroupCell
+    is selected and therefore the selection is handled by EditorCell; This cell is 
+    always below m_selectionStart.
+
+    See also m_hCaretPositionEnd
+   */
   MathCell *m_selectionEnd;
   int m_clickType;
   GroupCell *m_clickInGC;
@@ -409,27 +559,50 @@ private:
   EditorCell *m_cellMouseSelectionStartedIn;
   EditorCell *m_cellKeyboardSelectionStartedIn;
   //! true = blink the cursor
-  bool m_switchDisplayCaret;
+  bool m_blinkDisplayCaret;
   //! Is the blinking vertically-drawn cursor currently visible?
   bool m_hCaretBlinkVisible;
-  /*! Is editing enabled?
-
-    Editing is disabled while we are waiting for maxima.
-   */
-  bool m_editingEnabled;
-  wxTimer m_timer, m_caretTimer, m_animationTimer;
+  //! Time step for autoscrolll when the mouse is outside the window
+  wxTimer m_timer;
+  //! The cursor blink rate. Also the timeout for redrawing the worksheet
+  wxTimer m_caretTimer;
+  //! The animation playback timer
+  wxTimer m_animationTimer;
   //! True only when an animation is running
   bool m_animate;
   wxBitmap *m_memory;
   //! True if no changes have to be saved.
   bool m_saved;
-  double m_zoomFactor;
   AutoComplete m_autocomplete;
   wxArrayString m_completions;
   bool m_autocompleteTemplates;
 
 
 public:
+  ///@{
+  //! Request the worksheet to be redrawn
+  void MarkRefreshAsDone(){m_redrawStart = NULL; m_redrawRequested = false;}
+  //! Redraw the worksheet if RequestRedraw() has been called.
+  void RedrawIfRequested();
+  //! Redraw the worksheet region merging this action with an eventual RequestRedraw()
+  void RedrawRect(wxRect rect);
+  /*! Request the worksheet to be redrawn
+
+    \param start Which cell do we need to start the redraw in? Subsequent calls to 
+    this function with different cells start the redraw at the upmost of the cells
+    that were passed to it.
+   */
+  void RequestRedraw(GroupCell *start = NULL);
+  //! Redraw the window now and mark any pending redraw request as "handled".
+  void ForceRedraw(){RequestRedraw();RedrawIfRequested();}
+  //! Is a Redraw requested?
+  bool RedrawRequested(){return m_redrawRequested;}
+  ///@}
+
+  //! To be called after enabling or disabling the visibility of code cells
+  void CodeCellVisibilityChanged();
+  //! Re-read the configuration
+  void UpdateConfig(){m_configuration->ReadConfig();}
   //! The name of the currently-opened file
   wxString m_currentFile;
 
@@ -445,7 +618,7 @@ public:
   //! Get the last known GroupCell maxima was working on
   GroupCell *GetLastWorkingGroup();
   //! The find-and-replace-dialog
-  wxFindReplaceDialog *m_findDialog;
+  FindReplaceDialog *m_findDialog;
   
   /*! True = schedule an update of the table of contents
     
@@ -502,6 +675,7 @@ public:
     popid_add_comment,
     popid_insert_input,
     popid_copy_tex,
+    popid_copy_text,
     popid_copy_mathml,
     popid_image,
     popid_animation_save,
@@ -522,15 +696,14 @@ public:
   MathCtrl(wxWindow* parent, int id, wxPoint pos, wxSize size);
   //! The destructor
   ~MathCtrl();
+  //! The timer that tells us when the keyboard is inactive so an autosave isn't disrupting
   wxTimer m_keyboardInactiveTimer;
-  //! Reads true if the keyboard was inactive for > 10 seconds
-  bool m_keyboardInactive;
 
   //! Clear the whole worksheet
   void DestroyTree();
   //! Delete a  part of the worksheet that previously has been unlinked.
   void DestroyTree(MathCell* tree);
-  MathCell* CopyTree();
+  GroupCell* CopyTree();
   /*! Insert group cells into the worksheet
 
     \param cells The list of cells that has to be inserted
@@ -560,7 +733,11 @@ public:
     the line is appended to m_last, instead.
   */
   void InsertLine(MathCell *newLine, bool forceNewLine = false);
-  void Recalculate(bool force = false);  
+  //! Recalculate the worksheet starting with the cell start.
+  void Recalculate(GroupCell *start,bool force = false);
+  
+  void Recalculate(bool force = false) {Recalculate(m_tree,force);}
+  //! Force a full recalculation of the worksheet
   void RecalculateForce() {
     Recalculate(true);
   }
@@ -653,11 +830,17 @@ public:
   bool Copy(bool astext = false);
   //! Copy the selection to the clipboard as it would appear in a .wxm file
   bool CopyCells();
+  //! Copy a textual representation of the current selection to the clipboard
+  bool CopyText();
   //! Copy the TeX representation of the current selection to the clipboard
   bool CopyTeX();
+  //! Convert the current selection to MathML
+  wxString ConvertSelectionToMathML();
+  //! Convert the current selection to a bitmap
+  wxBitmap ConvertSelectionToBitmap();
   //! Copy the MathML representation of the current selection to the clipboard
   bool CopyMathML();
-  //! Copy a bitmap of the the current selection to the clipboard
+  //! Copy a bitmap of the current selection to the clipboard
   bool CopyBitmap();
   wxSize CopyToFile(wxString file);
   wxSize CopyToFile(wxString file, MathCell* start, MathCell* end, bool asData = false,int scale=1);
@@ -674,6 +857,10 @@ public:
                              worksheet's "modified" status.
   */
   bool ExportToWXMX(wxString file, bool markAsSaved = true);	
+  //! The start of a RTF document
+  wxString RTFStart();
+  //! The end of a RTF document
+  wxString RTFEnd();
   //! export to a LaTeX file
   bool ExportToTeX(wxString file);
   /*! Convert the current selection to a string 
@@ -696,15 +883,23 @@ public:
   //! Select the cell sel
   void SetSelection(MathCell* sel) { SetSelection(sel,sel); }
   //! Select the cell range start-end
-  void SetSelection(MathCell* start,MathCell* end) { m_selectionStart = start;m_selectionEnd = end; }
+  void SetSelection(MathCell* start,MathCell* end) {
+    m_selectionStart = start;
+    m_selectionEnd = end;
+    if(m_selectionStart == NULL)
+    {
+      m_hCaretPositionStart = NULL;
+      m_hCaretPositionEnd = NULL;
+    }
+  }
+  ContentAssistantPopup *m_autocompletePopup;
   bool CanEdit();
-  void EnableEdit(bool enable = true) { m_editingEnabled = enable; }
   bool ActivatePrevInput();
   bool ActivateNextInput(bool input = false);
   //! Scrolls to the cursor
   void ScrollToCaret();
   //! Scrolls to a given cell
-  void ScrollToCell(MathCell *cell);
+  void ScrollToCell(MathCell *cell,bool scrollToTop);
   //! Returns the cell the cursor that is drawn as a vertical line is in.
   EditorCell* GetActiveCell() { return m_activeCell; }
   //! Is the point currently visible on the worksheet?
@@ -712,6 +907,9 @@ public:
   //! Is the caret (hcaret or vcaret) currently visible on the worksheet?
   bool CaretVisibleIs();
 
+  //! The first groupCell that is currently visible.
+  GroupCell *FirstVisibleGC();
+
   //! Scrolls to a point on the worksheet
   void ShowPoint(wxPoint point);
   void OnSetFocus(wxFocusEvent& event);
@@ -728,8 +926,15 @@ public:
   bool AnimationRunning() { return m_animate; }
   //! Tell if an animation should run running
   void AnimationRunning(bool state) { m_animate = state; }
+  //! Is the editor active in the last cell of the worksheet?
   bool IsActiveInLast() { return m_activeCell != NULL && m_activeCell->GetParent() == m_last; }
+  //! Informs the worksheet which GroupCell maxima is currently working in
   void SetWorkingGroup(GroupCell *group);
+  //! Returns the last cell of the worksheet
+  GroupCell *GetLastCell()
+    {
+      return m_last;
+    }
   bool IsSelectionInWorking();
   void SetActiveCell(EditorCell *cell, bool callRefresh = true);
   void SetDefaultHCaret();
@@ -813,16 +1018,21 @@ public:
   void UnfoldAll();
   GroupCell *TearOutTree(GroupCell *start, GroupCell *end);
   // methods for zooming the document in and out
-  double GetZoomFactor() { return m_zoomFactor; }
   void SetZoomFactor(double newzoom, bool recalc = true);
   void CommentSelection();
   //! Called if the user is scrolling through the document.
   void OnScrollChanged(wxScrollEvent &ev);
+  /*! Do an incremental search from the cursor or the point the last search started at
+
+    Used by the find dialog.
+    \todo Keep a list of positions the last few letters were found at?
+   */
+  bool FindIncremental(wxString str, bool down, bool ignoreCase);
   /*! Find the next ocourrence of a string
 
     Used by the find dialog.
    */
-  bool FindNext(wxString str, bool down, bool ignoreCase);
+  bool FindNext(wxString str, bool down, bool ignoreCase,bool warn=true);
   /*! Replace the current ocourrence of a string
 
     Used by the find dialog.
@@ -843,7 +1053,7 @@ public:
   GroupCell *GetWorkingGroup() { return m_workingGroup; }
   void OpenNextOrCreateCell();
   //! The table of contents pane
-  Structure*    m_structure;
+  TableOfContents*    m_tableOfContents;
   //! Called when the "Scroll to currently evaluated" button is pressed.
   void OnFollow();
   //! The toolbar of the main window: We need to access it and therefore have it defined here.
@@ -875,6 +1085,8 @@ public:
   void OpenQuestionCaret(wxString txt=wxT(""));
 
  protected:
+  //! Is called if this element looses or gets the focus
+  void OnActivate(wxActivateEvent& event);
   DECLARE_EVENT_TABLE()
 };
 
