@@ -19,6 +19,10 @@
 //  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 //
 
+/*! \file
+  This file defines the class MathParser that reads wxmx data and math from maxma.
+*/
+
 #include <wx/config.h>
 #include <wx/tokenzr.h>
 #include <wx/sstream.h>
@@ -74,6 +78,7 @@ wxXmlNode* MathParser::GetNextTag(wxXmlNode* node)
 
 MathParser::MathParser(wxString zipfile)
 {
+  m_displayedDigits=100;
   m_workingDirectory = wxEmptyString;
   m_ParserStyle = MC_TYPE_DEFAULT;
   m_FracStyle = FracCell::FC_NORMAL;
@@ -261,17 +266,20 @@ MathCell* MathParser::ParseFracTag(wxXmlNode* node)
   if (child)
   {
     MathCell *enumerator = ParseTag(child, false);
+    wxASSERT_MSG(enumerator != NULL,_("Bug: Fraction without enumerator"));
     if(enumerator==NULL)
       return NULL;
     frac->SetNum(enumerator);
     child = GetNextTag(child);
     child = SkipWhitespaceNode(child);
+    wxASSERT_MSG(((child != NULL) && (enumerator != NULL)),_("Bug: Fraction without enumerator"));
     if ((child != NULL) && (enumerator != NULL))
     {
       MathCell *denominator = ParseTag(child, false);
       if(denominator != NULL)
       {
         frac->SetDenom(denominator);
+        wxASSERT_MSG(enumerator != NULL,_("Bug: Fraction without denominator"));
         frac->SetStyle(TS_VARIABLE);
         
         if(node->GetAttribute(wxT("line")) == wxT("no"))
@@ -287,8 +295,7 @@ MathCell* MathParser::ParseFracTag(wxXmlNode* node)
     }
   }
 
-  // The frac tag didn't have a child tag. TODO: Raise an assert in this
-  // case?
+  wxASSERT_MSG(false,_("bug:Invalid frac tag"));
   delete frac;
   return NULL;
 }
@@ -327,20 +334,24 @@ MathCell* MathParser::ParseSupTag(wxXmlNode* node)
   if (child)
   {
     expt->SetBase(ParseTag(child, false));
-    child = GetNextTag(child);
-    if (child)
+    if(expt != NULL)
     {
-      MathCell* power = ParseTag(child, false);
-      power->SetExponentFlag();
-      expt->SetPower(power);
-      expt->SetType(m_ParserStyle);
-      expt->SetStyle(TS_VARIABLE);
-      return expt;
+      child = GetNextTag(child);
+      if (child)
+      {
+        MathCell* power = ParseTag(child, false);
+        if(power)
+        {
+          power->SetExponentFlag();
+          expt->SetPower(power);
+          expt->SetType(m_ParserStyle);
+          expt->SetStyle(TS_VARIABLE);
+          return expt;
+        }
+      }
     }
   }
-
-  // SupTag without subscript => Return nothing. TODO: Raise an assert
-  // in this case?
+  wxASSERT_MSG(false,_("bug:Invalid sup tag"));
   delete expt;
   return NULL;
 }
@@ -372,8 +383,7 @@ MathCell* MathParser::ParseSubSupTag(wxXmlNode* node)
     }
   }
 
-  // subsup tag without super- or subscript => return nothing.
-  // TODO: Raise an assert in this case?
+  wxASSERT_MSG(false,_("bug:Invalid subsup tag"));
   delete subsup;
   return NULL;
 }
@@ -382,8 +392,7 @@ MathCell* MathParser::ParseSubTag(wxXmlNode* node)
 {
   SubCell *sub = new SubCell;
   wxXmlNode* child = node->GetChildren();
-  child = SkipWhitespaceNode(child);
-  
+  child = SkipWhitespaceNode(child);  
   if (child)
   {
     sub->SetBase(ParseTag(child, false));
@@ -400,12 +409,12 @@ MathCell* MathParser::ParseSubTag(wxXmlNode* node)
         return(NULL);
       sub->SetType(m_ParserStyle);
       sub->SetStyle(TS_VARIABLE);
+
       return sub;
     }
   }
 
-  // subtag without subscript => Return nothing:
-  // TODO: Raise an assert?
+  wxASSERT_MSG(false,_("bug:Invalid sub tag"));
   delete sub;
   return NULL;
 }
@@ -429,6 +438,7 @@ MathCell* MathParser::ParseAtTag(wxXmlNode* node)
       return at;
     }
   }
+  wxASSERT_MSG(false,_("bug:Invalid at tag"));
   delete at;
   return NULL;
 }
@@ -451,8 +461,7 @@ MathCell* MathParser::ParseFunTag(wxXmlNode* node)
       return fun;
     }
   }
-  // Function tag without parameters => Return nothing.
-  // TODO: Raise an assert?
+  wxASSERT_MSG(false,_("bug:Invalid function tag"));
   delete fun;
   return NULL;
 }
@@ -473,11 +482,11 @@ MathCell* MathParser::ParseText(wxXmlNode* node, int style)
     if (style == TS_NUMBER)
     {
       m_displayedDigits=100;
-      wxConfigBase *config = wxConfig::Get();      
+      wxConfigBase *config = wxConfig::Get();
       config->Read(wxT("displayedDigits"),&m_displayedDigits);
 
       if (m_displayedDigits<10)m_displayedDigits=10;
-      if (str.Length() > m_displayedDigits)
+      if ((long)str.Length() > m_displayedDigits)
 	{
 	  int left= m_displayedDigits/3;
 	  if (left>30) left=30;
@@ -619,8 +628,7 @@ MathCell* MathParser::ParseLimitTag(wxXmlNode* node)
     }
   }
 
-  // Limit tag with out argument => Return nothing.
-  // TODO: Raise an assert?
+  wxASSERT_MSG(false,_("bug:Invalid limit tag"));
   delete limit;
   return NULL;
 }
@@ -653,8 +661,7 @@ MathCell* MathParser::ParseSumTag(wxXmlNode* node)
       }
     }
   }
-  // Sum without nothing to sum up => Return nothing.
-  // TODO: Raise an assert?
+  wxASSERT_MSG(false,_("bug:Invalid sum tag"));
   delete sum;
   return NULL;
 }
@@ -709,6 +716,8 @@ MathCell* MathParser::ParseIntTag(wxXmlNode* node)
       }
     }
   }
+  
+  wxASSERT_MSG(false,_("bug:Invalid int tag"));
   delete in;
   return NULL;
 }
@@ -975,6 +984,10 @@ MathCell* MathParser::ParseTag(wxXmlNode* node, bool all)
         tmp = ParseTag(node->GetChildren());
       }
 
+      // The new cell may needing being equipped with a "altCopy" tag.
+      if ((tmp != NULL) && (node->GetAttribute(wxT("altCopy"), &altCopy)))
+        tmp->SetAltCopyText(altCopy);
+
       // Append the cell we found (tmp) to the list of cells we parsed so far (cell).
       if(tmp != NULL)
       {
@@ -994,10 +1007,7 @@ MathCell* MathParser::ParseTag(wxXmlNode* node, bool all)
       else
         cell->AppendCell(ParseText(node));
     }
-    
-    if (!all)
-      break;
-    
+
     if (cell != NULL)
     {
       // Append the new cell to the return value
@@ -1006,29 +1016,28 @@ MathCell* MathParser::ParseTag(wxXmlNode* node, bool all)
       else
         cell = cell->m_next;
     }
-    else if (warning)
+    else if ((warning) && (!all))
     {
+      // Tell the user we ran into problems.
       wxString name;
       name.Trim(true);
       name.Trim(false);
       if(cell != NULL) name = cell->ToString();
       if(name.Length()!= 0)
       {
-        wxMessageBox(_("Parts of the document will not be loaded correctly!\nFound unknown XML Tag name "+ name), _("Warning"),
+        wxMessageBox(_("Parts of the document will not be loaded correctly:\nFound unknown XML Tag name "+ name), _("Warning"),
                      wxOK | wxICON_WARNING);
         warning = false;
       }
     }
     
-    if ((cell != NULL) &&(node->GetAttribute(wxT("altCopy"), &altCopy)))
-      cell->SetAltCopyText(altCopy);
+    if (!all)
+      break;
     
     node = GetNextTag(node);
   }
   
-  if (retval != NULL)
-    return retval;
-  return cell;
+  return retval;
 }
 
 /***
@@ -1069,7 +1078,7 @@ MathCell* MathParser::ParseLine(wxString s, int style)
   graph.Replace(&s, wxT("?"));
 #endif
 
-  if ((s.Length() < showLength) || (showLength==0))
+  if (((long)s.Length() < showLength) || (showLength==0))
   {
 
     wxXmlDocument xml;
