@@ -1,6 +1,7 @@
 // -*- mode: c++; c-file-style: "linux"; c-basic-offset: 2; indent-tabs-mode: nil -*-
 //
 //  Copyright (C) 2004-2015 Andrej Vodopivec <andrej.vodopivec@gmail.com>
+//            (C) 2016 Gunter Königsmann <wxMaxima@physikbuch.de>
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
@@ -18,6 +19,12 @@
 //  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 //
 
+/*! \file
+  This file declares the class MathPrintOut
+
+  MathPrintOut is the class that handles printing.
+*/
+
 #ifndef MATHPRINTOUT_H
 #define MATHPRINTOUT_H
 
@@ -29,6 +36,7 @@
 #include <vector>
 
 #include "MathCell.h"
+#include "GroupCell.h"
 
 using namespace std;
 
@@ -38,8 +46,8 @@ public:
   MathPrintout(wxString title);
   ~MathPrintout();
   void DestroyTree();
-  void DestroyTree(MathCell* tree);
-  void SetData(MathCell* tree);
+  void DestroyTree(GroupCell* tree);
+  void SetData(GroupCell* tree);
   void SetupData();
   void BreakPages();
   void Recalculate();
@@ -54,9 +62,10 @@ public:
   double GetPPIScale();
   void GetScreenScale(double *scaleX, double *scaleY);
 private:
+  wxSize m_oldViewportSize;
   int m_numberOfPages;
   wxString m_title;
-  MathCell* m_tree;
+  GroupCell* m_tree;
   vector<MathCell*> m_pages;
 };
 
