@@ -1,7 +1,7 @@
 // -*- mode: c++; c-file-style: "linux"; c-basic-offset: 2; indent-tabs-mode: nil -*-
 //
 //  Copyright (C) 2004-2015 Andrej Vodopivec <andrej.vodopivec@gmail.com>
-//            (C) 2014-2015 Gunter Königsmann <wxMaxima@physikbuch.de>
+//            (C) 2014-2016 Gunter Königsmann <wxMaxima@physikbuch.de>
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
@@ -19,6 +19,13 @@
 //  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 //
 
+/*! \file
+  This file defines the class MatrCell
+
+  MatrCell is the MathCell type that represents matrices and matrix-like 
+  elements like the table_form command.
+*/
+
 #include "MatrCell.h"
 
 MatrCell::MatrCell() : MathCell()
@@ -78,12 +85,13 @@ void MatrCell::Destroy()
   m_next = NULL;
 }
 
-void MatrCell::RecalculateWidths(CellParser& parser, int fontsize)
+void MatrCell::RecalculateWidths(int fontsize)
 {
-  double scale = parser.GetScale();
+  Configuration *configuration = Configuration::Get();
+  double scale = configuration->GetScale();
   for (int i = 0; i < m_matWidth*m_matHeight; i++)
   {
-    m_cells[i]->RecalculateWidthsList(parser, MAX(MC_MIN_SIZE, fontsize - 2));
+    m_cells[i]->RecalculateWidthsList(MAX(MC_MIN_SIZE, fontsize - 2));
   }
   m_widths.clear();
   for (int i = 0; i < m_matWidth; i++)
@@ -104,13 +112,14 @@ void MatrCell::RecalculateWidths(CellParser& parser, int fontsize)
   ResetData();
 }
 
-void MatrCell::RecalculateSize(CellParser& parser, int fontsize)
+void MatrCell::RecalculateHeight(int fontsize)
 {
-  double scale = parser.GetScale();
+  Configuration *configuration = Configuration::Get();
+  double scale = configuration->GetScale();
 
   for (int i = 0; i < m_matWidth*m_matHeight; i++)
   {
-    m_cells[i]->RecalculateSizeList(parser, MAX(MC_MIN_SIZE, fontsize - 2));
+    m_cells[i]->RecalculateHeightList(MAX(MC_MIN_SIZE, fontsize - 2));
   }
   m_centers.clear();
   m_drops.clear();
@@ -134,12 +143,15 @@ void MatrCell::RecalculateSize(CellParser& parser, int fontsize)
   m_center = m_height / 2;
 }
 
-void MatrCell::Draw(CellParser& parser, wxPoint point, int fontsize)
+void MatrCell::Draw(wxPoint point, int fontsize)
 {
-  if (DrawThisCell(parser, point))
+  MathCell::Draw(point, fontsize);
+
+  if (DrawThisCell(point) && InUpdateRegion())
   {
-    wxDC& dc = parser.GetDC();
-    double scale = parser.GetScale();
+    Configuration *configuration = Configuration::Get();
+    wxDC& dc = configuration->GetDC();
+    double scale = configuration->GetScale();
     wxPoint mp;
     mp.x = point.x + SCALE_PX(5, scale);
     mp.y = point.y - m_center;
@@ -151,12 +163,12 @@ void MatrCell::Draw(CellParser& parser, wxPoint point, int fontsize)
         mp.y += m_centers[j];
         wxPoint mp1(mp);
         mp1.x = mp.x + (m_widths[i] - m_cells[j * m_matWidth + i]->GetFullWidth(scale)) / 2;
-        m_cells[j*m_matWidth + i]->DrawList(parser, mp1, MAX(MC_MIN_SIZE, fontsize - 2));
+        m_cells[j*m_matWidth + i]->DrawList(mp1, MAX(MC_MIN_SIZE, fontsize - 2));
         mp.y += (m_drops[j] + SCALE_PX(10, scale));
       }
       mp.x += (m_widths[i] + SCALE_PX(10, scale));
     }
-    SetPen(parser);
+    SetPen();
     if (m_specialMatrix)
     {
       if (m_inferenceMatrix)
@@ -206,34 +218,45 @@ void MatrCell::Draw(CellParser& parser, wxPoint point, int fontsize)
                   point.x + m_width - SCALE_PX(5, scale) - 1,
                   point.y + m_center - SCALE_PX(2, scale));
     }
-    UnsetPen(parser);
+    UnsetPen();
   }
-  MathCell::Draw(parser, point, fontsize);
 }
 
 wxString MatrCell::ToString()
 {
-  wxString s = wxT("matrix(");
+  wxString s = wxT("matrix(\n");
   for (int i = 0; i < m_matHeight; i++)
   {
-    s += wxT("[");
+    s += wxT("\t\t[");
     for (int j = 0; j < m_matWidth; j++)
     {
       s += m_cells[i * m_matWidth + j]->ListToString();
       if (j < m_matWidth - 1)
-        s += wxT(",");
+        s += wxT(",\t");
     }
     s += wxT("]");
     if (i < m_matHeight - 1)
       s += wxT(",");
+    s += wxT("\n");
   }
-  s += wxT(")");
+  s += wxT("\t)");
   return s;
 }
 
 wxString MatrCell::ToTeX()
 {
-  wxString s = wxT("\\begin{pmatrix}");
+  //ToDo: We ignore colNames and rowNames here. Are they currently in use?
+  wxString s;
+
+    if(!m_specialMatrix)
+      s = wxT("\\begin{pmatrix}");
+    else
+    {
+      s = wxT("\\begin{array}{");
+      for (int j = 0; j < m_matWidth; j++)
+        s += wxT("c");
+      s += wxT("}");
+    }
   for (int i = 0; i < m_matHeight; i++)
   {
     for (int j = 0; j < m_matWidth; j++)
@@ -243,12 +266,57 @@ wxString MatrCell::ToTeX()
         s += wxT(" & ");
     }
     if (i < m_matHeight - 1)
-      s += wxT("\\cr ");
+      s += wxT("\\\\\n");
   }
-  s += wxT("\\end{pmatrix}");
+    if(!m_specialMatrix)
+      s += wxT("\\end{pmatrix}");
+    else
+      s += wxT("\\end{array}");
   return s;
 }
 
+wxString MatrCell::ToMathML()
+{
+  wxString retval;
+  if(!m_specialMatrix)
+    retval = wxT("<mrow><mo>(</mo><mrow>");
+  retval +=  wxT("<mtable>");
+  
+  for (int i = 0; i < m_matHeight; i++)
+  {
+    retval += wxT("<mtr>");
+    for (int j = 0; j < m_matWidth; j++)
+      retval += wxT("<mtd>") + m_cells[i * m_matWidth + j]->ListToMathML() + wxT("</mtd>");
+    retval += wxT("</mtr>");
+  }
+  retval += wxT("</mtable>\n");
+  if(!m_specialMatrix)
+    retval += wxT("</mrow><mo>)</mo></mrow>\n");
+  return retval;
+}
+
+wxString MatrCell::ToOMML()
+{
+  wxString retval;
+
+  retval = wxT("<m:d>");
+   if(!m_specialMatrix)
+     retval += wxT("<m:dPr><m:begChr>(</m:begChr><m:endChr>)</m:endChr></m:dPr>");
+
+   retval += wxT("<m:e><m:m>");
+
+   for (int i = 0; i < m_matHeight; i++)
+   {
+     retval += wxT("<m:mr>");
+     for (int j = 0; j < m_matWidth; j++)
+       retval += wxT("<m:e>") + m_cells[i * m_matWidth + j]->ListToOMML() + wxT("</m:e>");
+     retval += wxT("</m:mr>");
+   }
+
+   retval += wxT("</m:m></m:e></m:d>");
+   return retval;
+}
+
 wxString MatrCell::ToXML()
 {
   wxString s = wxEmptyString;
