@@ -19,18 +19,23 @@
 //  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 //
 
+/*! \file
+  This file defines the class ParenCell
+
+  ParenCell is the MathCell type that represents a math element that is kept
+  between parenthesis.
+ */
+
 #include "ParenCell.h"
 #include "TextCell.h"
 
-#if defined __WXMSW__
- #define PAREN_LEFT_TOP "\xE6"
- #define PAREN_LEFT_BOTTOM "\xE8"
- #define PAREN_RIGHT_TOP "\xF6"
- #define PAREN_RIGHT_BOTTOM "\xF8"
- #define PAREN_LEFT_EXTEND "\xE7"
- #define PAREN_RIGHT_EXTEND "\xF7"
- #define PAREN_FONT_SIZE 12
-#endif
+#define PAREN_LEFT_TOP "\xE6"
+#define PAREN_LEFT_BOTTOM "\xE8"
+#define PAREN_RIGHT_TOP "\xF6"
+#define PAREN_RIGHT_BOTTOM "\xF8"
+#define PAREN_LEFT_EXTEND "\xE7"
+#define PAREN_RIGHT_EXTEND "\xF7"
+#define PAREN_FONT_SIZE 12
 
 #define PAREN_OPEN "\xB0"
 #define PAREN_CLOSE "\xD1"
@@ -48,6 +53,12 @@
 
 ParenCell::ParenCell() : MathCell()
 {
+  m_last1 = NULL;
+  m_signSize = 50;
+  m_signWidth = 18;
+  m_signTop = m_signSize / 2;
+  m_parenFontSize = 12;
+  m_bigParenType = PARENTHESIS_NORMAL;
   m_innerCell = NULL;
   m_print = true;
   m_open = new TextCell(wxT("("));
@@ -81,6 +92,7 @@ MathCell* ParenCell::Copy()
   ParenCell *tmp = new ParenCell;
   CopyData(this, tmp);
   tmp->SetInner(m_innerCell->CopyList(), m_type);
+  tmp->m_isBroken = m_isBroken;
 
   return tmp;
 }
@@ -111,62 +123,82 @@ void ParenCell::SetInner(MathCell *inner, int type)
   m_last1 = inner;
 }
 
-void ParenCell::RecalculateWidths(CellParser& parser, int fontsize)
+void ParenCell::RecalculateWidths(int fontsize)
 {
-  double scale = parser.GetScale();
+  Configuration *configuration = Configuration::Get();
+  double scale = configuration->GetScale();
   if (m_innerCell == NULL)
     m_innerCell = new TextCell;
 
-  m_innerCell->RecalculateWidthsList(parser, fontsize);
+  m_innerCell->RecalculateWidthsList(fontsize);
 
-  if (parser.CheckTeXFonts())
+  if (configuration->CheckTeXFonts())
   {
-    wxDC& dc = parser.GetDC();
-    m_innerCell->RecalculateSizeList(parser, fontsize);
+    wxDC& dc = configuration->GetDC();
+    m_innerCell->RecalculateHeightList(fontsize);
     int size = m_innerCell->GetMaxHeight();
-
+    /// BUG 2897415: Exporting equations to HTML locks up on Mac
+    ///  there is something wrong with what dc.GetTextExtent returns,
+    ///  make sure there is no infinite loop!
+    // Avoid a possible infinite loop.
+    if(size < 2) size = 12;
+    
     int fontsize1 = (int) ((fontsize * scale + 0.5));
 
-    if (size < 2*fontsize1)
-      m_bigParenType = 0;
-    else if (size < 4*fontsize1)
-      m_bigParenType = 1;
+    if (size <= 2*fontsize1)
+    {
+      m_bigParenType = PARENTHESIS_NORMAL;
+      wxFont font(fontsize1, wxFONTFAMILY_MODERN,
+                  wxFONTSTYLE_NORMAL, wxFONTWEIGHT_NORMAL, false,
+                  configuration->GetTeXCMRI());
+      font.SetPointSize(fontsize1);
+      dc.SetFont(font);
+      dc.GetTextExtent( wxT("("),&m_signWidth, &m_signSize);
+    }
     else
-      m_bigParenType = 2;
-
-    if (m_bigParenType < 2)
+    {
+      if (size <= 4.1*fontsize1)
+      {
+        m_bigParenType = PARENTHESIS_BIG;
+        wxFont font(fontsize1, wxFONTFAMILY_MODERN,
+                    wxFONTSTYLE_NORMAL, wxFONTWEIGHT_NORMAL, false,
+                    configuration->GetTeXCMEX());
+        font.SetPointSize(fontsize1);
+        dc.SetFont(font);
+        dc.GetTextExtent(wxT(PAREN_OPEN),&m_signWidth, &m_signSize);
+      }
+      else
+      {
+        m_bigParenType = PARENTHESIS_ASSEMBLED;
+#ifdef __WXMSW__
+        dc.GetTextExtent(wxT(PAREN_LEFT_TOP),
+                         &m_signWidth, &m_signSize);
+#else
+        dc.GetTextExtent(wxT(PAREN_OPEN),
+                         &m_signWidth, &m_signSize);
+#endif
+      }
+    }
+    
+    if (m_bigParenType != PARENTHESIS_ASSEMBLED)
     {
       m_parenFontSize = fontsize;
       fontsize1 = (int) ((m_parenFontSize * scale + 0.5));
-
-      dc.SetFont( wxFont(fontsize1, wxFONTFAMILY_MODERN,
-			 wxFONTSTYLE_NORMAL, wxFONTWEIGHT_NORMAL, false,
-			 m_bigParenType == 0 ?
-			 parser.GetTeXCMRI() :
-			 parser.GetTeXCMEX()));
-      dc.GetTextExtent(m_bigParenType == 0 ? wxT("(") :
-                       m_bigParenType == 1 ? wxT(PAREN_OPEN) :
-		       wxT(PAREN_OPEN_TOP),
-                       &m_signWidth, &m_signSize);
-
-      /// BUG 2897415: Exporting equations to HTML locks up on Mac
-      ///  there is something wrong with what dc.GetTextExtent returns,
-      ///  make sure there is no infinite loop!
+      
       int i=0;
       
-      // Avoid a possible infinite loop.
-      if(size < 2) size = 2;
-
-      wxASSERT_MSG(m_signSize > 0,_("Seems like something is broken with the maths font. Installing http://www.math.union.edu/~dpvc/jsmath/download/jsMath-fonts.html and checking \"Use JSmath fonts\" in the configuration dialogue should fix it."));
+      
       if(m_signSize > 0)
-        while (m_signSize < TRANSFORM_SIZE(m_bigParenType, size) && i<20)
+        while (m_signSize < TRANSFORM_SIZE(m_bigParenType, size) && i<40)
         {
-          int fontsize1 = (int) ((m_parenFontSize++ * scale + 0.5));
-          dc.SetFont(wxFont(fontsize1, wxFONTFAMILY_MODERN,
-                            wxFONTSTYLE_NORMAL, wxFONTWEIGHT_NORMAL, false,
-                            m_bigParenType == 0 ?
-                            parser.GetTeXCMRI() :
-                            parser.GetTeXCMEX()));
+          int fontsize1 = (int) ((++m_parenFontSize * scale + 0.5));
+          wxFont font(fontsize1, wxFONTFAMILY_MODERN,
+                      wxFONTSTYLE_NORMAL, wxFONTWEIGHT_NORMAL, false,
+                      m_bigParenType == 0 ?
+                      configuration->GetTeXCMRI() :
+                      configuration->GetTeXCMEX());
+          font.SetPointSize(fontsize1);
+          dc.SetFont(font);
           dc.GetTextExtent(m_bigParenType == 0 ? wxT("(") :
                            m_bigParenType == 1 ? wxT(PAREN_OPEN) :
                            wxT(PAREN_OPEN_TOP),
@@ -180,11 +212,13 @@ void ParenCell::RecalculateWidths(CellParser& parser, int fontsize)
     {
       m_parenFontSize = fontsize;
       fontsize1 = (int) ((m_parenFontSize * scale + 0.5));
-      dc.SetFont(wxFont(fontsize1, wxFONTFAMILY_MODERN,
-			wxFONTSTYLE_NORMAL, wxFONTWEIGHT_NORMAL, false,
-                        m_bigParenType < 1 ?
-			parser.GetTeXCMRI() :
-			parser.GetTeXCMEX()));
+      wxFont font(fontsize1, wxFONTFAMILY_MODERN,
+                  wxFONTSTYLE_NORMAL, wxFONTWEIGHT_NORMAL, false,
+                  m_bigParenType < 1 ?
+                  configuration->GetTeXCMRI() :
+                  configuration->GetTeXCMEX());
+      font.SetPointSize(fontsize1);
+      dc.SetFont(font);
       dc.GetTextExtent(wxT(PAREN_OPEN), &m_signWidth, &m_signSize);
     }
 
@@ -193,70 +227,87 @@ void ParenCell::RecalculateWidths(CellParser& parser, int fontsize)
   }
   else
   {
+    // No TeX fonts
 #if defined __WXMSW__
-    wxDC& dc = parser.GetDC();
+    wxDC& dc = configuration->GetDC();
     int fontsize1 = (int) ((PAREN_FONT_SIZE * scale + 0.5));
-    dc.SetFont(wxFont(fontsize1, wxFONTFAMILY_MODERN,
-                      parser.IsItalic(TS_DEFAULT),
-                      parser.IsBold(TS_DEFAULT),
-                      parser.IsUnderlined(TS_DEFAULT),
-                      parser.GetSymbolFontName()));
+    wxFont font(fontsize1, wxFONTFAMILY_MODERN,
+                configuration->IsItalic(TS_DEFAULT),
+                configuration->IsBold(TS_DEFAULT),
+                configuration->IsUnderlined(TS_DEFAULT),
+                configuration->GetSymbolFontName());
+    font.SetPointSize(fontsize1);
+    dc.SetFont(font);
     dc.GetTextExtent(PAREN_LEFT_TOP, &m_charWidth, &m_charHeight);
     if(m_charHeight < 2)
       m_charHeight = 2;
     m_width = m_innerCell->GetFullWidth(scale) + 2*m_charWidth;
 #else
-    m_width = m_innerCell->GetFullWidth(scale) + SCALE_PX(12, parser.GetScale());
+    m_width = m_innerCell->GetFullWidth(scale) + SCALE_PX(12, configuration->GetScale())
+      + 2 * GetDefaultLineWidth();
 #endif
   }
-  m_open->RecalculateWidthsList(parser, fontsize);
-  m_close->RecalculateWidthsList(parser, fontsize);
+  m_open->RecalculateWidthsList(fontsize);
+  m_close->RecalculateWidthsList(fontsize);
   ResetData();
 }
 
-void ParenCell::RecalculateSize(CellParser& parser, int fontsize)
+void ParenCell::RecalculateHeight(int fontsize)
 {
-  double scale = parser.GetScale();
-  m_innerCell->RecalculateSizeList(parser, fontsize);
+  Configuration *configuration = Configuration::Get();
+  double scale = configuration->GetScale();
+  m_innerCell->RecalculateHeightList(fontsize);
   m_height = m_innerCell->GetMaxHeight() + SCALE_PX(2, scale);
   m_center = m_innerCell->GetMaxCenter() + SCALE_PX(1, scale);
 
 #if defined __WXMSW__
-  if (!parser.CheckTeXFonts())
+  if (!configuration->CheckTeXFonts())
   {
-    wxDC& dc = parser.GetDC();
+    wxDC& dc = configuration->GetDC();
     int fontsize1 = (int) ((fontsize * scale + 0.5));
-    dc.SetFont(wxFont(fontsize1, wxFONTFAMILY_MODERN,
-                      wxFONTSTYLE_NORMAL, wxFONTWEIGHT_NORMAL, false,
-                      parser.GetFontName()));
+    wxFont font(fontsize1, wxFONTFAMILY_MODERN,
+                wxFONTSTYLE_NORMAL, wxFONTWEIGHT_NORMAL, false,
+                configuration->GetFontName());
+    font.SetPointSize(fontsize1);
+    dc.SetFont(font);
     dc.GetTextExtent(wxT("("), &m_charWidth1, &m_charHeight1);
     if(m_charHeight1 < 2)
       m_charHeight1 = 2;
   }
 #endif
 
-  m_open->RecalculateSizeList(parser, fontsize);
-  m_close->RecalculateSizeList(parser, fontsize);
+  m_open->RecalculateHeightList(fontsize);
+  m_close->RecalculateHeightList(fontsize);
+
+  if(m_isBroken)
+  {
+    m_height = MAX(m_innerCell->GetMaxHeight(),m_open->GetMaxHeight());
+    m_center = MAX(m_innerCell->GetMaxCenter(),m_open->GetMaxCenter());
+  }
 }
 
-void ParenCell::Draw(CellParser& parser, wxPoint point, int fontsize)
+void ParenCell::Draw(wxPoint point, int fontsize)
 {
-  if (DrawThisCell(parser, point))
+  Configuration *configuration = Configuration::Get();
+  MathCell::Draw(point, fontsize);
+  if (DrawThisCell(point)&&(InUpdateRegion()))
   {
-    double scale = parser.GetScale();
-    wxDC& dc = parser.GetDC();
+    double scale = configuration->GetScale();
+    wxDC& dc = configuration->GetDC();
     wxPoint in(point);
 
-    if (parser.CheckTeXFonts())
+    if (configuration->CheckTeXFonts())
     {
       in.x = point.x + m_signWidth;
-      SetForeground(parser);
+      SetForeground();
       int fontsize1 = (int) ((m_parenFontSize * scale + 0.5));
-      dc.SetFont(wxFont(fontsize1, wxFONTFAMILY_MODERN,
-			wxFONTSTYLE_NORMAL, wxFONTWEIGHT_NORMAL, false,
-                        m_bigParenType < 1 ?
-			parser.GetTeXCMRI() :
-			parser.GetTeXCMEX()));
+      wxFont font(fontsize1, wxFONTFAMILY_MODERN,
+                  wxFONTSTYLE_NORMAL, wxFONTWEIGHT_NORMAL, false,
+                  m_bigParenType < 1 ?
+                  configuration->GetTeXCMRI() :
+                  configuration->GetTeXCMEX());
+      font.SetPointSize(fontsize1);
+      dc.SetFont(font);
       if (m_bigParenType < 2)
       {
         dc.DrawText(m_bigParenType == 0 ? wxT("(") :
@@ -270,9 +321,10 @@ void ParenCell::Draw(CellParser& parser, wxPoint point, int fontsize)
                     point.y - m_center + SCALE_PX(MC_TEXT_PADDING, scale) -
                     (m_bigParenType > 0 ? m_signTop : 0));
       }
-      else {
-        int top = point.y - m_center - m_signTop;
-        int bottom = point.y + m_height - m_center - m_signTop - m_signSize / 2;
+      else
+      {
+        int top =    point.y - m_center - m_signTop;
+        int bottom = top + m_height - m_signSize / 2;
         dc.DrawText(wxT(PAREN_OPEN_TOP),
                     point.x,
                     top);
@@ -308,17 +360,20 @@ void ParenCell::Draw(CellParser& parser, wxPoint point, int fontsize)
     }
     else
     {
-#if defined __WXMSW__
+      // No TeX fonts
+#ifdef __WXMSW__
       in.x += m_charWidth;
       int fontsize1 = (int) ((PAREN_FONT_SIZE * scale + 0.5));
-      SetForeground(parser);
+      SetForeground();
       if (m_height < (3*m_charHeight)/2)
       {
         fontsize1 = (int) ((fontsize * scale + 0.5));
-        dc.SetFont(wxFont(fontsize1, wxFONTFAMILY_MODERN,
-                          wxFONTSTYLE_NORMAL, wxFONTWEIGHT_NORMAL,
-                          false,
-                          parser.GetFontName()));
+        wxFont font(fontsize1, wxFONTFAMILY_MODERN,
+                    wxFONTSTYLE_NORMAL, wxFONTWEIGHT_NORMAL,
+                    false,
+                    configuration->GetFontName());
+        font.SetPointSize(fontsize1);
+        dc.SetFont(font);
         dc.DrawText(wxT("("),
                     point.x + m_charWidth - m_charWidth1,
                     point.y - m_charHeight1 / 2);
@@ -328,11 +383,13 @@ void ParenCell::Draw(CellParser& parser, wxPoint point, int fontsize)
       }
       else
       {
-        dc.SetFont(wxFont(fontsize1, wxFONTFAMILY_MODERN,
-                          wxFONTSTYLE_NORMAL, wxFONTWEIGHT_NORMAL,
-                          false,
-                          parser.GetSymbolFontName(),
-			  wxFONTENCODING_CP1250));
+        wxFont font(fontsize1, wxFONTFAMILY_MODERN,
+                    wxFONTSTYLE_NORMAL, wxFONTWEIGHT_NORMAL,
+                    false,
+                    configuration->GetSymbolFontName(),
+                    wxFONTENCODING_CP1250);
+        font.SetPointSize(fontsize1);
+        dc.SetFont(font);
         dc.DrawText(PAREN_LEFT_TOP,
                     point.x,
                     point.y - m_center);
@@ -374,40 +431,39 @@ void ParenCell::Draw(CellParser& parser, wxPoint point, int fontsize)
         }
       }
 #else
-      in.x = point.x + SCALE_PX(6, scale);
-      SetPen(parser);
+      in.x = point.x + SCALE_PX(6, scale) + GetDefaultLineWidth();
+      SetPen();
       // left
-      dc.DrawLine(point.x + SCALE_PX(5, scale),
+      dc.DrawLine(point.x + SCALE_PX(5, scale) + GetDefaultLineWidth() / 2,
                   point.y - m_innerCell->GetMaxCenter() + SCALE_PX(1, scale),
-                  point.x + SCALE_PX(2, scale),
+                  point.x + SCALE_PX(2, scale) + GetDefaultLineWidth() / 2,
                   point.y - m_innerCell->GetMaxCenter() + SCALE_PX(7, scale));
-      dc.DrawLine(point.x + SCALE_PX(2, scale),
+      dc.DrawLine(point.x + SCALE_PX(2, scale) + GetDefaultLineWidth() / 2,
                   point.y - m_innerCell->GetMaxCenter() + SCALE_PX(7, scale),
-                  point.x + SCALE_PX(2, scale),
+                  point.x + SCALE_PX(2, scale) + GetDefaultLineWidth() / 2,
                   point.y + m_innerCell->GetMaxDrop() - SCALE_PX(7, scale));
-      dc.DrawLine(point.x + SCALE_PX(2, scale),
+      dc.DrawLine(point.x + SCALE_PX(2, scale) + GetDefaultLineWidth() / 2,
                   point.y + m_innerCell->GetMaxDrop() - SCALE_PX(7, scale),
-                  point.x + SCALE_PX(5, scale),
+                  point.x + SCALE_PX(5, scale) + GetDefaultLineWidth() / 2,
                   point.y + m_innerCell->GetMaxDrop() - SCALE_PX(1, scale));
       // right
-      dc.DrawLine(point.x + m_width - SCALE_PX(5, scale) - 1,
+      dc.DrawLine(point.x + m_width - SCALE_PX(5, scale) - 1 - GetDefaultLineWidth() / 2,
                   point.y - m_innerCell->GetMaxCenter() + SCALE_PX(1, scale),
-                  point.x + m_width - SCALE_PX(2, scale) - 1,
+                  point.x + m_width - SCALE_PX(2, scale) - 1 - GetDefaultLineWidth() / 2,
                   point.y - m_innerCell->GetMaxCenter() + SCALE_PX(7, scale));
-      dc.DrawLine(point.x + m_width - SCALE_PX(2, scale) - 1,
+      dc.DrawLine(point.x + m_width - SCALE_PX(2, scale) - 1 - GetDefaultLineWidth() / 2,
                   point.y - m_innerCell->GetMaxCenter() + SCALE_PX(7, scale),
-                  point.x + m_width - SCALE_PX(2, scale) - 1,
+                  point.x + m_width - SCALE_PX(2, scale) - 1 - GetDefaultLineWidth() / 2,
                   point.y + m_innerCell->GetMaxDrop() - SCALE_PX(7, scale));
-      dc.DrawLine(point.x + m_width - SCALE_PX(2, scale) - 1,
+      dc.DrawLine(point.x + m_width - SCALE_PX(2, scale) - 1 - GetDefaultLineWidth() / 2,
                   point.y + m_innerCell->GetMaxDrop() - SCALE_PX(7, scale),
-                  point.x + m_width - SCALE_PX(5, scale) - 1,
+                  point.x + m_width - SCALE_PX(5, scale) - 1 - GetDefaultLineWidth() / 2,
                   point.y + m_innerCell->GetMaxDrop() - SCALE_PX(1, scale));
-      UnsetPen(parser);
+      UnsetPen();
 #endif
     }
-    m_innerCell->DrawList(parser, in, fontsize);
+    m_innerCell->DrawList(in, fontsize);
   }
-  MathCell::Draw(parser, point, fontsize);
 }
 
 wxString ParenCell::ToString()
@@ -453,6 +509,13 @@ wxString ParenCell::ToTeX()
   return s;
 }
 
+wxString ParenCell::ToOMML()
+{
+  return wxT("<m:d><m:dPr m:begChr=\"") + XMLescape(m_open->ToString()) + wxT("\" m:endChr=\"") +
+    XMLescape(m_close->ToString()) + wxT("\" m:grow=\"1\"></m:dPr><m:e>") +
+    m_innerCell->ListToOMML()+wxT("</m:e></m:d>");
+}
+
 wxString ParenCell::ToMathML()
 {
   if(!m_print) return m_innerCell->ListToMathML();
@@ -460,9 +523,9 @@ wxString ParenCell::ToMathML()
   wxString open   = m_open->ToString();
   wxString close  = m_close->ToString();
   return(
-    wxT("<mfenced separators=\"\" open=\'") + open + wxT("\' close=\'") + close + wxT("\'>") +
+    wxT("<mrow><mo>") + XMLescape(open) + wxT("</mo>") +
     m_innerCell->ListToMathML() +
-    wxT("</mfenced>\n")
+    wxT("<mo>") + XMLescape(close) +  wxT("</mo></mrow>\n")
     );
 }
 
@@ -471,7 +534,7 @@ wxString ParenCell::ToXML()
 //  if( m_isBroken )
 //    return wxEmptyString;
   wxString s = m_innerCell->ListToXML();
-  return ( ( m_print )? _T("<p>") + s + _T("</p>") : s );
+  return ( ( m_print )? _T("<r><p>") + s + _T("</p></r>") : s );
 }
 
 void ParenCell::SelectInner(wxRect& rect, MathCell **first, MathCell **last)
@@ -496,12 +559,20 @@ bool ParenCell::BreakUp()
     m_isBroken = true;
     m_open->m_nextToDraw = m_innerCell;
     m_innerCell->m_previousToDraw = m_open;
-    m_last1->m_nextToDraw = m_close;
-    m_close->m_previousToDraw = m_last1;
+    wxASSERT_MSG(m_last1 != NULL,_("Bug: No last cell inside a parenthesis!"));
+    if(m_last1 != NULL)
+    {
+      m_last1->m_nextToDraw = m_close;
+      m_close->m_previousToDraw = m_last1;
+    }
     m_close->m_nextToDraw = m_nextToDraw;
     if (m_nextToDraw != NULL)
       m_nextToDraw->m_previousToDraw = m_close;
     m_nextToDraw = m_open;
+
+    m_height = MAX(m_innerCell->GetMaxHeight(),m_open->GetMaxHeight());
+    m_center = MAX(m_innerCell->GetMaxCenter(),m_open->GetMaxCenter());
+
     return true;
   }
   return false;
