@@ -34,12 +34,13 @@ public:
   void SetBase(MathCell *base);
   void SetIndex(MathCell *index);
   void SetExponent(MathCell *expt);
-  void RecalculateSize(CellParser& parser, int fontsize);
-  void RecalculateWidths(CellParser& parser, int fontsize);
-  void Draw(CellParser& parser, wxPoint point, int fontsize);
+  void RecalculateHeight(int fontsize);
+  void RecalculateWidths(int fontsize);
+  void Draw(wxPoint point, int fontsize);
   wxString ToString();
   wxString ToTeX();
   wxString ToXML();
+  wxString ToOMML();
   wxString ToMathML();
   void SelectInner(wxRect& rect, MathCell** first, MathCell** last);
   void SetParent(MathCell *parent);
