@@ -19,6 +19,14 @@
 //  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 //
 
+/*! \file
+  This file declares the class SumCell
+
+  SumCell is the MathCell type that represents maxima's <code>sum()</code>, 
+  <code>lsum</code> and <code>product()</code> 
+  commands.
+*/
+
 #ifndef SUMCELL_H
 #define SUMCELL_H
 
@@ -36,9 +44,9 @@ public:
   ~SumCell();
   void Destroy();
   MathCell* Copy();
-  void RecalculateSize(CellParser& parser, int fontsize);
-  void RecalculateWidths(CellParser& parser, int fontsize);
-  void Draw(CellParser& parser, wxPoint point, int fontsize);
+  void RecalculateHeight(int fontsize);
+  void RecalculateWidths(int fontsize);
+  void Draw(wxPoint point, int fontsize);
   void SetBase(MathCell* base);
   void SetUnder(MathCell* under);
   void SetOver(MathCell* name);
@@ -50,6 +58,7 @@ public:
   wxString ToTeX();
   wxString ToMathML();
   wxString ToXML();
+  wxString ToOMML();
   void SelectInner(wxRect& rect, MathCell** first, MathCell** last);
   void SetParent(MathCell *parent);
 protected:
