@@ -1,6 +1,6 @@
 // -*- mode: c++; c-file-style: "linux"; c-basic-offset: 2; indent-tabs-mode: nil -*-
 //
-//            (C) 2015 Gunter Königsmann <wxMaxima@physikbuch.de>
+//            (C) 2015-2016 Gunter Königsmann <wxMaxima@physikbuch.de>
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
@@ -18,6 +18,13 @@
 //  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 //
 
+/*!\file
+  This file defines the contents of the class XmlInspector 
+
+  XmlInspector is a sidebar allows to view the communication between maxima 
+  and wxMaxima.
+ */
+
 #include "XmlInspector.h"
 
 #include <wx/sizer.h>
@@ -49,7 +56,7 @@ wxString XmlInspector::IndentString(int level)
 
 void XmlInspector::Add(wxString text)
 {
-  int index=0;
+  size_t index=0;
   text.Replace(wxT("$FUNCTION:"),wxT("\n$FUNCTION:"));      
   while(index < text.Length())
   {
@@ -57,7 +64,11 @@ void XmlInspector::Add(wxString text)
 
     // Assume that all tags add indentation
     if(ch == wxT('>'))
+    {
       m_indentLevel ++;
+      if(text.Left(index+1).EndsWith(wxT("</wxxml-symbols>")))
+        m_indentLevel = 0;
+    }
       
     // A closing tag needs to remove the indentation of the opening tag 
     // plus the indentation of the closing tag
