@@ -19,6 +19,14 @@
 //  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 //
 
+/*!\file
+  This file declares the class wxMaxima that contains most of the program's logic. 
+
+  The worksheet is defined in the class MathCtrl instead and 
+  everything surrounding it in wxMaximaFrame.
+ */
+
+
 #ifndef WXMAXIMA_H
 #define WXMAXIMA_H
 
@@ -96,8 +104,6 @@ public:
     it between two expirys. 
    */
   wxTimer m_autoSaveTimer;
-  //! Has m_autoSaveTimer expired since the last save?
-  bool m_autoSaveIntervalExpired;
   //! Is triggered when a timer this class is responsible for requires
   void OnTimerEvent(wxTimerEvent& event);
   //! A timer that polls for output from the maxima process.
@@ -132,6 +138,10 @@ public:
   bool DocumentSaved() { return m_fileSaved; }
   void LoadImage(wxString file) { m_console->OpenHCaret(file, GC_TYPE_IMAGE); }
 private:
+  //! This string allows us to detect when the string we search for has changed.
+  wxString m_oldFindString;
+  //! This string allows us to detect when the string we search for has changed.
+  int m_oldFindFlags;
   //! On opening a new file we only need a new maxima process if the old one ever evaluated cells.
   bool m_hasEvaluatedCells;
   //! Searches for maxima's output prompts
@@ -140,7 +150,7 @@ private:
   int m_outputCellsFromCurrentCommand;
   //! The maximum number of lines per command we will display 
   int m_maxOutputCellsPerCommand;
-  //! The number of consecutive unsucessfull attempts to connect to the maxima server
+  //! The number of consecutive unsuccessful attempts to connect to the maxima server
   int m_unsuccessfullConnectionAttempts;
   //! The current working directory maxima's file I/O is relative to.
   wxString m_CWD;
@@ -170,30 +180,45 @@ protected:
   void ShowHTMLHelp(wxString helpfile,wxString otherhelpfile,wxString keyword);
   void CheckForUpdates(bool reportUpToDate = false);
   void OnRecentDocument(wxCommandEvent& event);
-  void OnIdle(wxIdleEvent& event);
-  void MenuCommand(wxString cmd);                  //
-  void FileMenu(wxCommandEvent& event);            //
-  void MaximaMenu(wxCommandEvent& event);          //
-  void AlgebraMenu(wxCommandEvent& event);         //
-  void EquationsMenu(wxCommandEvent& event);       //
-  void CalculusMenu(wxCommandEvent& event);        //!< event handling for menus
-  void SimplifyMenu(wxCommandEvent& event);        //
-  void PlotMenu(wxCommandEvent& event);            //
-  void NumericalMenu(wxCommandEvent& event);       //
-  void HelpMenu(wxCommandEvent& event);            //
-  void EditMenu(wxCommandEvent& event);            //
-  void Interrupt(wxCommandEvent& event);           //
-  /* Make the menu item, toolbars and panes visible that should be visible right now.
+  /*! The idle task that refreshes the gui (worksheet, menus, title line,...)
 
-     \todo Didn't update the stats pane. I assume this was a bug.
-   */
-  void UpdateMenus(wxUpdateUIEvent& event);
-  void UpdateToolBar(wxUpdateUIEvent& event);      //
-  void UpdateSlider(wxUpdateUIEvent& event);       //
+    In GUIs normally all events (mouse events, key presses,...) are 
+    put into a queue and then are executed one by one. This makes
+    sure that they will be processed in order - but makes a gui a
+    more or less single-processor-task.
+
+    If the queue is empty (which means that the computer has caught 
+    up with the incoming events) this function is called once.
+
+    Moving the screen update to this function makes sure that we
+    don't do many subsequent updates slowing down the computer any 
+    further if there are still a handful of key presses to process
+    lowering the average response times of the program.
+
+    The worksheet is refreshed by a timer task in case the computer 
+    is too busy to execute the idle task at all.
+  */
+  void OnIdle(wxIdleEvent& event);
+  void MenuCommand(wxString cmd);                  //!< Inserts command cmd into the worksheet
+  void FileMenu(wxCommandEvent& event);            //!< Processes "file menu" clicks
+  void MaximaMenu(wxCommandEvent& event);          //!< Processes "maxima menu" clicks
+  void AlgebraMenu(wxCommandEvent& event);         //!< Processes "algebra menu" clicks
+  void EquationsMenu(wxCommandEvent& event);       //!< Processes "equations menu" clicks
+  void CalculusMenu(wxCommandEvent& event);        //!< event handling for menus
+  void SimplifyMenu(wxCommandEvent& event);        //!< Processes "Simplify menu" clicks
+  void PlotMenu(wxCommandEvent& event);            //!< Processes "Plot menu" cloicks
+  void NumericalMenu(wxCommandEvent& event);       //!< Processes "Numerical menu" clicks
+  void HelpMenu(wxCommandEvent& event);            //!< Processes "Help menu" clicks
+  void EditMenu(wxCommandEvent& event);            //!< Processes "Edit menu" clicks
+  void Interrupt(wxCommandEvent& event);           //!< Interrupt button and hotkey presses
+  //! Make the menu item, toolbars and panes visible that should be visible right now.
+  void UpdateMenus(wxUpdateUIEvent& event);	   //!< Enables and disables the Right menu buttons
+  void UpdateToolBar(wxUpdateUIEvent& event);      //!< Enables and disables the Right toolbar buttons
+  void UpdateSlider(wxUpdateUIEvent& event);       //!< Updates the slider to show the right frame
   /*! Toggle the visibility of a pane
     \param event The event that triggered calling this function.
    */
-  void ShowPane(wxCommandEvent& event);            
+  void ShowPane(wxCommandEvent& event);            //<! Makes a sidebar visible
   void OnProcessEvent(wxProcessEvent& event);      //
   void PopupMenu(wxCommandEvent& event);           //
   void StatsMenu(wxCommandEvent& event);           //
@@ -233,7 +258,7 @@ protected:
   //! Issued on double click on a history item
   void HistoryDClick(wxCommandEvent& event);
   //! Issued on double click on a table of contents item
-  void StructureDClick(wxCommandEvent& event);
+  void TableOfContentsDClick(wxCommandEvent& event);
   void OnInspectorEvent(wxCommandEvent& ev);
   void DumpProcessOutput();
 
@@ -250,7 +275,7 @@ protected:
 
     Normally a restart is only needed if
       - maxima isn't currently in the process of starting up or
-      - maxima is runnning and has never evaluated any program 
+      - maxima is running and has never evaluated any program 
         so a restart won't change anything
     \param force true means to restart maxima unconditionally.
    */
@@ -272,7 +297,15 @@ protected:
                   After leaving this function data is empty again.
    */
   void ReadFirstPrompt(wxString &data);
-  /* Reads text that isn't enclosed between xml tags.
+  /*! Determine where the text for ReadMiscText ends
+
+    Every error message or other line maxima outputs should end in a newline character. 
+    But sometimes it doesn't and a <code><mth></code> tag comes first \f$ =>\f$ This 
+    function determines where the miscellaneous text ends.
+   */
+  int GetMiscTextEnd(const wxString &data);
+
+  /*! Reads text that isn't enclosed between xml tags.
 
      Some commands provide status messages before the math output or the command has finished.
      This function makes wxMaxima output them directly as they arrive.
@@ -280,16 +313,24 @@ protected:
      After processing the lines not enclosed in xml tags they are removed from data.
    */
   void ReadMiscText(wxString &data);
-  /* Reads the input prompt from Maxima.
+  /*! Reads the input prompt from Maxima.
 
      After processing the input prompt it is removed from data.
    */
   void ReadPrompt(wxString &data);
-  /* Reads the math cell's contents from Maxima.
+  /*! Reads the output of wxstatusbar() commands
+
+    wxstatusbar allows the user to give and update visual feedback from long-running 
+    commands and makes sure this feedback is deleted once the command is finished.
+   */
+  void ReadStatusBar(wxString &data);
+  /*! Reads the math cell's contents from Maxima.
      
      Math cells are enclosed between the tags \<mth\> and \</mth\>. 
      This function removes the from data after appending them
      to the console.
+
+     After processing the status bar marker is removed from data.
    */
   void ReadMath(wxString &data);
   /*! read lisp errors
@@ -332,7 +373,14 @@ protected:
   void ResetTitle(bool saved,bool force = false);
   void FirstOutput(wxString s);
 
-  // Opens a wxm file
+  /*! Opens a content.xml file that has been extracted from a broken .wxmx file
+   */
+  bool OpenXML(wxString file, MathCtrl *document, bool clearDocument = true);
+
+  //! Complains if the version string from the XML file indicates too low a maxima version
+  bool CheckWXMXVersion(wxString docversion);
+
+  //! Opens a wxm file
   bool OpenWXMFile(wxString file, MathCtrl *document, bool clearDocument = true);
   //! Opens a wxmx file
   bool OpenWXMXFile(wxString file, MathCtrl *document, bool clearDocument = true);
@@ -353,16 +401,17 @@ protected:
     }
   wxSocketBase *m_client;
   wxSocketServer *m_server;
+  //! Is the network connection to maxima working?
   bool m_isConnected;
+  //! Is maxima running?
   bool m_isRunning;
-  bool m_first;
   //! The process id of maxima. Is determined by ReadFirstPrompt.
   long m_pid;
   wxProcess *m_process;
-  // The stdout of the maxima process
-  wxInputStream *m_input;
-  // The stderr of the maxima process
-  wxInputStream *m_error;
+  //! The stdout of the maxima process
+  wxInputStream *m_maximaStdout;
+  //! The stderr of the maxima process
+  wxInputStream *m_maximaStderr;
   int m_port;
   wxString m_currentOutput;
   //! The marker for the start of a input prompt
